@@ -45,7 +45,9 @@ void emit_add32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
             BEQZ(s5, 8);
             ORI(xFlags, xFlags, 1 << F_CF);
         } else {
-            ADD(s5, s1, s2);
+            AND(s3, s1, xMASK);
+            AND(s4, s2, xMASK);
+            ADD(s5, s3, s4);
             SRLI(s5, s5, 0x20);
             BEQZ(s5, 8);
             ORI(xFlags, xFlags, 1 << F_CF);
@@ -137,7 +139,9 @@ void emit_add32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int64_t c, i
             BEQZ(s5, 8);
             ORI(xFlags, xFlags, 1 << F_CF);
         } else {
-            ADD(s5, s1, s2);
+            AND(s3, s1, xMASK);
+            AND(s4, s2, xMASK);
+            ADD(s5, s3, s4);
             SRLI(s5, s5, 0x20);
             BEQZ(s5, 8);
             ORI(xFlags, xFlags, 1 << F_CF);
@@ -282,10 +286,10 @@ void emit_add8(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         if(rv64_zbb) {
             ANDN(s3, s3, s1);   // s3 = ~res & (op1 | op2)
         } else {
-            NOT(s4, s1);   // s4 = ~res
-            AND(s3, s4, s3);   // s3 = ~res & (op1 | op2)
+            NOT(s2, s1);   // s4 = ~res
+            AND(s3, s2, s3);   // s3 = ~res & (op1 | op2)
         }
-        OR(s3, s3, s2);   // cc = (~res & (op1 | op2)) | (op1 & op2)
+        OR(s3, s3, s4);   // cc = (~res & (op1 | op2)) | (op1 & op2)
         IFX(X_AF) {
             ANDI(s4, s3, 0x08); // AF: cc & 0x08
             BEQZ(s4, 8);
@@ -617,7 +621,8 @@ void emit_inc8(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         }
     }
     IFX(X_SF) {
-        BGE(s1, xZR, 8);
+        ANDI(s2, s1, 0x80);
+        BEQZ(s2, 8);
         ORI(xFlags, xFlags, 1 << F_SF);
     }
     ANDI(s1, s1, 0xff);
@@ -672,7 +677,8 @@ void emit_dec8(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         }
     }
     IFX(X_SF) {
-        BGE(s1, xZR, 8);
+        ANDI(s2, s1, 0x80);
+        BEQZ(s2, 8);
         ORI(xFlags, xFlags, 1 << F_SF);
     }
     ANDI(s1, s1, 0xff);
@@ -1145,7 +1151,7 @@ void emit_neg32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         BEQZ(s1, 8);
         ORI(xFlags, xFlags, 1 << F_CF);
     }
-    
+
     IFX(X_AF | X_OF) {
         OR(s3, s1, s3); // s3 = res | op1
         IFX(X_AF) {
@@ -1162,7 +1168,7 @@ void emit_neg32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
             ANDI(s2, s2, 1);
             BEQZ(s2, 8);
             ORI(xFlags, xFlags, 1 << F_OF2);
-        }    
+        }
     }
     IFX(X_SF) {
         BGE(s1, xZR, 8);
@@ -1204,7 +1210,7 @@ void emit_neg16(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3)
         BEQZ(s1, 8);
         ORI(xFlags, xFlags, 1 << F_CF);
     }
-    
+
     IFX(X_AF | X_OF) {
         OR(s3, s1, s3); // s3 = res | op1
         IFX(X_AF) {
@@ -1221,7 +1227,7 @@ void emit_neg16(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3)
             ANDI(s2, s2, 1);
             BEQZ(s2, 8);
             ORI(xFlags, xFlags, 1 << F_OF2);
-        }    
+        }
     }
     IFX(X_SF) {
         SRLI(s3, s1, 15-F_SF);    // put sign bit in place
@@ -1261,7 +1267,7 @@ void emit_neg8(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3)
         BEQZ(s1, 8);
         ORI(xFlags, xFlags, 1 << F_CF);
     }
-    
+
     IFX(X_AF | X_OF) {
         OR(s3, s1, s3); // s3 = res | op1
         IFX(X_AF) {
@@ -1278,7 +1284,7 @@ void emit_neg8(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3)
             ANDI(s2, s2, 1);
             BEQZ(s2, 8);
             ORI(xFlags, xFlags, 1 << F_OF2);
-        }    
+        }
     }
     IFX(X_SF) {
         ANDI(s3, s1, 1 << F_SF);    // 1<<F_SF is sign bit, so just mask
@@ -1374,6 +1380,8 @@ void emit_adc32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         if (rex.w) {
             AND(s5, xMASK, s1);
             if(rv64_zba) ADDUW(s5, s2, s5); else {AND(s4, xMASK, s2); ADD(s5, s5, s4);} // lo
+            ANDI(s3, xFlags, 1);
+            ADD(s5, s5, s3);    // add carry
             SRLI(s3, s1, 0x20);
             SRLI(s4, s2, 0x20);
             ADD(s4, s4, s3);
@@ -1381,7 +1389,11 @@ void emit_adc32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
             ADD(s5, s5, s4); // hi
             SRAI(s6, s5, 0x20);
         } else {
-            ADD(s5, s1, s2);
+            AND(s3, s1, xMASK);
+            AND(s4, s2, xMASK);
+            ADD(s5, s3, s4);
+            ANDI(s3, xFlags, 1);
+            ADD(s5, s5, s3);    // add carry
             SRLI(s6, s5, 0x20);
         }
     }
@@ -1406,8 +1418,8 @@ void emit_adc32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         if(rv64_zbb) {
             ANDN(s3, s4, s1);   // s3 = ~res & (op1 | op2)
         } else {
-            NOT(s2, s1);     // s2 = ~res
-            AND(s3, s2, s4); // s3 = ~res & (op1 | op2)
+            NOT(s3, s1);     // s2 = ~res
+            AND(s3, s3, s4); // s3 = ~res & (op1 | op2)
         }
         OR(s3, s3, s5);  // cc = (~res & (op1 | op2)) | (op1 & op2)
         IFX(X_AF) {
