@@ -230,12 +230,12 @@ uintptr_t dynarec64_F0(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                                 // Unaligned
                                 ANDI(x5, wback, -(1 << (rex.w + 2)));
                                 MARK2; // Use MARK2 as a "MARKLOCK" since we're running out of marks.
-                                LDxw(x6, wback, 0);
-                                LRxw(x1, x5, 1, 1);
-                                SUBxw(x3, x6, xRAX);
+                                LDxw(x1, wback, 0);
+                                LRxw(x6, x5, 1, 1);
+                                SUBxw(x3, x1, xRAX);
                                 BNEZ_MARK(x3);
                                 // EAX == Ed
-                                SCxw(x4, x1, x5, 1, 1);
+                                SCxw(x4, x6, x5, 1, 1);
                                 BNEZ_MARK2(x4);
                                 SDxw(gd, wback, 0);
                                 MARK;
@@ -323,26 +323,22 @@ uintptr_t dynarec64_F0(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                                 AMOSWAP_W(xZR, xZR, x9, 1, 1);
                             } else {
                                 SMDMB();
+                                AND(x3, xRAX, xMASK);
+                                SLLI(x2, xRDX, 32);
+                                OR(x3, x3, x2);
+                                AND(x4, xRBX, xMASK);
+                                SLLI(x2, xRCX, 32);
+                                OR(x4, x4, x2);
                                 MARKLOCK;
                                 LR_D(x2, wback, 1, 1);
-                                AND(x3, x2, xMASK);
-                                AND(x4, xRAX, xMASK);
-                                SRLI(x5, x2, 32);
-                                AND(x6, xRDX, xMASK);
-                                BNE_MARK(x3, x4); // EAX != Ed[0]
-                                BNE_MARK(x5, x6); // EDX != Ed[1]
-                                SLLI(x2, xRCX, 32);
-                                AND(x3, xRBX, xMASK);
-                                OR(x2, x2, x3);
-                                SC_D(x3, x2, wback, 1, 1);
-                                BNEZ_MARKLOCK(x3);
+                                BNE_MARK(x2, x3); // EDX_EAX != Ed
+                                SC_D(x5, x4, wback, 1, 1);
+                                BNEZ_MARKLOCK(x5);
                                 ORI(xFlags, xFlags, 1<<F_ZF);
                                 B_MARK3_nocond;
                                 MARK;
-                                ADDI(xRAX, x3, 0);
-                                ADDI(xRDX, x5, 0);
-                                AND(xRAX, xRAX, xMASK);
-                                AND(xRDX, xRDX, xMASK);
+                                SLLI(xRDX, x2, 32);
+                                AND(xRAX, x2, xMASK);
                                 MARK3;
                                 SMDMB();
                             }
@@ -554,7 +550,7 @@ uintptr_t dynarec64_F0(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, (opcode==0x81)?4:1);
                         if(opcode==0x81) i64 = F32S; else i64 = F8S;
                         if (i64 < -2048 || i64 >= 2048)
-                            MOV64xw(x4, i64);
+                            MOV64xw(x9, i64);
                         ANDI(x1, wback, (1 << (rex.w + 2)) - 1);
                         BNEZ_MARK3(x1);
                         // Aligned
@@ -563,7 +559,7 @@ uintptr_t dynarec64_F0(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                         if (i64 >= -2048 && i64 < 2048)
                             ADDIxw(x4, x1, i64);
                         else
-                            ADDxw(x4, x1, x4);
+                            ADDxw(x4, x1, x9);
                         SCxw(x3, x4, wback, 1, 1);
                         BNEZ_MARKLOCK(x3);
                         B_MARK_nocond;
@@ -576,7 +572,7 @@ uintptr_t dynarec64_F0(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                         if (i64 >= -2048 && i64 < 2048)
                             ADDIxw(x4, x6, i64);
                         else
-                            ADDxw(x4, x6, x4);
+                            ADDxw(x4, x6, x9);
                         SCxw(x3, x1, x5, 1, 1);
                         BNEZ_MARK2(x3);
                         SDxw(x4, wback, 0);
@@ -600,13 +596,14 @@ uintptr_t dynarec64_F0(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                     } else {
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, (opcode==0x81)?4:1);
                         if(opcode==0x81) i64 = F32S; else i64 = F8S;
+                        if (i64 < -2048 || i64 >= 2048)
+                            MOV64xw(x9, i64);
                         MARKLOCK;
                         LRxw(x1, wback, 1, 1);
-                        if (i64>=-2048 && i64<2048) {
+                        if (i64 >= -2048 && i64 < 2048) {
                             ORI(x4, x1, i64);
                         } else {
-                            MOV64xw(x4, i64);
-                            OR(x4, x1, x4);
+                            OR(x4, x1, x9);
                         }
                         if (!rex.w) ZEROUP(x4);
                         SCxw(x3, x4, wback, 1, 1);
@@ -629,13 +626,14 @@ uintptr_t dynarec64_F0(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                     } else {
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, (opcode==0x81)?4:1);
                         if(opcode==0x81) i64 = F32S; else i64 = F8S;
+                        if (i64 < -2048 || i64 >= 2048)
+                            MOV64xw(x9, i64);
                         MARKLOCK;
                         LRxw(x1, wback, 1, 1);
-                        if (i64>=-2048 && i64<2048) {
+                        if (i64 >= -2048 && i64 < 2048) {
                             ANDI(x4, x1, i64);
                         } else {
-                            MOV64xw(x4, i64);
-                            AND(x4, x1, x4);
+                            AND(x4, x1, x9);
                         }
                         if (!rex.w) ZEROUP(x4);
                         SCxw(x3, x4, wback, 1, 1);
@@ -654,26 +652,81 @@ uintptr_t dynarec64_F0(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                     if(MODREG) {
                         if(opcode==0x81) i64 = F32S; else i64 = F8S;
                         ed = xRAX+(nextop&7)+(rex.b<<3);
-                        emit_sub32(dyn, ninst, rex, ed, i64, x3, x4, x5);
+                        emit_sub32c(dyn, ninst, rex, ed, i64, x3, x4, x5, x6);
                     } else {
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, (opcode==0x81)?4:1);
                         if(opcode==0x81) i64 = F32S; else i64 = F8S;
+                        if (i64 <= -2048 || i64 > 2048)
+                            MOV64xw(x9, i64);
+                        ANDI(x1, wback, (1 << (rex.w + 2)) - 1);
+                        BNEZ_MARK3(x1);
+                        // Aligned
                         MARKLOCK;
                         LRxw(x1, wback, 1, 1);
-                        if (i64>-2048 && i64<=2048) {
+                        if (i64 > -2048 && i64 <= 2048) {
                             ADDIxw(x4, x1, -i64);
                         } else {
-                            MOV64xw(x4, i64);
-                            SUBxw(x4, x1, x4);
+                            SUBxw(x4, x1, x9);
                         }
                         SCxw(x3, x4, wback, 1, 1);
                         BNEZ_MARKLOCK(x3);
-                        IFX(X_ALL|X_PEND)
+                        B_MARK_nocond;
+                        MARK3;
+                        // Unaligned
+                        ANDI(x5, wback, -(1 << (rex.w + 2)));
+                        MARK2; // Use MARK2 as a "MARKLOCK" since we're running out of marks.
+                        LDxw(x6, wback, 0);
+                        LRxw(x1, x5, 1, 1);
+                        if (i64 > -2048 && i64 <= 2048) {
+                            ADDIxw(x4, x6, -i64);
+                        } else {
+                            SUBxw(x4, x6, x9);
+                        }
+                        SCxw(x3, x1, x5, 1, 1);
+                        BNEZ_MARK2(x3);
+                        SDxw(x4, wback, 0);
+                        MARK;
+                        IFX (X_ALL | X_PEND)
                             emit_sub32c(dyn, ninst, rex, x1, i64, x3, x4, x5, x6);
+                        SMDMB();
                     }
                     break;
-                default:
-                    DEFAULT;
+                case 6: // XOR
+                    if (opcode == 0x81) {
+                        INST_NAME("LOCK XOR Ed, Id");
+                    } else {
+                        INST_NAME("LOCK XOR Ed, Ib");
+                    }
+                    SETFLAGS(X_ALL, SF_SET_PENDING);
+                    if (MODREG) {
+                        if (opcode == 0x81)
+                            i64 = F32S;
+                        else
+                            i64 = F8S;
+                        ed = xRAX + (nextop & 7) + (rex.b << 3);
+                        emit_xor32c(dyn, ninst, rex, ed, i64, x3, x4);
+                    } else {
+                        addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, (opcode == 0x81) ? 4 : 1);
+                        if (opcode == 0x81)
+                            i64 = F32S;
+                        else
+                            i64 = F8S;
+                        if (i64 < -2048 || i64 >= 2048)
+                            MOV64xw(x9, i64);
+                        MARKLOCK;
+                        LRxw(x1, wback, 1, 1);
+                        if (i64 >= -2048 && i64 < 2048) {
+                            XORI(x4, x1, i64);
+                        } else {
+                            XOR(x4, x1, x9);
+                        }
+                        SCxw(x3, x4, wback, 1, 1);
+                        BNEZ_MARKLOCK(x3);
+                        IFX(X_ALL | X_PEND)
+                            emit_xor32c(dyn, ninst, rex, x1, i64, x3, x4);
+                    }
+                    break;
+                default: DEFAULT;
             }
             SMDMB();
             break;
@@ -749,7 +802,7 @@ uintptr_t dynarec64_F0(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                         SCxw(x3, x4, wback, 1, 1);
                         BNEZ_MARKLOCK(x3);
                         IFX(X_ALL|X_PEND)
-                            emit_inc32(dyn, ninst, rex, x1, x3, x4, x5, x6);
+                            emit_dec32(dyn, ninst, rex, x1, x3, x4, x5, x6);
                     }
                     break;
                 default:
