@@ -20,7 +20,7 @@
 #include "emu/x87emu_private.h"
 #include "x64trace.h"
 #include "signals.h"
-#include "dynarec_rv64.h"
+#include "dynarec_native.h"
 #include "dynarec_rv64_private.h"
 #include "dynarec_rv64_functions.h"
 #include "custommem.h"
@@ -58,7 +58,7 @@ void fpu_free_reg(dynarec_rv64_t* dyn, int reg)
     int idx = EXTIDX(reg);
     // TODO: check upper limit?
     dyn->e.fpuused[idx] = 0;
-    if(dyn->e.extcache[idx].t!=EXT_CACHE_ST_F && dyn->e.extcache[idx].t!=EXT_CACHE_ST_D)
+    if (dyn->e.extcache[idx].t != EXT_CACHE_ST_F && dyn->e.extcache[idx].t != EXT_CACHE_ST_D && dyn->e.extcache[idx].t != EXT_CACHE_ST_I64)
         dyn->e.extcache[idx].v = 0;
 }
 // Get an MMX double reg
@@ -81,27 +81,41 @@ int fpu_get_reg_xmm(dynarec_rv64_t* dyn, int t, int xmm)
     return EXTREG(i);
 }
 // Reset fpu regs counter
+void fpu_reset_reg_extcache(extcache_t* e)
+{
+    e->fpu_reg = 0;
+    for (int i=0; i<24; ++i) {
+        e->fpuused[i]=0;
+        e->extcache[i].v = 0;
+    }
+}
 void fpu_reset_reg(dynarec_rv64_t* dyn)
 {
-    dyn->e.fpu_reg = 0;
-    for (int i=0; i<24; ++i) {
-        dyn->e.fpuused[i]=0;
-        dyn->e.extcache[i].v = 0;
+    fpu_reset_reg_extcache(&dyn->e);
+}
+
+int extcache_no_i64(dynarec_rv64_t* dyn, int ninst, int st, int a)
+{
+    if (a == EXT_CACHE_ST_I64) {
+        extcache_promote_double(dyn, ninst, st);
+        return EXT_CACHE_ST_D;
     }
+    return a;
 }
 
 int extcache_get_st(dynarec_rv64_t* dyn, int ninst, int a)
 {
     if (dyn->insts[ninst].e.swapped) {
-        if(dyn->insts[ninst].e.combined1 == a)
+        if (dyn->insts[ninst].e.combined1 == a)
             a = dyn->insts[ninst].e.combined2;
-        else if(dyn->insts[ninst].e.combined2 == a)
+        else if (dyn->insts[ninst].e.combined2 == a)
             a = dyn->insts[ninst].e.combined1;
     }
-    for(int i=0; i<24; ++i)
-        if((dyn->insts[ninst].e.extcache[i].t==EXT_CACHE_ST_F
-         || dyn->insts[ninst].e.extcache[i].t==EXT_CACHE_ST_D)
-         && dyn->insts[ninst].e.extcache[i].n==a)
+    for (int i = 0; i < 24; ++i)
+        if ((dyn->insts[ninst].e.extcache[i].t == EXT_CACHE_ST_F
+                || dyn->insts[ninst].e.extcache[i].t == EXT_CACHE_ST_D
+                || dyn->insts[ninst].e.extcache[i].t == EXT_CACHE_ST_I64)
+            && dyn->insts[ninst].e.extcache[i].n == a)
             return dyn->insts[ninst].e.extcache[i].t;
     // not in the cache yet, so will be fetched...
     return EXT_CACHE_ST_D;
@@ -110,12 +124,13 @@ int extcache_get_st(dynarec_rv64_t* dyn, int ninst, int a)
 int extcache_get_current_st(dynarec_rv64_t* dyn, int ninst, int a)
 {
     (void)ninst;
-    if(!dyn->insts)
+    if (!dyn->insts)
         return EXT_CACHE_ST_D;
-    for(int i=0; i<24; ++i)
-        if((dyn->e.extcache[i].t==EXT_CACHE_ST_F
-         || dyn->e.extcache[i].t==EXT_CACHE_ST_D)
-         && dyn->e.extcache[i].n==a)
+    for (int i = 0; i < 24; ++i)
+        if ((dyn->e.extcache[i].t == EXT_CACHE_ST_F
+                || dyn->e.extcache[i].t == EXT_CACHE_ST_D
+                || dyn->e.extcache[i].t == EXT_CACHE_ST_I64)
+            && dyn->e.extcache[i].n == a)
             return dyn->e.extcache[i].t;
     // not in the cache yet, so will be fetched...
     return EXT_CACHE_ST_D;
@@ -129,6 +144,16 @@ int extcache_get_st_f(dynarec_rv64_t* dyn, int ninst, int a)
             return i;
     return -1;
 }
+
+int extcache_get_st_f_i64(dynarec_rv64_t* dyn, int ninst, int a)
+{
+    for (int i = 0; i < 24; ++i)
+        if ((dyn->insts[ninst].e.extcache[i].t == EXT_CACHE_ST_I64 || dyn->insts[ninst].e.extcache[i].t == EXT_CACHE_ST_F)
+            && dyn->insts[ninst].e.extcache[i].n == a)
+            return i;
+    return -1;
+}
+
 int extcache_get_st_f_noback(dynarec_rv64_t* dyn, int ninst, int a)
 {
     for(int i=0; i<24; ++i)
@@ -137,6 +162,16 @@ int extcache_get_st_f_noback(dynarec_rv64_t* dyn, int ninst, int a)
             return i;
     return -1;
 }
+
+int extcache_get_st_f_i64_noback(dynarec_rv64_t* dyn, int ninst, int a)
+{
+    for (int i = 0; i < 24; ++i)
+        if ((dyn->insts[ninst].e.extcache[i].t == EXT_CACHE_ST_I64 || dyn->insts[ninst].e.extcache[i].t == EXT_CACHE_ST_F)
+            && dyn->insts[ninst].e.extcache[i].n == a)
+            return i;
+    return -1;
+}
+
 int extcache_get_current_st_f(dynarec_rv64_t* dyn, int a)
 {
     for(int i=0; i<24; ++i)
@@ -146,6 +181,15 @@ int extcache_get_current_st_f(dynarec_rv64_t* dyn, int a)
     return -1;
 }
 
+int extcache_get_current_st_f_i64(dynarec_rv64_t* dyn, int a)
+{
+    for (int i = 0; i < 24; ++i)
+        if ((dyn->e.extcache[i].t == EXT_CACHE_ST_I64 || dyn->e.extcache[i].t == EXT_CACHE_ST_F)
+            && dyn->e.extcache[i].n == a)
+            return i;
+    return -1;
+}
+
 static void extcache_promote_double_forward(dynarec_rv64_t* dyn, int ninst, int maxinst, int a);
 static void extcache_promote_double_internal(dynarec_rv64_t* dyn, int ninst, int maxinst, int a);
 static void extcache_promote_double_combined(dynarec_rv64_t* dyn, int ninst, int maxinst, int a)
@@ -155,7 +199,7 @@ static void extcache_promote_double_combined(dynarec_rv64_t* dyn, int ninst, int
             a = dyn->insts[ninst].e.combined2;
         } else
             a = dyn->insts[ninst].e.combined1;
-        int i = extcache_get_st_f_noback(dyn, ninst, a);
+        int i = extcache_get_st_f_i64_noback(dyn, ninst, a);
         //if(box64_dynarec_dump) dynarec_log(LOG_NONE, "extcache_promote_double_combined, ninst=%d combined%c %d i=%d (stack:%d/%d)\n", ninst, (a == dyn->insts[ninst].e.combined2)?'2':'1', a ,i, dyn->insts[ninst].e.stack_push, -dyn->insts[ninst].e.stack_pop);
         if(i>=0) {
             dyn->insts[ninst].e.extcache[i].t = EXT_CACHE_ST_D;
@@ -174,7 +218,7 @@ static void extcache_promote_double_internal(dynarec_rv64_t* dyn, int ninst, int
         return;
     while(ninst>=0) {
         a+=dyn->insts[ninst].e.stack_pop;    // adjust Stack depth: add pop'd ST (going backward)
-        int i = extcache_get_st_f(dyn, ninst, a);
+        int i = extcache_get_st_f_i64(dyn, ninst, a);
         //if(box64_dynarec_dump) dynarec_log(LOG_NONE, "extcache_promote_double_internal, ninst=%d, a=%d st=%d:%d, i=%d\n", ninst, a, dyn->insts[ninst].e.stack, dyn->insts[ninst].e.stack_next, i);
         if(i<0) return;
         dyn->insts[ninst].e.extcache[i].t = EXT_CACHE_ST_D;
@@ -209,7 +253,7 @@ static void extcache_promote_double_forward(dynarec_rv64_t* dyn, int ninst, int
             else if (a==dyn->insts[ninst].e.combined2)
                 a = dyn->insts[ninst].e.combined1;
         }
-        int i = extcache_get_st_f_noback(dyn, ninst, a);
+        int i = extcache_get_st_f_i64_noback(dyn, ninst, a);
         //if(box64_dynarec_dump) dynarec_log(LOG_NONE, "extcache_promote_double_forward, ninst=%d, a=%d st=%d:%d(%d/%d), i=%d\n", ninst, a, dyn->insts[ninst].e.stack, dyn->insts[ninst].e.stack_next, dyn->insts[ninst].e.stack_push, -dyn->insts[ninst].e.stack_pop, i);
         if(i<0) return;
         dyn->insts[ninst].e.extcache[i].t = EXT_CACHE_ST_D;
@@ -230,7 +274,7 @@ static void extcache_promote_double_forward(dynarec_rv64_t* dyn, int ninst, int
 
 void extcache_promote_double(dynarec_rv64_t* dyn, int ninst, int a)
 {
-    int i = extcache_get_current_st_f(dyn, a);
+    int i = extcache_get_current_st_f_i64(dyn, a);
     //if(box64_dynarec_dump) dynarec_log(LOG_NONE, "extcache_promote_double, ninst=%d a=%d st=%d i=%d\n", ninst, a, dyn->e.stack, i);
     if(i<0) return;
     dyn->e.extcache[i].t = EXT_CACHE_ST_D;
@@ -266,19 +310,21 @@ int extcache_combine_st(dynarec_rv64_t* dyn, int ninst, int a, int b)
     return EXT_CACHE_ST_D;
 }
 
-static int isCacheEmpty(dynarec_native_t* dyn, int ninst) {
-    if(dyn->insts[ninst].e.stack_next) {
+static int isCacheEmpty(dynarec_native_t* dyn, int ninst)
+{
+    if (dyn->insts[ninst].e.stack_next) {
         return 0;
     }
-    for(int i=0; i<24; ++i)
-        if(dyn->insts[ninst].e.extcache[i].v) {       // there is something at ninst for i
-            if(!(
-            (dyn->insts[ninst].e.extcache[i].t==EXT_CACHE_ST_F || dyn->insts[ninst].e.extcache[i].t==EXT_CACHE_ST_D)
-            && dyn->insts[ninst].e.extcache[i].n<dyn->insts[ninst].e.stack_pop))
+    for (int i = 0; i < 24; ++i)
+        if (dyn->insts[ninst].e.extcache[i].v) { // there is something at ninst for i
+            if (!(
+                    (dyn->insts[ninst].e.extcache[i].t == EXT_CACHE_ST_F
+                        || dyn->insts[ninst].e.extcache[i].t == EXT_CACHE_ST_D
+                        || dyn->insts[ninst].e.extcache[i].t == EXT_CACHE_ST_I64)
+                    && dyn->insts[ninst].e.extcache[i].n < dyn->insts[ninst].e.stack_pop))
                 return 0;
         }
     return 1;
-
 }
 
 int fpuCacheNeedsTransform(dynarec_rv64_t* dyn, int ninst) {
@@ -295,9 +341,11 @@ int fpuCacheNeedsTransform(dynarec_rv64_t* dyn, int ninst) {
         }
         for(int i=0; i<24 && !ret; ++i)
             if(dyn->insts[ninst].e.extcache[i].v) {       // there is something at ninst for i
-                if(!(
-                (dyn->insts[ninst].e.extcache[i].t==EXT_CACHE_ST_F || dyn->insts[ninst].e.extcache[i].t==EXT_CACHE_ST_D)
-                && dyn->insts[ninst].e.extcache[i].n<dyn->insts[ninst].e.stack_pop))
+                if (!(
+                        (dyn->insts[ninst].e.extcache[i].t == EXT_CACHE_ST_F
+                            || dyn->insts[ninst].e.extcache[i].t == EXT_CACHE_ST_D
+                            || dyn->insts[ninst].e.extcache[i].t == EXT_CACHE_ST_I64)
+                        && dyn->insts[ninst].e.extcache[i].n < dyn->insts[ninst].e.stack_pop))
                     ret = 1;
             }
         return ret;
@@ -329,7 +377,9 @@ void extcacheUnwind(extcache_t* cache)
         int a = -1;
         int b = -1;
         for(int j=0; j<24 && ((a==-1) || (b==-1)); ++j)
-            if((cache->extcache[j].t == EXT_CACHE_ST_D || cache->extcache[j].t == EXT_CACHE_ST_F)) {
+            if ((cache->extcache[j].t == EXT_CACHE_ST_D
+                    || cache->extcache[j].t == EXT_CACHE_ST_F
+                    || cache->extcache[j].t == EXT_CACHE_ST_I64)) {
                 if(cache->extcache[j].n == cache->combined1)
                     a = j;
                 else if(cache->extcache[j].n == cache->combined2)
@@ -352,17 +402,22 @@ void extcacheUnwind(extcache_t* cache)
     }
     // add/change bad regs
     for(int i=0; i<16; ++i) {
-        if(cache->olds[i].changed) {
-            cache->extcache[i].t = cache->olds[i].single?EXT_CACHE_SS:EXT_CACHE_SD;
-        } else if(cache->olds[i].purged) {
-            cache->extcache[i].n = i;
-            cache->extcache[i].t = cache->olds[i].single?EXT_CACHE_SS:EXT_CACHE_SD;
+        if (cache->extcache[i].t == EXT_CACHE_SS || cache->extcache[i].t == EXT_CACHE_SD) {
+            if (cache->olds[i].changed) {
+                cache->extcache[i].t = cache->olds[i].single ? EXT_CACHE_SS : EXT_CACHE_SD;
+            } else if (cache->olds[i].purged) {
+                cache->extcache[i].n = i;
+                cache->extcache[i].t = cache->olds[i].single ? EXT_CACHE_SS : EXT_CACHE_SD;
+            }
         }
     }
+
     if(cache->stack_push) {
         // unpush
         for(int j=0; j<24; ++j) {
-            if((cache->extcache[j].t == EXT_CACHE_ST_D || cache->extcache[j].t == EXT_CACHE_ST_F)) {
+            if ((cache->extcache[j].t == EXT_CACHE_ST_D
+                    || cache->extcache[j].t == EXT_CACHE_ST_F
+                    || cache->extcache[j].t == EXT_CACHE_ST_I64)) {
                 if(cache->extcache[j].n<cache->stack_push)
                     cache->extcache[j].v = 0;
                 else
@@ -370,11 +425,23 @@ void extcacheUnwind(extcache_t* cache)
             }
         }
         cache->x87stack-=cache->stack_push;
+        cache->tags>>=(cache->stack_push*2);
         cache->stack-=cache->stack_push;
+        if(cache->pushed>=cache->stack_push)
+            cache->pushed-=cache->stack_push;
+        else
+            cache->pushed = 0;
         cache->stack_push = 0;
     }
     cache->x87stack+=cache->stack_pop;
     cache->stack_next = cache->stack;
+    if(cache->stack_pop) {
+        if(cache->poped>=cache->stack_pop)
+            cache->poped-=cache->stack_pop;
+        else
+            cache->poped = 0;
+        cache->tags<<=(cache->stack_pop*2);
+    }
     cache->stack_pop = 0;
     cache->barrier = 0;
     // And now, rebuild the x87cache info with extcache
@@ -401,16 +468,26 @@ void extcacheUnwind(extcache_t* cache)
                     break;
                 case EXT_CACHE_SS:
                     cache->ssecache[cache->extcache[i].n].reg = EXTREG(i);
+                    cache->ssecache[cache->extcache[i].n].vector = 0;
                     cache->ssecache[cache->extcache[i].n].single = 1;
                     ++cache->fpu_reg;
                     break;
                 case EXT_CACHE_SD:
                     cache->ssecache[cache->extcache[i].n].reg = EXTREG(i);
+                    cache->ssecache[cache->extcache[i].n].vector = 0;
                     cache->ssecache[cache->extcache[i].n].single = 0;
                     ++cache->fpu_reg;
                     break;
+                case EXT_CACHE_XMMR:
+                case EXT_CACHE_XMMW:
+                    cache->ssecache[cache->extcache[i].n].reg = i;
+                    cache->ssecache[cache->extcache[i].n].vector = 1;
+                    cache->ssecache[cache->extcache[i].n].write = (cache->extcache[i].t == EXT_CACHE_XMMW) ? 1 : 0;
+                    ++cache->fpu_reg;
+                    break;
                 case EXT_CACHE_ST_F:
                 case EXT_CACHE_ST_D:
+                case EXT_CACHE_ST_I64:
                     cache->x87cache[x87reg] = cache->extcache[i].n;
                     cache->x87reg[x87reg] = EXTREG(i);
                     ++x87reg;
@@ -427,24 +504,28 @@ void extcacheUnwind(extcache_t* cache)
     }
 }
 
-
-uint8_t extract_byte(uint32_t val, void* address){
+// will go badly if address is unaligned
+static uint8_t extract_byte(uint32_t val, void* address)
+{
     int idx = (((uintptr_t)address)&3)*8;
     return (val>>idx)&0xff;
 }
-uint32_t insert_byte(uint32_t val, uint8_t b, void* address){
+
+static uint32_t insert_byte(uint32_t val, uint8_t b, void* address)
+{
     int idx = (((uintptr_t)address)&3)*8;
     val&=~(0xff<<idx);
     val|=(((uint32_t)b)<<idx);
     return val;
 }
 
-// will go badly if address is unaligned
-uint16_t extract_half(uint32_t val, void* address){
+static uint16_t extract_half(uint32_t val, void* address)
+{
     int idx = (((uintptr_t)address)&3)*8;
     return (val>>idx)&0xffff;
 }
-uint32_t insert_half(uint32_t val, uint16_t h, void* address){
+static uint32_t insert_half(uint32_t val, uint16_t h, void* address)
+{
     int idx = (((uintptr_t)address)&3)*8;
     val&=~(0xffff<<idx);
     val|=(((uint32_t)h)<<idx);
@@ -482,10 +563,13 @@ const char* getCacheName(int t, int n)
     switch(t) {
         case EXT_CACHE_ST_D: sprintf(buff, "ST%d", n); break;
         case EXT_CACHE_ST_F: sprintf(buff, "st%d", n); break;
+        case EXT_CACHE_ST_I64: sprintf(buff, "STi%d", n); break;
         case EXT_CACHE_MM: sprintf(buff, "MM%d", n); break;
         case EXT_CACHE_SS: sprintf(buff, "SS%d", n); break;
         case EXT_CACHE_SD: sprintf(buff, "SD%d", n); break;
         case EXT_CACHE_SCR: sprintf(buff, "Scratch"); break;
+        case EXT_CACHE_XMMW: sprintf(buff, "XMM%d", n); break;
+        case EXT_CACHE_XMMR: sprintf(buff, "xmm%d", n); break;
         case EXT_CACHE_NONE: buff[0]='\0'; break;
     }
     return buff;
@@ -500,6 +584,12 @@ void inst_name_pass3(dynarec_native_t* dyn, int ninst, const char* name, rex_t r
         "fs2", "fs3", "fs4", "fs5", "fs6", "fs7", "fs8", "fs9", "fs10", "fs11",
         "ft8", "ft9", "ft10", "ft11"
     };
+    static const char* vnames[] = {
+        "v0", "v1", "v2", "v3", "v4", "v5", "v6", "v7",
+        "v8", "v9", "v10", "v11", "v12", "v13", "v14", "v15",
+        "v16", "v17", "v18", "v19", "v20", "v21", "v22", "v23",
+        "v24", "v25", "v26", "v27", "v8", "v9", "v30", "v31",
+    };
     if(box64_dynarec_dump) {
         printf_x64_instruction(rex.is32bits?my_context->dec32:my_context->dec, &dyn->insts[ninst].x64, name);
         dynarec_log(LOG_NONE, "%s%p: %d emitted opcodes, inst=%d, barrier=%d state=%d/%d(%d), %s=%X/%X, use=%X, need=%X/%X, sm=%d/%d",
@@ -533,9 +623,12 @@ void inst_name_pass3(dynarec_native_t* dyn, int ninst, const char* name, rex_t r
             switch(dyn->insts[ninst].e.extcache[ii].t) {
                 case EXT_CACHE_ST_D: dynarec_log(LOG_NONE, " %s:%s", fnames[EXTREG(ii)], getCacheName(dyn->insts[ninst].e.extcache[ii].t, dyn->insts[ninst].e.extcache[ii].n)); break;
                 case EXT_CACHE_ST_F: dynarec_log(LOG_NONE, " %s:%s", fnames[EXTREG(ii)], getCacheName(dyn->insts[ninst].e.extcache[ii].t, dyn->insts[ninst].e.extcache[ii].n)); break;
+                case EXT_CACHE_ST_I64: dynarec_log(LOG_NONE, " %s:%s", fnames[EXTREG(ii)], getCacheName(dyn->insts[ninst].e.extcache[ii].t, dyn->insts[ninst].e.extcache[ii].n)); break;
                 case EXT_CACHE_MM: dynarec_log(LOG_NONE, " %s:%s", fnames[EXTREG(ii)], getCacheName(dyn->insts[ninst].e.extcache[ii].t, dyn->insts[ninst].e.extcache[ii].n)); break;
                 case EXT_CACHE_SS: dynarec_log(LOG_NONE, " %s:%s", fnames[EXTREG(ii)], getCacheName(dyn->insts[ninst].e.extcache[ii].t, dyn->insts[ninst].e.extcache[ii].n)); break;
                 case EXT_CACHE_SD: dynarec_log(LOG_NONE, " %s:%s", fnames[EXTREG(ii)], getCacheName(dyn->insts[ninst].e.extcache[ii].t, dyn->insts[ninst].e.extcache[ii].n)); break;
+                case EXT_CACHE_XMMR: dynarec_log(LOG_NONE, " %s:%s", vnames[EXTREG(ii)], getCacheName(dyn->insts[ninst].e.extcache[ii].t, dyn->insts[ninst].e.extcache[ii].n)); break;
+                case EXT_CACHE_XMMW: dynarec_log(LOG_NONE, " %s:%s", vnames[EXTREG(ii)], getCacheName(dyn->insts[ninst].e.extcache[ii].t, dyn->insts[ninst].e.extcache[ii].n)); break;
                 case EXT_CACHE_SCR: dynarec_log(LOG_NONE, " %s:%s", fnames[EXTREG(ii)], getCacheName(dyn->insts[ninst].e.extcache[ii].t, dyn->insts[ninst].e.extcache[ii].n)); break;
                 case EXT_CACHE_NONE:
                 default:    break;
@@ -562,3 +655,60 @@ void print_newinst(dynarec_native_t* dyn, int ninst)
         (box64_dynarec_dump>1)?"\e[m":""
         );
 }
+
+// x87 stuffs
+static void x87_reset(extcache_t* e)
+{
+    for (int i=0; i<8; ++i)
+        e->x87cache[i] = -1;
+    e->x87stack = 0;
+    e->stack = 0;
+    e->stack_next = 0;
+    e->stack_pop = 0;
+    e->stack_push = 0;
+    e->combined1 = e->combined2 = 0;
+    e->swapped = 0;
+    e->barrier = 0;
+    e->pushed = 0;
+    e->poped = 0;
+
+    for(int i=0; i<24; ++i)
+        if (e->extcache[i].t == EXT_CACHE_ST_F
+            || e->extcache[i].t == EXT_CACHE_ST_D
+            || e->extcache[i].t == EXT_CACHE_ST_I64)
+            e->extcache[i].v = 0;
+}
+
+static void mmx_reset(extcache_t* e)
+{
+    e->mmxcount = 0;
+    for (int i=0; i<8; ++i)
+        e->mmxcache[i] = -1;
+}
+
+static void sse_reset(extcache_t* e)
+{
+    for (int i=0; i<16; ++i)
+        e->ssecache[i].v = -1;
+}
+
+void fpu_reset(dynarec_rv64_t* dyn)
+{
+    x87_reset(&dyn->e);
+    mmx_reset(&dyn->e);
+    sse_reset(&dyn->e);
+    fpu_reset_reg(dyn);
+}
+
+void fpu_reset_ninst(dynarec_rv64_t* dyn, int ninst)
+{
+    x87_reset(&dyn->insts[ninst].e);
+    mmx_reset(&dyn->insts[ninst].e);
+    sse_reset(&dyn->insts[ninst].e);
+    fpu_reset_reg_extcache(&dyn->insts[ninst].e);
+}
+
+int fpu_is_st_freed(dynarec_rv64_t* dyn, int ninst, int st)
+{
+    return (dyn->e.tags&(0b11<<(st*2)))?1:0;
+}
