@@ -3,8 +3,7 @@
 
 typedef struct library_s library_t;
 typedef struct needed_libs_s needed_libs_t;
-typedef struct kh_mapsymbols_s kh_mapsymbols_t;
-typedef struct kh_defaultversion_s kh_defaultversion_t;
+typedef struct cleanup_s cleanup_t;
 
 #include <elf.h>
 #include "elfloader.h"
@@ -47,9 +46,13 @@ typedef struct elfheader_s {
     int         szVerDef;
     int         e_type;
     uint32_t    flags;
+    uintptr_t   hash;
+    uintptr_t   gnu_hash;
 
     intptr_t    delta;  // should be 0
-    void*       image;
+    void*       image;  // base of the elf image
+    void*       raw;    // raw pointer (might be unaligned vs image and max_align of elf)
+    size_t      raw_size;
 
     uintptr_t   entrypoint;
     uintptr_t   initentry;
@@ -65,6 +68,9 @@ typedef struct elfheader_s {
     uintptr_t   rela;
     size_t      relasz;
     int         relaent;
+    uintptr_t   relr;
+    size_t      relrsz;
+    int         relrent;
     uintptr_t   jmprel;
     size_t      pltsz;
     int         pltent;
@@ -99,6 +105,7 @@ typedef struct elfheader_s {
     int         fini_done;
     int         refcnt;     // ref count for the elf
     int         malloc_hook_2;  // this elf hook malloc, hacking it
+    int         gnuunique;  // set if contains some STB_GNU_UNIQUE binding, preventing dlclose to unload the lib
 
     char*       memory;     // char* and not void* to allow math on memory pointer
     multiblock_t*  multiblocks;
@@ -110,11 +117,10 @@ typedef struct elfheader_s {
     FILE*       file;
     int         fileno;
 
-    kh_mapsymbols_t   *mapsymbols;
-    kh_mapsymbols_t   *weaksymbols;
-    kh_mapsymbols_t   *localsymbols;
-    kh_defaultversion_t *globaldefver;  // the global default version for symbols (the XXX@@vvvv of symbols)
-    kh_defaultversion_t *weakdefver;    // the weak default version for symbols (the XXX@@vvvv of symbols)
+    cleanup_t           *cleanups;          // atexit functions
+    int                 clean_sz;
+    int                 clean_cap;
+
 } elfheader_t;
 
 #define R_X86_64_NONE           0       /* No reloc */
@@ -162,6 +168,29 @@ typedef struct elfheader_s {
 #define R_X86_64_REX_GOTPCRELX  42      /* Load from 32 bit signed pc relative offset to GOT entry with REX prefix, relaxable.  */
 #define R_X86_64_NUM            43
 
+#ifndef STB_GNU_UNIQUE
+#define STB_GNU_UNIQUE  10
+#endif
+
+#ifndef ELF64_ST_VISIBILITY
+#define ELF64_ST_VISIBILITY(o)   ((o) & 0x03)
+#endif
+
 elfheader_t* ParseElfHeader(FILE* f, const char* name, int exec);
 
+const char* BindSym(int bind);
+
+Elf64_Half GetSymbolVersionFlag(elfheader_t* h, int index);
+
+uint32_t old_elf_hash(const char* name);
+Elf64_Sym* old_elf_lookup(elfheader_t* h, const char* symname, int ver, const char* vername, int local, int veropt);
+void old_elf_hash_dump(elfheader_t* h);
+uint32_t new_elf_hash(const char *name);
+Elf64_Sym* new_elf_lookup(elfheader_t* h, const char* symname, int ver, const char* vername, int local, int veropt);
+void new_elf_hash_dump(elfheader_t* h);
+
+Elf64_Sym* ElfLookup(elfheader_t* h, const char* symname, int ver, const char* vername, int local, int veropt);
+Elf64_Sym* ElfSymTabLookup(elfheader_t* h, const char* symname);
+Elf64_Sym* ElfDynSymLookup(elfheader_t* h, const char* symname);
+
 #endif //__ELFLOADER_PRIVATE_H_
