@@ -1,3 +1,5 @@
+#include <stddef.h>
+
 #define F8      *(uint8_t*)(addr++)
 #define F8S     *(int8_t*)(addr++)
 #define F16     *(uint16_t*)(addr+=2, addr-2)
@@ -26,6 +28,7 @@
 #define GETE8xw(D)          oped=TestEd8xw(test, rex.w, &addr, rex, nextop, D)
 #define GETED32(D)          oped=TestEd32O(test, &addr, rex, nextop, D, 0)
 #define GETED_OFFS(D, O)    oped=TestEdO(test, &addr, rex, nextop, D, O)
+#define GETED_OFFS_16(O)    oped=TestEd16off(test, &addr, rex, nextop, O)
 #define GETGD               opgd=GetGd(test->emu, &addr, rex, nextop)
 #define GETEB(D)            oped=TestEb(test, &addr, rex, nextop, D)
 #define GETEB32(D)          oped=TestEb32O(test, &addr, rex, nextop, D, 0)
@@ -36,13 +39,20 @@
 #define GETEW_OFFS(D, O)    oped=TestEdO(test, &addr, rex, nextop, D, O)
 #define GETEW_OFFS_16(O)    oped=TestEw16off(test, &addr, rex, nextop, O)
 #define GETGW               opgd=GetGw(test->emu, &addr, rex, nextop)
-#define GETEX(D)            opex=TestEx(test, &addr, rex, nextop, D)
+#define GETEX(D)            opex=TestEx(test, &addr, rex, nextop, D, 16)
+#define GETEX4(D)           opex=TestEx(test, &addr, rex, nextop, D, 4)
+#define GETEX8(D)           opex=TestEx(test, &addr, rex, nextop, D, 8)
 #define GETEX32(D)          opex=TestEx32O(test, &addr, rex, nextop, D, 0)
 #define GETEX_OFFS(D, O)    opex=TestExO(test, &addr, rex, nextop, D, O)
 #define GETGX               opgx=GetGx(test->emu, &addr, rex, nextop)
+#define GETGY               opgy=GetGy(test->emu, &addr, rex, nextop)
+#define GETEY               opey=TestEy(test, &addr, rex, nextop)
 #define GETEM(D)            opem=TestEm(test, &addr, rex, nextop, D)
 #define GETEM32(D)          opem=TestEm32O(test, &addr, rex, nextop, D, 0)
 #define GETGM               opgm=GetGm(test->emu, &addr, rex, nextop)
+#define GETVX               opvx=&test->emu->xmm[vex.v]
+#define GETVY               opvy=&test->emu->ymm[vex.v]
+#define GETVD               opvd=&test->emu->regs[vex.v]
 #else
 #define GETED(D)            oped=GetEd(emu, &addr, rex, nextop, D)
 #define GETE4(D)            GETED(D)
@@ -51,6 +61,7 @@
 #define GETE8xw(D)          GETED(D)
 #define GETED32(D)          oped=GetEd32O(emu, &addr, rex, nextop, D, 0)
 #define GETED_OFFS(D, O)    oped=GetEdO(emu, &addr, rex, nextop, D, O)
+#define GETED_OFFS_16(O)    oped=GetEd16off(emu, &addr, rex, nextop, O)
 #define GETGD               opgd=GetGd(emu, &addr, rex, nextop)
 #define GETEB(D)            oped=GetEb(emu, &addr, rex, nextop, D)
 #define GETEB32(D)          oped=GetEb32O(emu, &addr, rex, nextop, D, 0)
@@ -62,28 +73,43 @@
 #define GETEW_OFFS_16(O)    oped=GetEw16off(emu, &addr, rex, nextop, O)
 #define GETGW               opgd=GetGw(emu, &addr, rex, nextop)
 #define GETEX(D)            opex=GetEx(emu, &addr, rex, nextop, D)
+#define GETEX4(D)           GETEX(D)
+#define GETEX8(D)           GETEX(D)
 #define GETEX32(D)          opex=GetEx32O(emu, &addr, rex, nextop, D, 0)
 #define GETEX_OFFS(D, O)    opex=GetExO(emu, &addr, rex, nextop, D, O)
 #define GETGX               opgx=GetGx(emu, &addr, rex, nextop)
+#define GETGY               opgy=GetGy(emu, &addr, rex, nextop)
+#define GETEY               opey=(opex>=&emu->xmm[0] && opex<=&emu->xmm[15])?((sse_regs_t*)((uintptr_t)opex+offsetof(x64emu_t, ymm)-offsetof(x64emu_t, xmm))):((sse_regs_t*)((uintptr_t)opex+16))
 #define GETEM(D)            opem=GetEm(emu, &addr, rex, nextop, D)
 #define GETEM32(D)          opem=GetEm32O(emu, &addr, rex, nextop, D, 0)
 #define GETGM               opgm=GetGm(emu, &addr, rex, nextop)
+#define GETVX               opvx=&emu->xmm[vex.v]
+#define GETVY               opvy=&emu->ymm[vex.v]
+#define GETVD               opvd=&emu->regs[vex.v]
 #endif
 #define ED  oped
 #define GD  opgd
+#define VD  opvd
 #define EB  oped
 #define GB  opgd->byte[0]
 #define EW  oped
 #define GW  opgd
 #define EX  opex
 #define GX  opgx
+#define VX  opvx
+#define EY  opey
+#define GY  opgy
+#define VY  opvy
 #define EM  opem
 #define GM  opgm
 #define FAKEED(D)           GetEd(emu, &addr, rex, nextop, D)
 #define FAKEED32(D)         GetEd32O(emu, &addr, rex, nextop, D, 0)
 #define GETEA(D)            GetEA(emu, &addr, rex, nextop, D)
+#define GETEA32(D)          GetEA32(emu, &addr, rex, nextop, D)
 #define _GETED(D)           oped=GetEd(emu, &addr, rex, nextop, D)
 #define _GETED32(D)         oped=GetEd32O(emu, &addr, rex, nextop, D, 0)
+#define _GETEB(D)           oped=GetEb(emu, &addr, rex, nextop, D)
+#define _GETEX(D)           opex=GetEx(emu, &addr, rex, nextop, D)
 
 #define MODREG  ((nextop&0xC0)==0xC0)
 
