@@ -17,12 +17,11 @@
 #include "x64run_private.h"
 #include "callback.h"
 #include "bridge.h"
+#include "elfs/elfloader_private.h"
 #ifdef HAVE_TRACE
 #include "x64trace.h"
 #endif
-#ifdef DYNAREC
 #include "custommem.h"
-#endif
 // for the applyFlushTo0
 #ifdef __x86_64__
 #include <immintrin.h>
@@ -31,33 +30,24 @@
 #warning Architecture cannot follow SSE Flush to 0 flag
 #endif
 
-// from src/wrapped/wrappedlibc.c
-int my_munmap(x64emu_t* emu, void* addr, unsigned long length);
-
 typedef struct cleanup_s {
     void*       f;
     int         arg;
     void*       a;
-    void*       dso;
 } cleanup_t;
 
 static uint32_t x86emu_parity_tab[8] =
 {
-	0x96696996,
-	0x69969669,
-	0x69969669,
-	0x96696996,
-	0x69969669,
-	0x96696996,
-	0x96696996,
-	0x69969669,
+    0x96696996,
+    0x69969669,
+    0x69969669,
+    0x96696996,
+    0x69969669,
+    0x96696996,
+    0x96696996,
+    0x69969669,
 };
 
-uint32_t* GetParityTab()
-{
-    return x86emu_parity_tab;
-}
-
 static void internalX64Setup(x64emu_t* emu, box64context_t *context, uintptr_t start, uintptr_t stack, int stacksize, int ownstack)
 {
     emu->context = context;
@@ -137,46 +127,46 @@ void SetTraceEmu(uintptr_t start, uintptr_t end)
 }
 #endif
 
-void AddCleanup(x64emu_t *emu, void *p, void* dso_handle)
+void AddCleanup(x64emu_t *emu, void *p)
 {
     (void)emu;
     
     if(my_context->clean_sz == my_context->clean_cap) {
-        my_context->clean_cap += 4;
+        my_context->clean_cap += 32;
         my_context->cleanups = (cleanup_t*)box_realloc(my_context->cleanups, sizeof(cleanup_t)*my_context->clean_cap);
     }
     my_context->cleanups[my_context->clean_sz].arg = 0;
     my_context->cleanups[my_context->clean_sz].a = NULL;
-    my_context->cleanups[my_context->clean_sz].dso = dso_handle;
     my_context->cleanups[my_context->clean_sz++].f = p;
 }
 
-void AddCleanup1Arg(x64emu_t *emu, void *p, void* a, void* dso_handle)
+void AddCleanup1Arg(x64emu_t *emu, void *p, void* a, elfheader_t* h)
 {
     (void)emu;
+    if(!h)
+        return;
     
-    if(my_context->clean_sz == my_context->clean_cap) {
-        my_context->clean_cap += 4;
-        my_context->cleanups = (cleanup_t*)box_realloc(my_context->cleanups, sizeof(cleanup_t)*my_context->clean_cap);
+    if(h->clean_sz == h->clean_cap) {
+        h->clean_cap += 32;
+        h->cleanups = (cleanup_t*)box_realloc(h->cleanups, sizeof(cleanup_t)*h->clean_cap);
     }
-    my_context->cleanups[my_context->clean_sz].arg = 1;
-    my_context->cleanups[my_context->clean_sz].a = a;
-    my_context->cleanups[my_context->clean_sz].dso = dso_handle;
-    my_context->cleanups[my_context->clean_sz++].f = p;
+    h->cleanups[h->clean_sz].arg = 1;
+    h->cleanups[h->clean_sz].a = a;
+    h->cleanups[h->clean_sz++].f = p;
 }
 
-void CallCleanup(x64emu_t *emu, void* p)
+void CallCleanup(x64emu_t *emu, elfheader_t* h)
 {
-    printf_log(LOG_DEBUG, "Calling atexit registered functions for %p mask\n", p);
-    for(int i=my_context->clean_sz-1; i>=0; --i) {
-        if(p==my_context->cleanups[i].dso) {
-            printf_log(LOG_DEBUG, "Call cleanup #%d\n", i);
-            RunFunctionWithEmu(emu, 0, (uintptr_t)(my_context->cleanups[i].f), my_context->cleanups[i].arg, my_context->cleanups[i].a );
-            // now remove the cleanup
-            if(i!=my_context->clean_sz-1)
-                memmove(my_context->cleanups+i, my_context->cleanups+i+1, (my_context->clean_sz-i-1)*sizeof(cleanup_t));
-            --my_context->clean_sz;
-        }
+    printf_log(LOG_DEBUG, "Calling atexit registered functions for elf: %p/%s\n", h, h?h->name:"(nil)");
+    if(!h)
+        return;
+    for(int i=h->clean_sz-1; i>=0; --i) {
+        printf_log(LOG_DEBUG, "Call cleanup #%d\n", i);
+        RunFunctionWithEmu(emu, 0, (uintptr_t)(h->cleanups[i].f), h->cleanups[i].arg, h->cleanups[i].a );
+        // now remove the cleanup
+        if(i!=h->clean_sz-1)
+            memmove(h->cleanups+i, h->cleanups+i+1, (h->clean_sz-i-1)*sizeof(cleanup_t));
+        --h->clean_sz;
     }
 }
 
@@ -194,8 +184,10 @@ void CallAllCleanup(x64emu_t *emu)
 
 static void internalFreeX64(x64emu_t* emu)
 {
-    if(emu && emu->stack2free)
-        my_munmap(NULL, emu->stack2free, emu->size_stack);
+    if(emu && emu->stack2free) {
+        if(!internal_munmap(emu->stack2free, emu->size_stack))
+            freeProtection((uintptr_t)emu->stack2free, emu->size_stack);
+    }
 }
 
 EXPORTDYN
@@ -227,22 +219,23 @@ void FreeX64EmuFromStack(x64emu_t **emu)
 
 void CloneEmu(x64emu_t *newemu, const x64emu_t* emu)
 {
-	memcpy(newemu->regs, emu->regs, sizeof(emu->regs));
+    memcpy(newemu->regs, emu->regs, sizeof(emu->regs));
     memcpy(&newemu->ip, &emu->ip, sizeof(emu->ip));
-	memcpy(&newemu->eflags, &emu->eflags, sizeof(emu->eflags));
+    memcpy(&newemu->eflags, &emu->eflags, sizeof(emu->eflags));
     newemu->old_ip = emu->old_ip;
     memcpy(newemu->segs, emu->segs, sizeof(emu->segs));
     memset(newemu->segs_serial, 0, sizeof(newemu->segs_serial));
-	memcpy(newemu->x87, emu->x87, sizeof(emu->x87));
-	memcpy(newemu->mmx, emu->mmx, sizeof(emu->mmx));
+    memcpy(newemu->x87, emu->x87, sizeof(emu->x87));
+    memcpy(newemu->mmx, emu->mmx, sizeof(emu->mmx));
     memcpy(newemu->fpu_ld, emu->fpu_ld, sizeof(emu->fpu_ld));
     memcpy(newemu->fpu_ll, emu->fpu_ll, sizeof(emu->fpu_ll));
-	memcpy(newemu->p_regs, emu->p_regs, sizeof(emu->p_regs));
-	newemu->cw = emu->cw;
-	newemu->sw = emu->sw;
-	newemu->top = emu->top;
+    newemu->fpu_tags = emu->fpu_tags;
+    newemu->cw = emu->cw;
+    newemu->sw = emu->sw;
+    newemu->top = emu->top;
     newemu->fpu_stack = emu->fpu_stack;
     memcpy(newemu->xmm, emu->xmm, sizeof(emu->xmm));
+    memcpy(newemu->ymm, emu->ymm, sizeof(emu->ymm));
     newemu->df = emu->df;
     newemu->df_sav = emu->df_sav;
     newemu->op1 = emu->op1;
@@ -261,22 +254,23 @@ void CloneEmu(x64emu_t *newemu, const x64emu_t* emu)
 
 void CopyEmu(x64emu_t *newemu, const x64emu_t* emu)
 {
-	memcpy(newemu->regs, emu->regs, sizeof(emu->regs));
+    memcpy(newemu->regs, emu->regs, sizeof(emu->regs));
     memcpy(&newemu->ip, &emu->ip, sizeof(emu->ip));
-	memcpy(&newemu->eflags, &emu->eflags, sizeof(emu->eflags));
+    memcpy(&newemu->eflags, &emu->eflags, sizeof(emu->eflags));
     newemu->old_ip = emu->old_ip;
     memcpy(newemu->segs, emu->segs, sizeof(emu->segs));
     memcpy(newemu->segs_serial, emu->segs_serial, sizeof(emu->segs_serial));
     memcpy(newemu->segs_offs, emu->segs_offs, sizeof(emu->segs_offs));
-	memcpy(newemu->x87, emu->x87, sizeof(emu->x87));
-	memcpy(newemu->mmx, emu->mmx, sizeof(emu->mmx));
+    memcpy(newemu->x87, emu->x87, sizeof(emu->x87));
+    memcpy(newemu->mmx, emu->mmx, sizeof(emu->mmx));
     memcpy(newemu->xmm, emu->xmm, sizeof(emu->xmm));
+    memcpy(newemu->ymm, emu->ymm, sizeof(emu->ymm));
     memcpy(newemu->fpu_ld, emu->fpu_ld, sizeof(emu->fpu_ld));
     memcpy(newemu->fpu_ll, emu->fpu_ll, sizeof(emu->fpu_ll));
-	memcpy(newemu->p_regs, emu->p_regs, sizeof(emu->p_regs));
-	newemu->cw = emu->cw;
-	newemu->sw = emu->sw;
-	newemu->top = emu->top;
+    newemu->fpu_tags = emu->fpu_tags;
+    newemu->cw = emu->cw;
+    newemu->sw = emu->sw;
+    newemu->top = emu->top;
     newemu->fpu_stack = emu->fpu_stack;
     newemu->df = emu->df;
     newemu->df_sav = emu->df_sav;
@@ -424,13 +418,28 @@ const char* DumpCPURegs(x64emu_t* emu, uintptr_t ip, int is32bits)
                 sprintf(tmp, "%02d:%016lx-%016lx", i, emu->xmm[i].q[1], emu->xmm[i].q[0]);
             }
             strcat(buff, tmp);
-            if ((i&3)==3) strcat(buff, "\n"); else strcat(buff, " ");
+            if(box64_avx) {
+                if (trace_regsdiff && (emu->old_ymm[i].q[0] != emu->ymm[i].q[0] || emu->old_ymm[i].q[1] != emu->ymm[i].q[1])) {
+                    sprintf(tmp, "\e[1;35m-%016lx-%016lx\e[m", emu->ymm[i].q[1], emu->ymm[i].q[0]);
+                    emu->old_ymm[i].q[0] = emu->ymm[i].q[0];
+                    emu->old_ymm[i].q[1] = emu->ymm[i].q[1];
+                } else {
+                    sprintf(tmp, "-%016lx-%016lx", emu->ymm[i].q[1], emu->ymm[i].q[0]);
+                }
+                strcat(buff, tmp);
+            }
+            if(box64_avx)
+                if ((i&1)==1) strcat(buff, "\n"); else strcat(buff, " ");
+            else
+                if ((i&3)==3) strcat(buff, "\n"); else strcat(buff, " ");
         }
     }
 #endif
     // start with FPU regs...
     if(emu->fpu_stack) {
-        for (int i=0; i<emu->fpu_stack; i++) {
+        int stack = emu->fpu_stack;
+        if(stack>8) stack = 8;
+        for (int i=0; i<stack; i++) {
             sprintf(tmp, "ST%d=%f", i, ST(i).d);
             strcat(buff, tmp);
             int c = 10-strlen(tmp);
@@ -438,6 +447,8 @@ const char* DumpCPURegs(x64emu_t* emu, uintptr_t ip, int is32bits)
             while(c--) strcat(buff, " ");
             if(i==3) strcat(buff, "\n");
         }
+        sprintf(tmp, " C3210 = %d%d%d%d", emu->sw.f.F87_C3, emu->sw.f.F87_C2, emu->sw.f.F87_C1, emu->sw.f.F87_C0);
+        strcat(buff, tmp);
         strcat(buff, "\n");
     }
     for (int i=0; i<6; ++i) {
@@ -561,6 +572,8 @@ void EmuCall(x64emu_t* emu, uintptr_t addr)
     uint64_t old_rip = R_RIP;
     //Push64(emu, GetRBP(emu));   // set frame pointer
     //SetRBP(emu, GetRSP(emu));   // save RSP
+    R_RSP -= 200;
+    R_RSP &= ~63LL;
     PushExit(emu);
     R_RIP = addr;
     emu->df = d_none;
@@ -580,15 +593,77 @@ void EmuCall(x64emu_t* emu, uintptr_t addr)
     }
 }
 
+#if defined(RV64)
+static size_t readBinarySizeFromFile(const char* fname)
+{
+    if (access(fname, R_OK) != 0) return -1;
+    FILE* fp = fopen(fname, "r");
+    if (fp == NULL) return -1;
+
+    char b[sizeof(uint64_t)] = { 0 }, tmp;
+    ssize_t n = fread(b, 1, sizeof(b), fp);
+    if (n <= 0) return -1;
+
+    for (ssize_t i = 0; i < n / 2; i++) {
+        tmp = b[n - i - 1];
+        b[n - i - 1] = b[i];
+        b[i] = tmp;
+    }
+    return *(uint64_t*)b;
+}
+
+static inline uint64_t readCycleCounter()
+{
+    uint64_t val;
+    asm volatile("rdtime %0"
+                 : "=r"(val));
+    return val;
+}
+
+static inline uint64_t readFreq()
+{
+    static size_t val = -1;
+
+    val = readBinarySizeFromFile("/sys/firmware/devicetree/base/cpus/timebase-frequency");
+    if (val != (size_t)-1) return val;
+
+    // fallback to rdtime + sleep
+    struct timespec ts;
+    ts.tv_sec = 0;
+    ts.tv_nsec = 50000000; // 50 milliseconds
+    uint64_t cycles = readCycleCounter();
+    nanosleep(&ts, NULL);
+    // round to MHz
+    val = (size_t)round(((double)(readCycleCounter() - cycles) * 20) / 1e6) * 1e6;
+    return (uint64_t)val;
+}
+#elif defined(ARM64)
+static inline uint64_t readCycleCounter()
+{
+    uint64_t val;
+    asm volatile("mrs %0, cntvct_el0"
+                 : "=r"(val));
+    return val;
+}
+static inline uint64_t readFreq()
+{
+    uint64_t val;
+    asm volatile("mrs %0, cntfrq_el0"
+                 : "=r"(val));
+    return val;
+}
+#endif
+
 uint64_t ReadTSC(x64emu_t* emu)
 {
     (void)emu;
     
-    //TODO: implement hardware counter read? (only available in kernel space?)
-    // Read the TimeStamp Counter as 64bits.
-    // this is supposed to be the number of instructions executed since last reset
+    // Hardware counter, per architecture
+#if defined(ARM64) || defined(RV64)
+    if (!box64_rdtsc) return readCycleCounter();
+#endif
     // fall back to gettime...
-#ifndef NOGETCLOCK
+#if !defined(NOGETCLOCK)
     struct timespec ts;
     clock_gettime(CLOCK_MONOTONIC_COARSE, &ts);
     return (uint64_t)(ts.tv_sec) * 1000000000LL + ts.tv_nsec;
@@ -599,6 +674,21 @@ uint64_t ReadTSC(x64emu_t* emu)
 #endif
 }
 
+uint64_t ReadTSCFrequency(x64emu_t* emu)
+{
+    (void)emu;
+    // Hardware counter, per architecture
+#if defined(ARM64) || defined(RV64)
+    if (!box64_rdtsc) return readFreq();
+#endif
+    // fall back to get time
+#if !defined(NOGETCLOCK)
+    return 1000000000LL;
+#else
+    return 1000000;
+#endif
+}
+
 void ResetSegmentsCache(x64emu_t *emu)
 {
     if(!emu)
@@ -611,7 +701,7 @@ void applyFlushTo0(x64emu_t* emu)
     #ifdef __x86_64__
     _mm_setcsr(_mm_getcsr() | (emu->mxcsr.x32&0x8040));
     #elif defined(__aarch64__)
-    #ifdef __ANDROID__
+    #if defined(__ANDROID__) || defined(__clang__)
     uint64_t fpcr;
     __asm__ __volatile__ ("mrs    %0, fpcr":"=r"(fpcr));
     #else
@@ -620,7 +710,7 @@ void applyFlushTo0(x64emu_t* emu)
     fpcr &= ~((1<<24) | (1<<1));    // clear bit FZ (24) and AH (1)
     fpcr |= (emu->mxcsr.f.MXCSR_FZ)<<24;  // set FZ as mxcsr FZ
     fpcr |= ((emu->mxcsr.f.MXCSR_DAZ)^(emu->mxcsr.f.MXCSR_FZ))<<1; // set AH if DAZ different from FZ
-    #ifdef __ANDROID__
+    #if defined(__ANDROID__) || defined(__clang__)
     __asm__ __volatile__ ("msr    fpcr, %0"::"r"(fpcr));
     #else
     __builtin_aarch64_set_fpcr(fpcr);
