@@ -20,9 +20,13 @@ typedef struct forkpty_s {
 
 typedef union multiuint_s {
     uint8_t     u8;
+    int8_t      i8;
     uint16_t    u16;
+    int16_t     i16;
     uint32_t    u32;
+    int32_t     i32;
     uint64_t    u64;
+    int64_t     i64;
 } multiuint_t;
 
 typedef struct x64emu_s x64emu_t;
@@ -33,7 +37,8 @@ typedef struct x64test_s {
     int         memsize;
     int         test;
     int         clean;
-    uint8_t     mem[16];
+    int         notest;
+    uint8_t     mem[32];
 } x64test_t;
 
 typedef struct emu_flags_s {
@@ -53,23 +58,27 @@ typedef struct emu_flags_s {
 
 typedef struct x64emu_s {
     // cpu
-	reg64_t     regs[16];
-	x64flags_t  eflags;
+    reg64_t     regs[16];
+    x64flags_t  eflags;
     reg64_t     ip;
     // sse
     sse_regs_t  xmm[16];
+    sse_regs_t  ymm[16];
     // fpu / mmx
-	mmx87_regs_t x87[8];
-	mmx87_regs_t mmx[8];
-	x87flags_t  sw;
-	uint32_t    top;        // top is part of sw, but it's faster to have it separately
+    mmx87_regs_t x87[8];
+    mmx87_regs_t mmx[8];
+    x87flags_t  sw;
+    uint32_t    top;        // top is part of sw, but it's faster to have it separately
     int         fpu_stack;
-	x87control_t cw;
+    x87control_t cw;
     uint16_t    dummy_cw;   // align...
     mmxcontrol_t mxcsr;
+    #ifdef RV64         // it would be better to use a dedicated register for this like arm64 xSavedSP, but we're running of of free registers.
+    uintptr_t xSPSave;  // sp base value of current dynarec frame, used by call/ret optimization to reset stack when unmatch.
+    #endif
     fpu_ld_t    fpu_ld[8]; // for long double emulation / 80bits fld fst
     fpu_ll_t    fpu_ll[8]; // for 64bits fild / fist sequence
-	fpu_p_reg_t p_regs[8];
+    uint64_t    fpu_tags;   // tags for the x87 regs, stacked, only on a 16bits anyway
     // old ip
     uintptr_t   old_ip;
     // deferred flags
@@ -82,10 +91,6 @@ typedef struct x64emu_s {
     multiuint_t res_sav;
     deferred_flags_t df_sav;
     uint32_t    *x64emu_parity_tab; // helper
-    #ifdef HAVE_TRACE
-    reg64_t     oldregs[16];
-    uintptr_t   prev2_ip;
-    #endif
     // segments
     uint16_t    segs[6];        // only 32bits value?
     uint16_t    dummy_seg6, dummy_seg7; // to stay aligned
@@ -106,6 +111,9 @@ typedef struct x64emu_s {
     x64test_t   test;       // used for dynarec testing
     #ifdef HAVE_TRACE
     sse_regs_t  old_xmm[16];
+    sse_regs_t  old_ymm[16];
+    reg64_t     oldregs[16];
+    uintptr_t   prev2_ip;
     #endif
     // scratch stack, used for alignment of double and 64bits ints on arm. 200 elements should be enough
     uint64_t    scratch[200];
@@ -114,6 +122,9 @@ typedef struct x64emu_s {
     void*       init_stack; // initial stack (owned or not)
     uint32_t    size_stack; // stack size (owned or not)
     JUMPBUFF*   jmpbuf;
+    #ifdef RV64
+    uintptr_t   old_savedsp;
+    #endif
 
     x64_ucontext_t *uc_link; // to handle setcontext
 
