@@ -91,7 +91,7 @@ uintptr_t Run66DD(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     char* p =(char*)ED;
                     p += 14;
                     for (int i=0; i<8; ++i) {
-                        LD2D(p, &ST(i).d);
+                        LD2D(p, &emu->x87[7-i].d);
                         p+=10;
                     }
                 }
@@ -106,7 +106,7 @@ uintptr_t Run66DD(x64emu_t *emu, rex_t rex, uintptr_t addr)
                     char* p =(char*)ED;
                     p += 14;
                     for (int i=0; i<8; ++i) {
-                        D2LD(&ST(i).d, p);
+                        D2LD(&emu->x87[7-i].d, p);
                         p+=10;
                     }
                 }
