@@ -111,7 +111,11 @@ uintptr_t Run67(x64emu_t *emu, rex_t rex, int rep, uintptr_t addr)
     GO(0x00, add)                   /* ADD 0x00 -> 0x05 */
     GO(0x08, or)                    /*  OR 0x08 -> 0x0D */
     case 0x0F:
+        #ifdef TEST_INTERPRETER
+        return Test670F(test, rex, rep, addr);
+        #else
         return Run670F(emu, rex, rep, addr);
+        #endif
     GO(0x10, adc)                   /* ADC 0x10 -> 0x15 */
     GO(0x18, sbb)                   /* SBB 0x18 -> 0x1D */
     GO(0x20, and)                   /* AND 0x20 -> 0x25 */
@@ -328,6 +332,64 @@ uintptr_t Run67(x64emu_t *emu, rex_t rex, int rep, uintptr_t addr)
         }
         break;
 
+    case 0xC4:                      /* LES Gd,Ed */
+        nextop = F8;
+        if(rex.is32bits && !(MODREG)) {
+            return 0;
+        } else {
+            if(rex.is32bits) {
+                return 0;
+            }
+            vex_t vex = {0};
+            vex.rex = rex;
+            tmp8u = nextop;
+            vex.m = tmp8u&0b00011111;
+            vex.rex.b = (tmp8u&0b00100000)?0:1;
+            vex.rex.x = (tmp8u&0b01000000)?0:1;
+            vex.rex.r = (tmp8u&0b10000000)?0:1;
+            tmp8u = F8;
+            vex.p = tmp8u&0b00000011;
+            vex.l = (tmp8u>>2)&1;
+            vex.v = ((~tmp8u)>>3)&0b1111;
+            vex.rex.w = (tmp8u>>7)&1;
+            #ifdef TEST_INTERPRETER 
+            addr = Test67AVX(test, vex, addr);
+            #else
+            addr = Run67AVX(emu, vex, addr);
+            #endif
+        }
+        break;
+    case 0xC5:                      /* LDS Gd,Ed */
+        nextop = F8;
+        if(rex.is32bits && !(MODREG)) {
+            return 0;
+        } else {
+            if(rex.is32bits) {
+                return 0;
+            }
+            vex_t vex = {0};
+            vex.rex = rex;
+            tmp8u = nextop;
+            vex.p = tmp8u&0b00000011;
+            vex.l = (tmp8u>>2)&1;
+            vex.v = ((~tmp8u)>>3)&0b1111;
+            vex.rex.r = (tmp8u&0b10000000)?0:1;
+            vex.rex.b = 0;
+            vex.rex.x = 0;
+            vex.rex.w = 0;
+            vex.m = VEX_M_0F;
+            #ifdef TEST_INTERPRETER 
+            addr = Test67AVX(test, vex, addr);
+            #else
+            addr = Run67AVX(emu, vex, addr);
+            #endif
+        }
+        break;
+    case 0xC6:                      /* MOV Eb,Ib */
+        nextop = F8;
+        GETEB32(1);
+        EB->byte[0] = F8;
+        break;
     case 0xC7:                      /* MOV Ed,Id */
         nextop = F8;
         GETED32(4);
@@ -400,6 +462,9 @@ uintptr_t Run67(x64emu_t *emu, rex_t rex, int rep, uintptr_t addr)
                     break;
                 case 7:                 /* IDIV Ed */
                     idiv64(emu, ED->q[0]);
+                    #ifdef TEST_INTERPRETER
+                    test->notest = 1;
+                    #endif
                     break;
             }
         } else {
@@ -444,7 +509,42 @@ uintptr_t Run67(x64emu_t *emu, rex_t rex, int rep, uintptr_t addr)
             }
         }
         break;
-            
+    
+    case 0xFF:
+        nextop = F8;
+        switch((nextop>>3)&7) {
+                case 0:                 /* INC Ed */
+                GETED32(0);
+                if(rex.w)
+                    ED->q[0] = inc64(emu, ED->q[0]);
+                else {
+                    if(MODREG)
+                        ED->q[0] = inc32(emu, ED->dword[0]);
+                    else
+                        ED->dword[0] = inc32(emu, ED->dword[0]);
+                }
+                break;
+            case 1:                 /* DEC Ed */
+                GETED32(0);
+                if(rex.w)
+                    ED->q[0] = dec64(emu, ED->q[0]);
+                else {
+                    if(MODREG)
+                        ED->q[0] = dec32(emu, ED->dword[0]);
+                    else
+                        ED->dword[0] = dec32(emu, ED->dword[0]);
+                }
+                break;
+           case 2:                 /* CALL NEAR Ed */
+                GETED32(0);
+                tmp64u = (uintptr_t)getAlternate((void*)ED->q[0]);
+                Push64(emu, addr);
+                addr = tmp64u;
+                break;
+            default:
+                return 0;
+        }
+        break;
     default:
         return 0;
     }
