@@ -26,10 +26,10 @@
 #include "bridge.h"
 
 #define PARITY(x)   (((emu->x64emu_parity_tab[(x) / 32] >> ((x) % 32)) & 1) == 0)
-#define XOR2(x) 	(((x) ^ ((x)>>1)) & 0x1)
+#define XOR2(x)     (((x) ^ ((x)>>1)) & 0x1)
 
 #ifdef ANDROID
-void EXPORT my___libc_init(x64emu_t* emu, void* raw_args __unused, void (*onexit)(void) __unused, int (*main)(int, char**, char**), void const * const structors __unused)
+void EXPORT my___libc_init(x64emu_t* emu, void* raw_args , void (*onexit)(void) , int (*main)(int, char**, char**), void const * const structors )
 {
     //TODO: register fini
     // let's cheat and set all args...
@@ -40,15 +40,20 @@ void EXPORT my___libc_init(x64emu_t* emu, void* raw_args __unused, void (*onexit
     printf_log(LOG_DEBUG, "Transfert to main(%d, %p, %p)=>%p from __libc_init\n", my_context->argc, my_context->argv, my_context->envv, main);
     // should call structors->preinit_array and structors->init_array!
     // call main and finish
+    Push64(emu, GetRBP(emu));   // set frame pointer
+    SetRBP(emu, GetRSP(emu));   // save RSP
+    SetRSP(emu, GetRSP(emu)&~0xFLL);    // Align RSP
     PushExit(emu);
     R_RIP=(uintptr_t)main;
 
     DynaRun(emu);
 
+    SetRSP(emu, GetRBP(emu));   // restore RSP
+    SetRBP(emu, Pop64(emu));    // restore RBP
     emu->quit = 1; // finished!
 }
 #else
-int32_t EXPORT my___libc_start_main(x64emu_t* emu, int *(main) (int, char * *, char * *), int argc, char * * ubp_av, void (*init) (void), void (*fini) (void), void (*rtld_fini) (void), void (* stack_end))
+int32_t EXPORT my___libc_start_main(x64emu_t* emu, int (*main) (int, char * *, char * *), int argc, char * * ubp_av, void (*init) (void), void (*fini) (void), void (*rtld_fini) (void), void (* stack_end))
 {
     (void)argc; (void)ubp_av; (void)fini; (void)rtld_fini; (void)stack_end;
 
@@ -63,7 +68,7 @@ int32_t EXPORT my___libc_start_main(x64emu_t* emu, int *(main) (int, char * *, c
         SetRSI(emu, (uint64_t)my_context->argv);
         SetRDI(emu, (uint64_t)my_context->argc);
         R_RIP=(uint64_t)*init;
-        printf_log(LOG_DEBUG, "Calling init(%p) from __libc_start_main\n", *init);
+        printf_dump(LOG_DEBUG, "Calling init(%p) from __libc_start_main\n", *init);
         DynaRun(emu);
         if(emu->error)  // any error, don't bother with more
             return 0;
@@ -74,6 +79,7 @@ int32_t EXPORT my___libc_start_main(x64emu_t* emu, int *(main) (int, char * *, c
         emu->quit = 0;
     } else {
         if(my_context->elfs[0]) {
+            printf_dump(LOG_DEBUG, "Calling init from main elf\n");
             RunElfInit(my_context->elfs[0], emu);
         }
     }
@@ -121,7 +127,7 @@ const char* GetNativeName(void* p)
         if(info.dli_sname) {
             strcpy(buff, info.dli_sname);
             if(info.dli_fname) {
-                strcat(buff, " ("); strcat(buff, info.dli_fname); strcat(buff, ")");
+                strcat(buff, "("); strcat(buff, info.dli_fname); strcat(buff, ")");
             }
         } else {
             sprintf(buff, "%s(%s+%p)", "???", info.dli_fname, (void*)(p-info.dli_fbase));
@@ -495,88 +501,55 @@ void UpdateFlags(x64emu_t *emu)
             CONDITIONAL_SET_FLAG(bc & 0x8, F_AF);
             break;
         case d_shl8:
-            if (emu->op2.u8 < 8) {
-                cnt = emu->op2.u8 % 8;
-                if (cnt > 0) {
-                    cc = emu->op1.u8 & (1 << (8 - cnt));
-                    CONDITIONAL_SET_FLAG(cc, F_CF);
-                    CONDITIONAL_SET_FLAG((emu->res.u8 & 0xff) == 0, F_ZF);
-                    CONDITIONAL_SET_FLAG(emu->res.u8 & 0x80, F_SF);
-                    CONDITIONAL_SET_FLAG(PARITY(emu->res.u8 & 0xff), F_PF);
-                    if (cnt == 1) {
-                        CONDITIONAL_SET_FLAG((((emu->res.u8 & 0x80) == 0x80) ^(ACCESS_FLAG(F_CF) != 0)), F_OF);
-                    } else {
-                        CLEAR_FLAG(F_OF);
-                    }
-                    if(box64_dynarec_test) {
-                        CLEAR_FLAG(F_AF);
-                    }
+            cnt = emu->op2.u8 & 0x1f;
+            if (cnt > 0) {
+                cc = emu->op1.u8 & (1 << (8 - cnt));
+                CONDITIONAL_SET_FLAG(cc, F_CF);
+                CONDITIONAL_SET_FLAG((emu->res.u8 & 0xff) == 0, F_ZF);
+                CONDITIONAL_SET_FLAG(emu->res.u8 & 0x80, F_SF);
+                CONDITIONAL_SET_FLAG(PARITY(emu->res.u8 & 0xff), F_PF);
+                if (cnt == 1) {
+                    CONDITIONAL_SET_FLAG((((emu->res.u8 & 0x80) == 0x80) ^(ACCESS_FLAG(F_CF) != 0)), F_OF);
+                } else {
+                    CLEAR_FLAG(F_OF);
                 }
-            } else {
-                CONDITIONAL_SET_FLAG((emu->op1.u8 << (emu->op2.u8-1)) & 0x80, F_CF);
-                CLEAR_FLAG(F_OF);
-                CLEAR_FLAG(F_SF);
-                SET_FLAG(F_PF);
-                SET_FLAG(F_ZF);
                 if(box64_dynarec_test) {
                     CLEAR_FLAG(F_AF);
                 }
             }
             break;
         case d_shl16:
-            if (emu->op2.u16 < 16) {
-                cnt = emu->op2.u16 % 16;
-                if (cnt > 0) {
-                    cc = emu->op1.u16 & (1 << (16 - cnt));
-                    CONDITIONAL_SET_FLAG(cc, F_CF);
-                    CONDITIONAL_SET_FLAG((emu->res.u16 & 0xffff) == 0, F_ZF);
-                    CONDITIONAL_SET_FLAG(emu->res.u16 & 0x8000, F_SF);
-                    CONDITIONAL_SET_FLAG(PARITY(emu->res.u16 & 0xff), F_PF);
-                    if (cnt == 1) {
-                        CONDITIONAL_SET_FLAG(((!!(emu->res.u16 & 0x8000)) ^(ACCESS_FLAG(F_CF) != 0)), F_OF);
-                    } else {
-                        CLEAR_FLAG(F_OF);
-                    }
-                    if(box64_dynarec_test) {
-                        CLEAR_FLAG(F_AF);
-                    }
+            cnt = emu->op2.u16 & 0x1f;
+            if (cnt > 0) {
+                cc = emu->op1.u16 & (1 << (16 - cnt));
+                CONDITIONAL_SET_FLAG(cc, F_CF);
+                CONDITIONAL_SET_FLAG((emu->res.u16 & 0xffff) == 0, F_ZF);
+                CONDITIONAL_SET_FLAG(emu->res.u16 & 0x8000, F_SF);
+                CONDITIONAL_SET_FLAG(PARITY(emu->res.u16 & 0xff), F_PF);
+                if (cnt == 1) {
+                    CONDITIONAL_SET_FLAG(((!!(emu->res.u16 & 0x8000)) ^(ACCESS_FLAG(F_CF) != 0)), F_OF);
+                } else {
+                    CLEAR_FLAG(F_OF);
                 }
-            } else {
-                CONDITIONAL_SET_FLAG((emu->op1.u16 << (emu->op2.u16-1)) & 0x8000, F_CF);
-                CLEAR_FLAG(F_OF);
-                CLEAR_FLAG(F_SF);
-                SET_FLAG(F_PF);
-                SET_FLAG(F_ZF);
                 if(box64_dynarec_test) {
                     CLEAR_FLAG(F_AF);
                 }
             }
             break;
         case d_shl32:
-            if (emu->op2.u32 < 32) {
-                cnt = emu->op2.u32 % 32;
-                if (cnt > 0) {
-                    cc = emu->op1.u32 & (1 << (32 - cnt));
-                    CONDITIONAL_SET_FLAG(cc, F_CF);
-                    CONDITIONAL_SET_FLAG(!emu->res.u32, F_ZF);
-                    CONDITIONAL_SET_FLAG(emu->res.u32 & 0x80000000, F_SF);
-                    CONDITIONAL_SET_FLAG(PARITY(emu->res.u32 & 0xff), F_PF);
-                    if (cnt == 1) {
-                        CONDITIONAL_SET_FLAG(((!!(emu->res.u32 & 0x80000000)) ^
-                                                (ACCESS_FLAG(F_CF) != 0)), F_OF);
-                    } else {
-                        CLEAR_FLAG(F_OF);
-                    }
-                    if(box64_dynarec_test) {
-                        CLEAR_FLAG(F_AF);
-                    }
+            cnt = emu->op2.u32 & 0x1f;
+            if (cnt > 0) {
+                cc = emu->op1.u32 & (1 << (32 - cnt));
+                CONDITIONAL_SET_FLAG(cc, F_CF);
+                CONDITIONAL_SET_FLAG(!emu->res.u32, F_ZF);
+                CONDITIONAL_SET_FLAG(emu->res.u32 & 0x80000000, F_SF);
+                CONDITIONAL_SET_FLAG(PARITY(emu->res.u32 & 0xff), F_PF);
+                if (cnt == 1) {
+                    CONDITIONAL_SET_FLAG(((!!(emu->res.u32 & 0x80000000)) ^
+                                            (ACCESS_FLAG(F_CF) != 0)), F_OF);
+                } else {
+                    CLEAR_FLAG(F_OF);
                 }
-            } else {
-                CONDITIONAL_SET_FLAG((emu->op1.u32 << (emu->op2.u32-1)) & 0x80000000, F_CF);
-                CLEAR_FLAG(F_OF);
-                CLEAR_FLAG(F_SF);
-                SET_FLAG(F_PF);
-                SET_FLAG(F_ZF);
                 if(box64_dynarec_test) {
                     CLEAR_FLAG(F_AF);
                 }
@@ -601,93 +574,42 @@ void UpdateFlags(x64emu_t *emu)
             }
             break;
         case d_sar8:
-            if (emu->op2.u8 < 8) {
-                if(emu->op2.u8) {
-                    cc = emu->op1.u8 & (1 << (emu->op2.u8 - 1));
-                    CONDITIONAL_SET_FLAG(cc, F_CF);
-                    CONDITIONAL_SET_FLAG((emu->res.u8 & 0xff) == 0, F_ZF);
-                    CONDITIONAL_SET_FLAG(PARITY(emu->res.u8 & 0xff), F_PF);
-                    CONDITIONAL_SET_FLAG(emu->res.u8 & 0x80, F_SF);
-                    if(emu->op2.u8==1)
-                        CLEAR_FLAG(F_OF);
-                    if(box64_dynarec_test) {
-                        CLEAR_FLAG(F_AF);
-                    }
-                }
-            } else {
-                if (emu->op1.u8&0x80) {
-                    SET_FLAG(F_CF);
-                    SET_FLAG(F_SF);
-                    CLEAR_FLAG(F_ZF);
-                    SET_FLAG(F_PF);
-                } else {
-                    CLEAR_FLAG(F_CF);
-                    CLEAR_FLAG(F_SF);
-                    SET_FLAG(F_ZF);
-                    SET_FLAG(F_PF);
-                }
+            if(emu->op2.u8) {
+                cc = (emu->op1.i8 >> (emu->op2.u8 - 1)) & 1;
+                CONDITIONAL_SET_FLAG(cc, F_CF);
+                CONDITIONAL_SET_FLAG((emu->res.u8 & 0xff) == 0, F_ZF);
+                CONDITIONAL_SET_FLAG(PARITY(emu->res.u8 & 0xff), F_PF);
+                CONDITIONAL_SET_FLAG(emu->res.u8 & 0x80, F_SF);
+                if(emu->op2.u8==1)
+                    CLEAR_FLAG(F_OF);
                 if(box64_dynarec_test) {
                     CLEAR_FLAG(F_AF);
                 }
             }
             break;
         case d_sar16:
-            if (emu->op2.u16 < 16) {
-                if(emu->op2.u16) {
-                    cc = emu->op1.u16 & (1 << (emu->op2.u16 - 1));
-                    CONDITIONAL_SET_FLAG(cc, F_CF);
-                    CONDITIONAL_SET_FLAG((emu->res.u16 & 0xffff) == 0, F_ZF);
-                    CONDITIONAL_SET_FLAG(emu->res.u16 & 0x8000, F_SF);
-                    CONDITIONAL_SET_FLAG(PARITY(emu->res.u16 & 0xff), F_PF);
-                    if(emu->op2.u16==1)
-                        CLEAR_FLAG(F_OF);
-                    if(box64_dynarec_test) {
-                        CLEAR_FLAG(F_AF);
-                    }
-                }
-            } else {
-                if (emu->op1.u16&0x8000) {
-                    SET_FLAG(F_CF);
-                    CLEAR_FLAG(F_ZF);
-                    SET_FLAG(F_SF);
-                    SET_FLAG(F_PF);
-                } else {
-                    CLEAR_FLAG(F_CF);
-                    SET_FLAG(F_ZF);
-                    CLEAR_FLAG(F_SF);
-                    SET_FLAG(F_PF);
-                }
+            if(emu->op2.u16) {
+                cc = (emu->op1.i16 >> (emu->op2.u16 - 1)) & 1;
+                CONDITIONAL_SET_FLAG(cc, F_CF);
+                CONDITIONAL_SET_FLAG((emu->res.u16 & 0xffff) == 0, F_ZF);
+                CONDITIONAL_SET_FLAG(emu->res.u16 & 0x8000, F_SF);
+                CONDITIONAL_SET_FLAG(PARITY(emu->res.u16 & 0xff), F_PF);
+                if(emu->op2.u16==1)
+                    CLEAR_FLAG(F_OF);
                 if(box64_dynarec_test) {
                     CLEAR_FLAG(F_AF);
                 }
             }
             break;
         case d_sar32:
-            if (emu->op2.u32 < 32) {
-                if(emu->op2.u32) {
-                    cc = emu->op1.u32 & (1 << (emu->op2.u32 - 1));
-                    CONDITIONAL_SET_FLAG(cc, F_CF);
-                    CONDITIONAL_SET_FLAG((emu->res.u32 & 0xffffffff) == 0, F_ZF);
-                    CONDITIONAL_SET_FLAG(emu->res.u32 & 0x80000000, F_SF);
-                    CONDITIONAL_SET_FLAG(PARITY(emu->res.u32 & 0xff), F_PF);
-                    if(emu->op2.u32==1)
-                        CLEAR_FLAG(F_OF);
-                    if(box64_dynarec_test) {
-                        CLEAR_FLAG(F_AF);
-                    }
-                }
-            } else {
-                if (emu->op1.u32&0x80000000) {
-                    SET_FLAG(F_CF);
-                    CLEAR_FLAG(F_ZF);
-                    SET_FLAG(F_SF);
-                    SET_FLAG(F_PF);
-                } else {
-                    CLEAR_FLAG(F_CF);
-                    SET_FLAG(F_ZF);
-                    CLEAR_FLAG(F_SF);
-                    SET_FLAG(F_PF);
-                }
+            if(emu->op2.u32) {
+                cc = emu->op1.u32 & (1 << (emu->op2.u32 - 1));
+                CONDITIONAL_SET_FLAG(cc, F_CF);
+                CONDITIONAL_SET_FLAG((emu->res.u32 & 0xffffffff) == 0, F_ZF);
+                CONDITIONAL_SET_FLAG(emu->res.u32 & 0x80000000, F_SF);
+                CONDITIONAL_SET_FLAG(PARITY(emu->res.u32 & 0xff), F_PF);
+                if(emu->op2.u32==1)
+                    CLEAR_FLAG(F_OF);
                 if(box64_dynarec_test) {
                     CLEAR_FLAG(F_AF);
                 }
@@ -708,82 +630,52 @@ void UpdateFlags(x64emu_t *emu)
             }
             break;
         case d_shr8:
-            if (emu->op2.u8 < 8) {
-                cnt = emu->op2.u8 % 8;
-                if (cnt > 0) {
-                    cc = emu->op1.u8 & (1 << (cnt - 1));
-                    CONDITIONAL_SET_FLAG(cc, F_CF);
-                    CONDITIONAL_SET_FLAG((emu->res.u8 & 0xff) == 0, F_ZF);
-                    CONDITIONAL_SET_FLAG(emu->res.u8 & 0x80, F_SF);
-                    CONDITIONAL_SET_FLAG(PARITY(emu->res.u8 & 0xff), F_PF);
-                    if(box64_dynarec_test) {
-                        CLEAR_FLAG(F_AF);
-                    }
-                }
-                if (cnt == 1) {
-                    CONDITIONAL_SET_FLAG(emu->op1.u8 & 0x80, F_OF);
-                }
-            } else {
-                CONDITIONAL_SET_FLAG((emu->op1.u8 >> (emu->op2.u8-1)) & 0x1, F_CF);
-                CLEAR_FLAG(F_SF);
-                SET_FLAG(F_PF);
-                SET_FLAG(F_ZF);
+            cnt = emu->op2.u8 & 0x1f;
+            if (cnt > 0) {
+                cc = emu->op1.u8 & (1 << (cnt - 1));
+                CONDITIONAL_SET_FLAG(cc, F_CF);
+                CONDITIONAL_SET_FLAG((emu->res.u8 & 0xff) == 0, F_ZF);
+                CONDITIONAL_SET_FLAG(emu->res.u8 & 0x80, F_SF);
+                CONDITIONAL_SET_FLAG(PARITY(emu->res.u8 & 0xff), F_PF);
                 if(box64_dynarec_test) {
                     CLEAR_FLAG(F_AF);
                 }
             }
+            if (cnt == 1) {
+                CONDITIONAL_SET_FLAG(emu->op1.u8 & 0x80, F_OF);
+            }
             break;
         case d_shr16:
-            if (emu->op2.u16 < 16) {
-                cnt = emu->op2.u16 % 16;
-                if (cnt > 0) {
-                    cc = emu->op1.u16 & (1 << (cnt - 1));
-                    CONDITIONAL_SET_FLAG(cc, F_CF);
-                    CONDITIONAL_SET_FLAG((emu->res.u16 & 0xffff) == 0, F_ZF);
-                    CONDITIONAL_SET_FLAG(emu->res.u16 & 0x8000, F_SF);
-                    CONDITIONAL_SET_FLAG(PARITY(emu->res.u16 & 0xff), F_PF);
-                    if(box64_dynarec_test) {
-                        CLEAR_FLAG(F_AF);
-                    }
-                }
-                if (cnt == 1) {
-                    CONDITIONAL_SET_FLAG(emu->op1.u16 & 0x8000, F_OF);
-                }
-            } else {
-                CLEAR_FLAG(F_CF);
-                SET_FLAG(F_ZF);
-                CLEAR_FLAG(F_SF);
-                SET_FLAG(F_PF);
+            cnt = emu->op2.u16 & 0x1f;
+            if (cnt > 0) {
+                cc = emu->op1.u16 & (1 << (cnt - 1));
+                CONDITIONAL_SET_FLAG(cc, F_CF);
+                CONDITIONAL_SET_FLAG((emu->res.u16 & 0xffff) == 0, F_ZF);
+                CONDITIONAL_SET_FLAG(emu->res.u16 & 0x8000, F_SF);
+                CONDITIONAL_SET_FLAG(PARITY(emu->res.u16 & 0xff), F_PF);
                 if(box64_dynarec_test) {
                     CLEAR_FLAG(F_AF);
                 }
             }
+            if (cnt == 1) {
+                CONDITIONAL_SET_FLAG(emu->op1.u16 & 0x8000, F_OF);
+            }
             break;
         case d_shr32:
-            if (emu->op2.u32 < 32) {
-                cnt = emu->op2.u32 % 32;
-                if (cnt > 0) {
-                    cc = emu->op1.u32 & (1 << (cnt - 1));
-                    CONDITIONAL_SET_FLAG(cc, F_CF);
-                    CONDITIONAL_SET_FLAG((emu->res.u32 & 0xffffffff) == 0, F_ZF);
-                    CONDITIONAL_SET_FLAG(emu->res.u32 & 0x80000000, F_SF);
-                    CONDITIONAL_SET_FLAG(PARITY(emu->res.u32 & 0xff), F_PF);
-                    if(box64_dynarec_test) {
-                        CLEAR_FLAG(F_AF);
-                    }
-                }
-                if (cnt == 1) {
-                    CONDITIONAL_SET_FLAG(emu->op1.u32 & 0x80000000, F_OF);
-                }
-            } else {
-                CLEAR_FLAG(F_CF);
-                SET_FLAG(F_ZF);
-                CLEAR_FLAG(F_SF);
-                SET_FLAG(F_PF);
+            cnt = emu->op2.u32 & 0x1f;
+            if (cnt > 0) {
+                cc = emu->op1.u32 & (1 << (cnt - 1));
+                CONDITIONAL_SET_FLAG(cc, F_CF);
+                CONDITIONAL_SET_FLAG((emu->res.u32 & 0xffffffff) == 0, F_ZF);
+                CONDITIONAL_SET_FLAG(emu->res.u32 & 0x80000000, F_SF);
+                CONDITIONAL_SET_FLAG(PARITY(emu->res.u32 & 0xff), F_PF);
                 if(box64_dynarec_test) {
                     CLEAR_FLAG(F_AF);
                 }
             }
+            if (cnt == 1) {
+                CONDITIONAL_SET_FLAG(emu->op1.u32 & 0x80000000, F_OF);
+            }
             break;
         case d_shr64:
             cnt = emu->op2.u64;
@@ -973,29 +865,29 @@ void UpdateFlags(x64emu_t *emu)
             CONDITIONAL_SET_FLAG(bc & 0x8, F_AF);
             break;
         case d_cmp32:
-        	CONDITIONAL_SET_FLAG(emu->res.u32 & 0x80000000, F_SF);
-        	CONDITIONAL_SET_FLAG(!emu->res.u32, F_ZF);
-        	CONDITIONAL_SET_FLAG(PARITY(emu->res.u32 & 0xff), F_PF);
-        	bc = (emu->res.u32 & (~emu->op1.u32 | emu->op2.u32)) | (~emu->op1.u32 & emu->op2.u32);
-        	CONDITIONAL_SET_FLAG(bc & 0x80000000, F_CF);
-        	CONDITIONAL_SET_FLAG(XOR2(bc >> 30), F_OF);
-        	CONDITIONAL_SET_FLAG(bc & 0x8, F_AF);
+            CONDITIONAL_SET_FLAG(emu->res.u32 & 0x80000000, F_SF);
+            CONDITIONAL_SET_FLAG(!emu->res.u32, F_ZF);
+            CONDITIONAL_SET_FLAG(PARITY(emu->res.u32 & 0xff), F_PF);
+            bc = (emu->res.u32 & (~emu->op1.u32 | emu->op2.u32)) | (~emu->op1.u32 & emu->op2.u32);
+            CONDITIONAL_SET_FLAG(bc & 0x80000000, F_CF);
+            CONDITIONAL_SET_FLAG(XOR2(bc >> 30), F_OF);
+            CONDITIONAL_SET_FLAG(bc & 0x8, F_AF);
             break;
         case d_cmp64:
-        	CONDITIONAL_SET_FLAG(emu->res.u64 & 0x8000000000000000LL, F_SF);
-        	CONDITIONAL_SET_FLAG(!emu->res.u64, F_ZF);
-        	CONDITIONAL_SET_FLAG(PARITY(emu->res.u64 & 0xff), F_PF);
-        	bc = (emu->res.u64 & (~emu->op1.u64 | emu->op2.u64)) | (~emu->op1.u64 & emu->op2.u64);
-        	CONDITIONAL_SET_FLAG(bc & 0x8000000000000000LL, F_CF);
-        	CONDITIONAL_SET_FLAG(XOR2(bc >> 62), F_OF);
-        	CONDITIONAL_SET_FLAG(bc & 0x8, F_AF);
+            CONDITIONAL_SET_FLAG(emu->res.u64 & 0x8000000000000000LL, F_SF);
+            CONDITIONAL_SET_FLAG(!emu->res.u64, F_ZF);
+            CONDITIONAL_SET_FLAG(PARITY(emu->res.u64 & 0xff), F_PF);
+            bc = (emu->res.u64 & (~emu->op1.u64 | emu->op2.u64)) | (~emu->op1.u64 & emu->op2.u64);
+            CONDITIONAL_SET_FLAG(bc & 0x8000000000000000LL, F_CF);
+            CONDITIONAL_SET_FLAG(XOR2(bc >> 62), F_OF);
+            CONDITIONAL_SET_FLAG(bc & 0x8, F_AF);
             break;
         case d_tst8:
-        	CLEAR_FLAG(F_OF);
-        	CONDITIONAL_SET_FLAG(emu->res.u8 & 0x80, F_SF);
-        	CONDITIONAL_SET_FLAG(emu->res.u8 == 0, F_ZF);
-        	CONDITIONAL_SET_FLAG(PARITY(emu->res.u8 & 0xff), F_PF);
-        	CLEAR_FLAG(F_CF);
+            CLEAR_FLAG(F_OF);
+            CONDITIONAL_SET_FLAG(emu->res.u8 & 0x80, F_SF);
+            CONDITIONAL_SET_FLAG(emu->res.u8 == 0, F_ZF);
+            CONDITIONAL_SET_FLAG(PARITY(emu->res.u8 & 0xff), F_PF);
+            CLEAR_FLAG(F_CF);
             break;
         case d_tst16:
             CLEAR_FLAG(F_OF);
@@ -1005,18 +897,18 @@ void UpdateFlags(x64emu_t *emu)
             CLEAR_FLAG(F_CF);
             break;
         case d_tst32:
-        	CLEAR_FLAG(F_OF);
-        	CONDITIONAL_SET_FLAG(emu->res.u32 & 0x80000000, F_SF);
-        	CONDITIONAL_SET_FLAG(emu->res.u32 == 0, F_ZF);
-        	CONDITIONAL_SET_FLAG(PARITY(emu->res.u32 & 0xff), F_PF);
-        	CLEAR_FLAG(F_CF);
+            CLEAR_FLAG(F_OF);
+            CONDITIONAL_SET_FLAG(emu->res.u32 & 0x80000000, F_SF);
+            CONDITIONAL_SET_FLAG(emu->res.u32 == 0, F_ZF);
+            CONDITIONAL_SET_FLAG(PARITY(emu->res.u32 & 0xff), F_PF);
+            CLEAR_FLAG(F_CF);
             break;
         case d_tst64:
-        	CLEAR_FLAG(F_OF);
-        	CONDITIONAL_SET_FLAG(emu->res.u64 & 0x8000000000000000LL, F_SF);
-        	CONDITIONAL_SET_FLAG(emu->res.u64 == 0, F_ZF);
-        	CONDITIONAL_SET_FLAG(PARITY(emu->res.u64 & 0xff), F_PF);
-        	CLEAR_FLAG(F_CF);
+            CLEAR_FLAG(F_OF);
+            CONDITIONAL_SET_FLAG(emu->res.u64 & 0x8000000000000000LL, F_SF);
+            CONDITIONAL_SET_FLAG(emu->res.u64 == 0, F_ZF);
+            CONDITIONAL_SET_FLAG(PARITY(emu->res.u64 & 0xff), F_PF);
+            CLEAR_FLAG(F_CF);
             break;
         case d_adc8:
             CONDITIONAL_SET_FLAG(emu->res.u16 & 0x100, F_CF);
@@ -1115,25 +1007,25 @@ void UpdateFlags(x64emu_t *emu)
             if(emu->op2.u8 == 1) {
                 CONDITIONAL_SET_FLAG((emu->res.u8 + (emu->res.u8 >> 7)) & 1, F_OF);
             }
-        	CONDITIONAL_SET_FLAG(emu->res.u16 & 0x1, F_CF);
+            CONDITIONAL_SET_FLAG(emu->res.u8 & 0x1, F_CF);
             break;
         case d_rol16:
             if(emu->op2.u16 == 1) {
                 CONDITIONAL_SET_FLAG((emu->res.u16 + (emu->res.u16 >> 15)) & 1, F_OF);
             }
-        	CONDITIONAL_SET_FLAG(emu->res.u16 & 0x1, F_CF);
+            CONDITIONAL_SET_FLAG(emu->res.u16 & 0x1, F_CF);
             break;
         case d_rol32:
             if(emu->op2.u32 == 1) {
                 CONDITIONAL_SET_FLAG((emu->res.u32 + (emu->res.u32 >> 31)) & 1, F_OF);
             }
-        	CONDITIONAL_SET_FLAG(emu->res.u32 & 0x1, F_CF);
+            CONDITIONAL_SET_FLAG(emu->res.u32 & 0x1, F_CF);
             break;
         case d_rol64:
             if(emu->op2.u64 == 1) {
                 CONDITIONAL_SET_FLAG((emu->res.u64 + (emu->res.u64 >> 63)) & 1, F_OF);
             }
-        	CONDITIONAL_SET_FLAG(emu->res.u64 & 0x1, F_CF);
+            CONDITIONAL_SET_FLAG(emu->res.u64 & 0x1, F_CF);
             break;
         case d_ror8:
             if(emu->op2.u8 == 1) {
@@ -1160,14 +1052,6 @@ void UpdateFlags(x64emu_t *emu)
             CONDITIONAL_SET_FLAG(emu->res.u64 & (1L << 63), F_CF);
             break;
 
-        case d_rcl8:
-        case d_rcl16:
-        case d_rcl32:
-        case d_rcl64:
-        case d_rcr8:
-        case d_rcr16:
-        case d_rcr32:
-        case d_rcr64:
         case d_unknown:
             printf_log(LOG_NONE, "Box64: %p trying to evaluate Unknown deferred Flags\n", (void*)R_RIP);
             break;
@@ -1226,7 +1110,7 @@ const char* getAddrFunctionName(uintptr_t addr)
         else if(addr==start)
             sprintf(ret, "%s/%s", ElfName(elf), symbname);
         else
-            sprintf(ret, "%s/%s + %ld", ElfName(elf), symbname, addr - start);
+            sprintf(ret, "%s/%s + 0x%lx", ElfName(elf), symbname, addr - start);
     } else {
         if(elf) {
             sprintf(ret, "%s + 0x%lx", ElfName(elf), addr - (uintptr_t)GetBaseAddress(elf));
@@ -1246,7 +1130,7 @@ int printFunctionAddr(uintptr_t nextaddr, const char* text)
         if(nextaddr==start)
             printf_log(LOG_NONE, " (%s%s:%s)", text, ElfName(FindElfAddress(my_context, nextaddr)), symbname);
         else
-            printf_log(LOG_NONE, " (%s%s:%s + %ld)", text, ElfName(FindElfAddress(my_context, nextaddr)), symbname, nextaddr - start);
+            printf_log(LOG_NONE, " (%s%s:%s + 0x%lx)", text, ElfName(FindElfAddress(my_context, nextaddr)), symbname, nextaddr - start);
         return 1;
     }
     return 0;
@@ -1270,7 +1154,9 @@ void PrintTrace(x64emu_t* emu, uintptr_t ip, int dynarec)
         mutex_lock(&my_context->mutex_trace);
 #ifdef DYNAREC
         if((my_context->trace_tid != tid) || (my_context->trace_dynarec!=dynarec)) {
-            printf_log(LOG_NONE, "Thread %04d| (%s) |\n", tid, dynarec?"dyn":"int");
+            printf_log(LOG_NONE, "Thread %04d| (%s) ", tid, dynarec?"dyn":"int");
+            printFunctionAddr(ip, "here: ");
+            printf_log(LOG_NONE, "\n");
             my_context->trace_tid = tid;
             my_context->trace_dynarec = dynarec;
         }
@@ -1314,7 +1200,7 @@ void PrintTrace(x64emu_t* emu, uintptr_t ip, int dynarec)
             } else if(peek==0x57 && rex.b) {
                 printf_log(LOG_NONE, " => STACK_TOP: %p", *(void**)(R_RSP));
                 printFunctionAddr(ip, "here: ");
-            } else if((peek==0x55 || peek==0x53) && !is32bits) {
+            } else if((peek==0x55 /*|| peek==0x53*/) && !is32bits) {
                 if(!printFunctionAddr(*(uintptr_t*)(R_RSP), " STACK_TOP: "))
                     printf_log(LOG_NONE, " STACK_TOP: %p ", (void*)*(uintptr_t*)(R_RSP));
             } else if((peek==0x55 || peek==0x56) && is32bits) {
@@ -1346,6 +1232,10 @@ void PrintTrace(x64emu_t* emu, uintptr_t ip, int dynarec)
                     uintptr_t nextaddr = *(uintptr_t*)(R_R10 + R_RAX*8);
                     printf_log(LOG_NONE, " => %p", (void*)nextaddr);
                     printFunctionAddr(nextaddr, "=> ");
+                } else if(PK(1)==0xE1 && rex.rex==0x41) {
+                    uintptr_t nextaddr = R_R9;
+                    printf_log(LOG_NONE, " => %p", (void*)nextaddr);
+                    printFunctionAddr(nextaddr, "=> ");
                 }
 
             }
@@ -1604,7 +1494,7 @@ reg64_t* TestEd(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t
 {
     uint8_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
-         return &test->emu->regs[(m&0x07)+(rex.b<<3)];
+        return &test->emu->regs[(m&0x07)+(rex.b<<3)];
     } else {
         reg64_t* ret =  GetECommon(test->emu, addr, rex, m, delta);
         test->memsize = 4<<rex.w;
@@ -1620,7 +1510,7 @@ reg64_t* TestEd4(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t
 {
     uint8_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
-         return &test->emu->regs[(m&0x07)+(rex.b<<3)];
+        return &test->emu->regs[(m&0x07)+(rex.b<<3)];
     } else {
         reg64_t* ret =  GetECommon(test->emu, addr, rex, m, delta);
         test->memsize = 4;
@@ -1633,7 +1523,7 @@ reg64_t* TestEd8(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t
 {
     uint8_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
-         return &test->emu->regs[(m&0x07)+(rex.b<<3)];
+        return &test->emu->regs[(m&0x07)+(rex.b<<3)];
     } else {
         reg64_t* ret =  GetECommon(test->emu, addr, rex, m, delta);
         test->memsize = 8;
@@ -1646,7 +1536,7 @@ reg64_t* TestEdt(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t
 {
     uint8_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
-         return &test->emu->regs[(m&0x07)+(rex.b<<3)];
+        return &test->emu->regs[(m&0x07)+(rex.b<<3)];
     } else {
         reg64_t* ret =  GetECommon(test->emu, addr, rex, m, delta);
         test->memsize = 4;
@@ -1659,7 +1549,7 @@ reg64_t* TestEd8xw(x64test_t *test, int w, uintptr_t* addr, rex_t rex, uint8_t v
 {
     uint8_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
-         return &test->emu->regs[(m&0x07)+(rex.b<<3)];
+        return &test->emu->regs[(m&0x07)+(rex.b<<3)];
     } else {
         reg64_t* ret =  GetECommon(test->emu, addr, rex, m, delta);
         test->memsize = 8<<w;
@@ -1670,6 +1560,22 @@ reg64_t* TestEd8xw(x64test_t *test, int w, uintptr_t* addr, rex_t rex, uint8_t v
         return (reg64_t*)test->mem;
     }
 }
+reg64_t* TestEw(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t delta)
+{
+    uint8_t m = v&0xC7;    // filter Ed
+    if(m>=0xC0) {
+        return &test->emu->regs[(m&0x07)+(rex.b<<3)];
+    } else {
+        reg64_t* ret =  GetECommon(test->emu, addr, rex, m, delta);
+        test->memsize = rex.w?8:2;
+        test->memaddr = (uintptr_t)ret;
+        if(rex.w)
+            *(uint64_t*)test->mem = ret->q[0];
+        else
+            *(uint16_t*)test->mem = ret->word[0];
+        return (reg64_t*)test->mem;
+    }
+}
 
 uintptr_t GetEA(x64emu_t *emu, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t delta)
 {
@@ -1679,6 +1585,14 @@ uintptr_t GetEA(x64emu_t *emu, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t de
     } else return (uintptr_t)GetECommon(emu, addr, rex, m, delta);
 }
 
+uintptr_t GetEA32(x64emu_t *emu, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t delta)
+{
+    uint8_t m = v&0xC7;    // filter Ed
+    if(m>=0xC0) {
+         return (uintptr_t)&emu->regs[(m&0x07)+(rex.b<<3)];
+    } else return (uintptr_t)GetECommon32O(emu, addr, rex, m, delta, 0);
+}
+
 reg64_t* GetEdO(x64emu_t *emu, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t delta, uintptr_t offset)
 {
     uint8_t m = v&0xC7;    // filter Ed
@@ -1691,7 +1605,7 @@ reg64_t* TestEdO(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t
 {
     uint8_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
-         return &test->emu->regs[(m&0x07)+(rex.b<<3)];
+        return &test->emu->regs[(m&0x07)+(rex.b<<3)];
     } else {
         reg64_t* ret =  GetECommonO(test->emu, addr, rex, m, delta, offset);
         test->memsize = 4<<rex.w;
@@ -1716,7 +1630,7 @@ reg64_t* TestEd32O(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uint8
 {
     uint8_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
-         return &test->emu->regs[(m&0x07)+(rex.b<<3)];
+        return &test->emu->regs[(m&0x07)+(rex.b<<3)];
     } else {
         reg64_t* ret =  GetECommon32O(test->emu, addr, rex, m, delta, offset);
         test->memsize = 4<<rex.w;
@@ -1800,7 +1714,7 @@ reg64_t* TestEw16(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v)
 
     uint8_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
-         return &emu->regs[(m&0x07)];
+        return &emu->regs[(m&0x07)];
     } else {
         uintptr_t base = 0;
         switch(m&7) {
@@ -1855,6 +1769,35 @@ reg64_t* GetEw16off(x64emu_t *emu, uintptr_t* addr, rex_t rex, uint8_t v, uintpt
     }
 }
 
+reg64_t* GetEd16off(x64emu_t *emu, uintptr_t* addr, rex_t rex, uint8_t v, uintptr_t offset)
+{
+    (void)rex;
+
+    uint32_t m = v&0xC7;    // filter Ed
+    if(m>=0xC0) {
+         return &emu->regs[(m&0x07)];
+    } else {
+        uint32_t base = 0;
+        switch(m&7) {
+            case 0: base = R_BX+R_SI; break;
+            case 1: base = R_BX+R_DI; break;
+            case 2: base = R_BP+R_SI; break;
+            case 3: base = R_BP+R_DI; break;
+            case 4: base =      R_SI; break;
+            case 5: base =      R_DI; break;
+            case 6: base =      R_BP; break;
+            case 7: base =      R_BX; break;
+        }
+        switch((m>>6)&3) {
+            case 0: if((m&7)==6) base = F16S(addr); break;
+            case 1: base += F8S(addr); break;
+            case 2: base += F16S(addr); break;
+            // case 3 is C0..C7, already dealt with
+        }
+        return (reg64_t*)(base+offset);
+    }
+}
+
 reg64_t* TestEw16off(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uintptr_t offset)
 {
     (void)rex;
@@ -1862,7 +1805,7 @@ reg64_t* TestEw16off(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uin
 
     uint32_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
-         return &emu->regs[(m&0x07)];
+        return &emu->regs[(m&0x07)];
     } else {
         uint32_t base = 0;
         switch(m&7) {
@@ -1888,6 +1831,39 @@ reg64_t* TestEw16off(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uin
     }
 }
 
+reg64_t* TestEd16off(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uintptr_t offset)
+{
+    (void)rex;
+    x64emu_t* emu = test->emu;
+
+    uint32_t m = v&0xC7;    // filter Ed
+    if(m>=0xC0) {
+        return &emu->regs[(m&0x07)];
+    } else {
+        uint32_t base = 0;
+        switch(m&7) {
+            case 0: base = R_BX+R_SI; break;
+            case 1: base = R_BX+R_DI; break;
+            case 2: base = R_BP+R_SI; break;
+            case 3: base = R_BP+R_DI; break;
+            case 4: base =      R_SI; break;
+            case 5: base =      R_DI; break;
+            case 6: base =      R_BP; break;
+            case 7: base =      R_BX; break;
+        }
+        switch((m>>6)&3) {
+            case 0: if((m&7)==6) base = F16S(addr); break;
+            case 1: base += F8S(addr); break;
+            case 2: base += F16S(addr); break;
+            // case 3 is C0..C7, already dealt with
+        }
+        test->memsize = 4;
+        *(uint32_t*)test->mem = *(uint32_t*)(base+offset);
+        test->memaddr = (uintptr_t)(base+offset);
+        return (reg64_t*)test->mem;
+    }
+}
+
 mmx87_regs_t* GetEm(x64emu_t *emu, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t delta)
 {
     uint8_t m = v&0xC7;    // filter Ed
@@ -1900,7 +1876,7 @@ mmx87_regs_t* TestEm(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uin
 {
     uint8_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
-         return &test->emu->mmx[m&0x07];
+        return &test->emu->mmx[m&0x07];
     } else {
         mmx87_regs_t* ret = (mmx87_regs_t*)GetECommon(test->emu, addr, rex, m, delta);
         test->memsize = 8;
@@ -1918,21 +1894,34 @@ sse_regs_t* GetEx(x64emu_t *emu, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t
     } else return (sse_regs_t*)GetECommon(emu, addr, rex, m, delta);
 }
 
-sse_regs_t* TestEx(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t delta)
+sse_regs_t* TestEx(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t delta, int sz)
 {
     uint8_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
         test->memsize=0;
-         return &test->emu->xmm[(m&0x07)+(rex.b<<3)];
+        return &test->emu->xmm[(m&0x07)+(rex.b<<3)];
     } else {
         sse_regs_t* ret = (sse_regs_t*)GetECommon(test->emu, addr, rex, m, delta);
-        test->memsize = 16;
-        ((uint64_t*)test->mem)[0] = ret->q[0];
-        ((uint64_t*)test->mem)[1] = ret->q[1];
+        test->memsize = sz;
+        memcpy(test->mem, ret, sz);
         test->memaddr = (uintptr_t)ret;
         return (sse_regs_t*)test->mem;
     }
 }
+sse_regs_t* TestEy(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v)
+{
+    uint8_t m = v&0xC7;    // filter Ed
+    if(m>=0xC0) {
+        test->memsize=0;
+        return &test->emu->ymm[(m&0x07)+(rex.b<<3)];
+    } else {
+        sse_regs_t* ret = (sse_regs_t*)(test->memaddr+16);
+        test->memsize += 16;
+        ((uint64_t*)test->mem)[2] = ret->q[0];
+        ((uint64_t*)test->mem)[3] = ret->q[1];
+        return (sse_regs_t*)&test->mem[16];
+    }
+}
 
 sse_regs_t* GetExO(x64emu_t *emu, uintptr_t* addr, rex_t rex, uint8_t v, uint8_t delta, uintptr_t offset)
 {
@@ -1946,7 +1935,7 @@ sse_regs_t* TestExO(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, uint
 {
     uint8_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
-         return &test->emu->xmm[(m&0x07)+(rex.b<<3)];
+        return &test->emu->xmm[(m&0x07)+(rex.b<<3)];
     } else {
         sse_regs_t* ret = (sse_regs_t*)GetECommonO(test->emu, addr, rex, m, delta, offset);
         test->memsize = 16;
@@ -1969,7 +1958,7 @@ sse_regs_t* TestEx32O(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v, ui
 {
     uint8_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
-         return &test->emu->xmm[(m&0x07)+(rex.b<<3)];
+        return &test->emu->xmm[(m&0x07)+(rex.b<<3)];
     } else {
         sse_regs_t* ret = (sse_regs_t*)GetECommon32O(test->emu, addr, rex, m, delta, offset);
         test->memsize = 16;
@@ -1992,7 +1981,7 @@ mmx87_regs_t* TestEm32O(x64test_t *test, uintptr_t* addr, rex_t rex, uint8_t v,
 {
     uint8_t m = v&0xC7;    // filter Ed
     if(m>=0xC0) {
-         return &test->emu->mmx[(m&0x07)];
+        return &test->emu->mmx[(m&0x07)];
     } else {
         mmx87_regs_t* ret = (mmx87_regs_t*)GetECommon32O(test->emu, addr, rex, m, delta, offset);
         test->memsize = 8;
@@ -2029,3 +2018,9 @@ sse_regs_t* GetGx(x64emu_t *emu, uintptr_t* addr, rex_t rex, uint8_t v)
     uint8_t m = (v&0x38)>>3;
     return &emu->xmm[(m&7)+(rex.r<<3)];
 }
+
+sse_regs_t* GetGy(x64emu_t *emu, uintptr_t* addr, rex_t rex, uint8_t v)
+{
+    uint8_t m = (v&0x38)>>3;
+    return &emu->ymm[(m&7)+(rex.r<<3)];
+}
