@@ -36,6 +36,7 @@ uintptr_t RunD8(x64emu_t *emu, rex_t rex, uintptr_t addr)
     #endif
 
     nextop = F8;
+    if(MODREG)
     switch (nextop) {
 
         case 0xC0:
@@ -120,6 +121,8 @@ uintptr_t RunD8(x64emu_t *emu, rex_t rex, uintptr_t addr)
             ST0.d = ST(nextop&7).d / ST0.d;
             break;
         default:
+            return 0;
+    } else
         switch((nextop>>3)&7) {
             case 0:         /* FADD ST0, float */
                 GETE4(0);
@@ -157,6 +160,5 @@ uintptr_t RunD8(x64emu_t *emu, rex_t rex, uintptr_t addr)
             default:
                 return 0;
         }
-    }
    return addr;
 }
\ No newline at end of file
