@@ -146,10 +146,17 @@ int my_arch_prctl(x64emu_t *emu, int code, void* addr)
 {
     printf_log(LOG_DEBUG, "%04d| arch_prctl(%s, %p) (RSP=%p, FS=0x%x, GS=0x%x)\n", GetTID(), arch_prctl_param(code), addr,(void*)R_RSP, emu->segs[_FS], emu->segs[_GS]);
 
-    #define ARCH_SET_GS          0x1001
-    #define ARCH_SET_FS          0x1002
-    #define ARCH_GET_FS          0x1003
-    #define ARCH_GET_GS          0x1004
+    #define ARCH_SET_GS                 0x1001
+    #define ARCH_SET_FS                 0x1002
+    #define ARCH_GET_FS                 0x1003
+    #define ARCH_GET_GS                 0x1004
+    #define ARCH_GET_CPUID              0x1011
+    #define ARCH_SET_CPUID              0x1012
+    #define ARCH_GET_XCOMP_SUPP         0x1021
+    #define ARCH_GET_XCOMP_PERM         0x1022
+    #define ARCH_REQ_XCOMP_PERM         0x1023
+    #define ARCH_GET_XCOMP_GUEST_PERM   0x1024
+    #define ARCH_REQ_XCOMP_GUEST_PERM   0x1025
     int seg = 0;
     int idx = 0;
     errno = 0;
@@ -198,6 +205,12 @@ int my_arch_prctl(x64emu_t *emu, int code, void* addr)
                 pthread_setspecific(my_context->segtls[idx].key, addr);
             ResetSegmentsCache(emu);
             return 0;
+        case ARCH_GET_XCOMP_SUPP:
+        case ARCH_GET_XCOMP_PERM:
+        case ARCH_REQ_XCOMP_PERM:
+            // unsupported, this is for AVX512...
+            errno = ENOSYS;
+            return -1;
     }
     // other are unsupported
     printf_log(LOG_INFO, "warning, call to unsupported arch_prctl(0x%x, %p)\n", code, addr);
@@ -225,9 +238,11 @@ int my_arch_prctl(x64emu_t *emu, int code, void* addr)
  uintptr_t  ssp_base        0x78
  .... padding ....          0x200?
 */
+#define NELFSIZE_MASK 0x3FF
+#define NELFSIZE (NELFSIZE_MASK+1)
 static int sizeDTS(box64context_t* context)
 {
-    return ((context->elfsize+0xff)&~0xff)*16;
+    return ((context->elfsize+NELFSIZE_MASK)&~NELFSIZE_MASK)*16;
 }
 static int sizeTLSData(int s)
 {
@@ -281,9 +296,9 @@ static void* resizeTLSData(box64context_t *context, void* oldptr)
 {
         mutex_lock(&context->mutex_tls);
         tlsdatasize_t* oldata = (tlsdatasize_t*)oldptr;
-        if(sizeTLSData(oldata->tlssize)!=sizeTLSData(context->tlssize) || (oldata->n_elfs/0xff)!=(context->elfsize/0xff)) {
+        if(sizeTLSData(oldata->tlssize)!=sizeTLSData(context->tlssize) || (oldata->n_elfs/NELFSIZE)!=(context->elfsize/NELFSIZE)) {
             if(sizeTLSData(oldata->tlssize)) {
-                printf_log(LOG_INFO, "Warning, resizing of TLS occurred! size: %d->%d / n_elfs: %d->%d\n", sizeTLSData(oldata->tlssize), sizeTLSData(context->tlssize), 1+(oldata->n_elfs/0xff), 1+(context->elfsize/0xff));
+                printf_log(LOG_INFO, "Warning, resizing of TLS occurred! size: %d->%d / n_elfs: %d->%d\n", sizeTLSData(oldata->tlssize), sizeTLSData(context->tlssize), 1+(oldata->n_elfs/NELFSIZE), 1+(context->elfsize/NELFSIZE));
             }
             tlsdatasize_t *data = setupTLSData(context);
             // copy the relevent old part, in case something changed
