@@ -10,24 +10,29 @@ extern int box64_dynarec;
 extern uintptr_t box64_pagesize;
 extern uintptr_t box64_load_addr;
 extern int box64_dynarec_test;
+extern int box64_maxcpu;
+extern int box64_mmap32;
+extern int box64_ignoreint3;
+extern int box64_rdtsc;
+extern int box64_rdtsc_1ghz;
+extern uint8_t box64_rdtsc_shift;
 #ifdef DYNAREC
 extern int box64_dynarec_dump;
 extern int box64_dynarec_trace;
 extern int box64_dynarec_forced;
 extern uintptr_t box64_nodynarec_start, box64_nodynarec_end;
+extern uintptr_t box64_dynarec_test_start, box64_dynarec_test_end;
 extern int box64_dynarec_bigblock;
 extern int box64_dynarec_forward;
 extern int box64_dynarec_strongmem;
 extern int box64_dynarec_fastnan;
 extern int box64_dynarec_fastround;
 extern int box64_dynarec_x87double;
+extern int box64_dynarec_div0;
 extern int box64_dynarec_safeflags;
 extern int box64_dynarec_callret;
 extern int box64_dynarec_bleeding_edge;
-extern int box64_dynarec_jvm;
 extern int box64_dynarec_tbb;
-extern int box64_dynarec_hotpage;
-extern int box64_dynarec_fastpage;
 extern int box64_dynarec_wait;
 extern int box64_dynarec_missing;
 extern int box64_dynarec_aligned_atomics;
@@ -39,13 +44,17 @@ extern int arm64_crc32;
 extern int arm64_atomics;
 extern int arm64_sha1;
 extern int arm64_sha2;
+extern int arm64_uscat;
 extern int arm64_flagm;
 extern int arm64_flagm2;
+extern int arm64_frintts;
+extern int arm64_rndr;
 #elif defined(RV64)
 extern int rv64_zba;
 extern int rv64_zbb;
 extern int rv64_zbc;
 extern int rv64_zbs;
+extern int rv64_vector;
 extern int rv64_xtheadba;
 extern int rv64_xtheadbb;
 extern int rv64_xtheadbs;
@@ -55,9 +64,16 @@ extern int rv64_xtheadmempair;
 extern int rv64_xtheadfmemidx;
 extern int rv64_xtheadmac;
 extern int rv64_xtheadfmv;
+#elif defined(LA64)
+extern int la64_lbt;
+extern int la64_lam_bh;
+extern int la64_lamcas;
+extern int la64_scq;
 #endif
 #endif
 extern int box64_libcef;
+extern int box64_jvm;
+extern int box64_unityplayer;
 extern int box64_sdl2_jguid;
 extern int dlsym_error;    // log dlsym error
 extern int cycle_log;      // if using rolling logs
@@ -73,11 +89,17 @@ extern uint64_t start_cnt;
 #endif
 extern int box64_nosandbox;
 extern int box64_inprocessgpu;
+extern int box64_cefdisablegpu;
+extern int box64_cefdisablegpucompositor;
+extern int box64_maxcpu_immutable;
 extern int box64_malloc_hack;
 extern int box64_dummy_crashhandler;
 extern int box64_sse_flushto0;
 extern int box64_x87_no80bits;
 extern int box64_sync_rounding;
+extern int box64_sse42;
+extern int box64_avx;
+extern int box64_avx2;
 extern int allow_missing_libs;
 extern int box64_mapclean;
 extern int box64_prefer_wrapped;
@@ -122,8 +144,10 @@ void printf_ftrace(const char* fmt, ...);
 #define EXPORTDYN
 #endif
 
+#ifndef STATICBUILD
 void init_malloc_hook(void);
-#ifdef ANDROID
+#endif
+#if defined(ANDROID) || defined(STATICBUILD)
 #define box_malloc      malloc
 #define box_realloc     realloc
 #define box_calloc      calloc
