@@ -3,6 +3,7 @@
 
 typedef struct x64emu_s x64emu_t;
 typedef struct box64context_s box64context_t;
+typedef struct elfheader_s elfheader_t;
 
 x64emu_t *NewX64Emu(box64context_t *context, uintptr_t start, uintptr_t stack, int stacksize, int ownstack);
 x64emu_t *NewX64EmuFromStack(x64emu_t* emu, box64context_t *context, uintptr_t start, uintptr_t stack, int stacksize, int ownstack);
@@ -14,7 +15,6 @@ void CopyEmu(x64emu_t *newemu, const x64emu_t* emu);
 void SetTraceEmu(uintptr_t trace_start, uintptr_t trace_end);
 
 box64context_t* GetEmuContext(x64emu_t* emu);
-uint32_t* GetParityTab(void);
 
 uint32_t GetEAX(x64emu_t *emu);
 uint64_t GetRAX(x64emu_t *emu);
@@ -45,13 +45,14 @@ const char* DumpCPURegs(x64emu_t* emu, uintptr_t ip, int is32bits);
 
 void StopEmu(x64emu_t* emu, const char* reason, int is32bits);
 void EmuCall(x64emu_t* emu, uintptr_t addr);
-void AddCleanup(x64emu_t *emu, void *p, void* dso_handle);
-void AddCleanup1Arg(x64emu_t *emu, void *p, void* a, void* dso_handle);
-void CallCleanup(x64emu_t *emu, void* p);
+void AddCleanup(x64emu_t *emu, void *p);
+void AddCleanup1Arg(x64emu_t *emu, void *p, void* a, elfheader_t* h);
+void CallCleanup(x64emu_t *emu, elfheader_t* h);
 void CallAllCleanup(x64emu_t *emu);
 void UnimpOpcode(x64emu_t* emu, int is32bits);
 
 uint64_t ReadTSC(x64emu_t* emu);
+uint64_t ReadTSCFrequency(x64emu_t* emu);
 
 double FromLD(void* ld);        // long double (80bits pointer) -> double
 long double LD2localLD(void* ld);        // long double (80bits pointer) -> long double (80 or 128bits)
