@@ -33,27 +33,31 @@
 #define GO(P, N) int wrapped##N##_init(library_t* lib, box64context_t *box64); \
                  void wrapped##N##_fini(library_t* lib);
 
+#ifdef STATICBUILD
+#include "library_list_static.h"
+#else
 #include "library_list.h"
+#endif
 
 #undef GO
 
 #define GO(P, N) {P, wrapped##N##_init, wrapped##N##_fini},
 wrappedlib_t wrappedlibs[] = {
+#ifdef STATICBUILD
+#include "library_list_static.h"
+#else
 #include "library_list.h"
+#endif
 };
 #undef GO
 
-typedef struct bridged_s {
-    char*       name;
-    uintptr_t   start;
-    uintptr_t   end;
-} bridged_t;
-
-KHASH_MAP_INIT_STR(bridgemap, bridged_t)
-
 KHASH_MAP_IMPL_STR(symbolmap, symbol1_t)
 KHASH_MAP_IMPL_STR(symbol2map, symbol2_t)
+#ifdef STATICBUILD
+KHASH_MAP_IMPL_STR(datamap, datamap_t)
+#else
 KHASH_MAP_IMPL_STR(datamap, uint64_t)
+#endif
 
 char* Path2Name(const char* path)
 {
@@ -98,11 +102,15 @@ void WrappedLib_CommonInit(library_t *lib) {
     lib->w.mydatamap = kh_init(datamap);
 }
 
+static void initDummyLib(library_t *lib);
+
 void EmuLib_Fini(library_t* lib)
 {
 }
 void WrappedLib_FinishFini(library_t* lib)
 {
+    if(lib->type!=LIB_WRAPPED)
+        return;
     if(lib->w.lib)
         dlclose(lib->w.lib);
     lib->w.lib = NULL;
@@ -115,14 +123,17 @@ void WrappedLib_FinishFini(library_t* lib)
     }
     FreeBridge(&lib->w.bridge);
 }
+void DummyLib_Fini(library_t* lib)
+{
+}
 
-int WrappedLib_GetWeak(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int version, const char* vername, int local, const char* defver)
+int WrappedLib_GetWeak(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int* version, const char** vername, int local, int* veropt, void** elfsym)
 {
     // ignoring asked size on wrapped libs
     uintptr_t addr = 0;
     uintptr_t size = 0;
     int wk = 0;
-    if (!getSymbolInMaps(lib, name, 0, &addr, &size, &wk, version, vername, local)) {
+    if (!getSymbolInMaps(lib, name, 0, &addr, &size, &wk, *version, *vername, local, *veropt)) {
         return 0;
     }
     if(!addr && !size)
@@ -132,29 +143,37 @@ int WrappedLib_GetWeak(library_t* lib, const char* name, uintptr_t *offs, uintpt
     *offs = addr;
     *sz = size;
     *weak = wk;
+    if(elfsym) *elfsym = NULL;
     return 1;
 }
-int EmuLib_GetWeak(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int*weak, int version, const char* vername, int local, const char* defver)
+int EmuLib_GetWeak(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int* version, const char** vername, int local, int* veropt, void** elfsym)
 {
     // symbols...
     uintptr_t start, end;
     // weak symbols...
-    if(GetSizedSymbolStartEnd(GetWeakSymbols(lib->e.elf), name, &start, &end, asked_size, version, vername, local, defver))
+    void* sym;
+    if((sym = ElfGetWeakSymbolStartEnd(lib->e.elf, &start, &end, name, version, vername, local, veropt)))
     {
         *offs = start;
         *sz = end-start;
         *weak = 1;
+        if(elfsym) *elfsym = sym;
         return 1;
     }
     return 0;
 }
-int WrappedLib_GetGlobal(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int version, const char* vername, int local, const char* defver)
+int DummyLib_GetWeak(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int* version, const char** vername, int local, int* veropt, void** elfsym)
+{
+    return 0;
+}
+
+int WrappedLib_GetGlobal(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int* version, const char** vername, int local, int* veropt, void** elfsym)
 {
     // ignoring asked size on wrapped libs
     uintptr_t addr = 0;
     uintptr_t size = 0;
     int wk = 0;
-    if (!getSymbolInMaps(lib, name, 1, &addr, &size, &wk, version, vername, local)) {
+    if (!getSymbolInMaps(lib, name, 1, &addr, &size, &wk, *version, *vername, local, *veropt)) {
         return 0;
     }
     if(!addr && !size)
@@ -164,35 +183,49 @@ int WrappedLib_GetGlobal(library_t* lib, const char* name, uintptr_t *offs, uint
     *offs = addr;
     *sz = size;
     *weak = 0;
+    if(elfsym) *elfsym = NULL;
     return 1;
 }
-int EmuLib_GetGlobal(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int version, const char* vername, int local, const char* defver)
+int EmuLib_GetGlobal(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int* version, const char** vername, int local, int* veropt, void** elfsym)
 {
     uintptr_t start, end;
-    if(GetSizedSymbolStartEnd(GetMapSymbols(lib->e.elf), name, &start, &end, asked_size, version, vername, local, defver))
+    void* sym;
+    if((sym = ElfGetGlobalSymbolStartEnd(lib->e.elf, &start, &end, name, version, vername, local, veropt)))
     {
         *offs = start;
         *sz = end-start;
         *weak = 0;
+        if(elfsym) *elfsym = sym;
         return 1;
     }
     return 0;
 }
-int EmuLib_GetLocal(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int version, const char* vername, int local, const char* defver)
+int DummyLib_GetGlobal(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int* version, const char** vername, int local, int* veropt, void** elfsym)
+{
+    return 0;
+}
+int EmuLib_GetLocal(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int* version, const char** vername, int local, int* veropt, void** elfsym)
 {
     // ignoring asked size on wrapped libs
     uintptr_t start, end;
-    if(GetSymbolStartEnd(GetLocalSymbols(lib->e.elf), name, &start, &end, version, vername, local, defver))
+    void* sym;
+    if((sym = ElfGetLocalSymbolStartEnd(lib->e.elf, &start, &end, name, version, vername, local, veropt)))
     {
         *offs = start;
         *sz = end-start;
         *weak = 0;
+        if(elfsym) *elfsym = sym;
         return 1;
     }
     return 0;
 }
 
-int WrappedLib_GetLocal(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int version, const char* vername, int local, const char* defver)
+int WrappedLib_GetLocal(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int* version, const char** vername, int local, int* veropt, void** elfsym)
+{
+    (void)lib; (void)name; (void)offs; (void)sz; (void)version; (void)vername; (void)local;
+    return 0;
+}
+int DummyLib_GetLocal(library_t* lib, const char* name, uintptr_t *offs, uintptr_t *sz, size_t asked_size, int* weak, int* version, const char** vername, int local, int* veropt, void** elfsym)
 {
     (void)lib; (void)name; (void)offs; (void)sz; (void)version; (void)vername; (void)local;
     return 0;
@@ -217,7 +250,7 @@ static void initWrappedLib(library_t *lib, box64context_t* context) {
             lib->type = LIB_WRAPPED;
             lib->w.refcnt = 1;
             // Call librarian to load all dependant elf
-            if(AddNeededLib(context->maplib, 0, 0, lib->w.needed, NULL, context, thread_get_emu())) {
+            if(AddNeededLib(context->maplib, 0, 0, 0, lib->w.needed, NULL, context, thread_get_emu())) {
                 printf_log(LOG_NONE, "Error: loading a needed libs in elf %s\n", lib->name);
                 return;
             }
@@ -301,16 +334,21 @@ static int loadEmulatedLib(const char* libname, library_t *lib, box64context_t*
             box64_dynarec_bigblock = 0;
             box64_dynarec_strongmem = 1;
         }
-        if(libname && box64_dynarec_jvm && strstr(libname, "libjvm.so")) {
-            printf_dump(LOG_INFO, "libjvm detected, disable Dynarec BigBlock and enable Dynarec StrongMem\n");
-            box64_dynarec_bigblock = 0;
-            box64_dynarec_strongmem = 1;
-        }
         if(libname && box64_dynarec_tbb && strstr(libname, "libtbb.so")) {
             printf_dump(LOG_INFO, "libtbb detected, enable Dynarec StrongMem\n");
             box64_dynarec_strongmem = 3;
         }
         #endif
+        if(libname && box64_jvm && strstr(libname, "libjvm.so")) {
+            #ifdef DYNAREC
+            printf_dump(LOG_INFO, "libjvm detected, disable Dynarec BigBlock and enable Dynarec StrongMem, hide SSE 4.2\n");
+            box64_dynarec_bigblock = 0;
+            box64_dynarec_strongmem = 1;
+            #else
+            printf_dump(LOG_INFO, "libjvm detected, hide SSE 4.2\n");
+            #endif
+            box64_sse42 = 0;
+        }
         if(libname && box64_libcef && strstr(libname, "libcef.so")) {
             printf_dump(LOG_INFO, "libcef detected, using malloc_hack_2\n");
             box64_malloc_hack = 2;
@@ -346,28 +384,65 @@ static void initEmulatedLib(const char* path, library_t *lib, box64context_t* co
         }
 }
 
+static void initDummyLib(library_t *lib)
+{
+    memset(lib, 0, sizeof(library_t));
+    lib->fini = DummyLib_Fini;
+    lib->getglobal = DummyLib_GetGlobal;
+    lib->getweak = DummyLib_GetWeak;
+    lib->getlocal = DummyLib_GetLocal;
+    lib->name = NULL;
+    lib->path = NULL;
+    lib->type = LIB_UNNKNOW;
+}
+
 static const char* essential_libs[] = {
     "libc.so.6", "libpthread.so.0", "librt.so.1", "libGL.so.1", "libGL.so", "libX11.so.6", 
-    "libasound.so.2", "libdl.so.2", "libm.so.6",
+    "libasound.so.2", "libdl.so.2", "libm.so.6", "libbsd.so.0", "libutil.so.1", "libresolv.so.2", "libresolv.so", 
     "libXxf86vm.so.1", "libXinerama.so.1", "libXrandr.so.2", "libXext.so.6", "libXfixes.so.3", "libXcursor.so.1",
     "libXrender.so.1", "libXft.so.2", "libXi.so.6", "libXss.so.1", "libXpm.so.4", "libXau.so.6", "libXdmcp.so.6",
     "libX11-xcb.so.1", "libxcb.so.1", "libxcb-xfixes.so.0", "libxcb-shape.so.0", "libxcb-shm.so.0", "libxcb-randr.so.0",
     "libxcb-image.so.0", "libxcb-keysyms.so.1", "libxcb-xtest.so.0", "libxcb-glx.so.0", "libxcb-dri2.so.0", "libxcb-dri3.so.0",
     "libXtst.so.6", "libXt.so.6", "libXcomposite.so.1", "libXdamage.so.1", "libXmu.so.6", "libxkbcommon.so.0", 
     "libxkbcommon-x11.so.0", "libpulse-simple.so.0", "libpulse.so.0", "libvulkan.so.1", "libvulkan.so",
-    "ld-linux-x86-64.so.2", "crashhandler.so", "libtcmalloc_minimal.so.0", "libtcmalloc_minimal.so.4"
+    "ld-linux-x86-64.so.2", "crashhandler.so", "libtcmalloc_minimal.so.0", "libtcmalloc_minimal.so.4", "libanl.so.1"
 };
 static int isEssentialLib(const char* name) {
-    for (int i=0; i<sizeof(essential_libs)/sizeof(essential_libs[0]); ++i)
+    for (unsigned int i=0; i<sizeof(essential_libs)/sizeof(essential_libs[0]); ++i)
         if(!strcmp(name, essential_libs[i]))
             return 1;
     return 0;
 }
 
+#define NLIBS       16
+typedef struct lib_brick_s {
+    library_t           libs[NLIBS];
+    struct lib_brick_s* next;
+} lib_brick_t;
+
+static lib_brick_t* lib_head = NULL;
+static lib_brick_t* cur_brick = NULL;
+static size_t cur_lib = 0;
+static size_t lib_cap = 0;
+
 library_t *NewLibrary(const char* path, box64context_t* context, elfheader_t* verneeded)
 {
     printf_dump(LOG_DEBUG, "Trying to load \"%s\"\n", path);
-    library_t *lib = (library_t*)box_calloc(1, sizeof(library_t));
+    //library_t *lib = (library_t*)box_calloc(1, sizeof(library_t));
+    if(cur_lib==lib_cap) {
+        lib_brick_t *new_brick = calloc(1, sizeof(lib_brick_t));
+        if(!lib_head)
+            cur_brick = lib_head = new_brick;
+        else {
+            cur_brick->next = new_brick;
+            cur_brick = new_brick;
+        }
+        lib_cap += NLIBS;
+    }
+    library_t *lib = &cur_brick->libs[cur_lib%NLIBS];
+    ++cur_lib;
+    memset(lib, 0, sizeof(library_t));
+
     lib->path = box_realpath(path, NULL);
     if(!lib->path)
         lib->path = box_strdup(path);
@@ -382,7 +457,8 @@ library_t *NewLibrary(const char* path, box64context_t* context, elfheader_t* ve
         if(strstr(lib->name, "libpulse.so")==lib->name || strstr(lib->name, "libpulse-simple.so")==lib->name) {
             box_free(lib->name);
             box_free(lib->path);
-            box_free(lib);
+            initDummyLib(lib);
+            //box_free(lib);
             return NULL;
         }
     }
@@ -390,7 +466,8 @@ library_t *NewLibrary(const char* path, box64context_t* context, elfheader_t* ve
         if(strstr(lib->name, "libvulkan.so")==lib->name) {
             box_free(lib->name);
             box_free(lib->path);
-            box_free(lib);
+            initDummyLib(lib);
+            //box_free(lib);
             return NULL;
         }
     }
@@ -401,6 +478,12 @@ library_t *NewLibrary(const char* path, box64context_t* context, elfheader_t* ve
     int precise = (!box64_prefer_wrapped && !essential && path && strchr(path, '/'))?1:0;
     if(!notwrapped && precise && strstr(path, "libtcmalloc_minimal.so"))
         precise = 0;    // allow native version for tcmalloc_minimum
+    /*
+    if(!notwrapped && precise && strstr(path, "libEGL.so"))
+        precise = 0;    // allow native version for EGL
+    if(!notwrapped && precise && strstr(path, "libGLESv2.so"))
+        precise = 0;    // allow native version for GLESv2
+    */
     // check if name is libSDL_sound-1.0.so.1 but with SDL2 loaded, then try emulated first...
     if(!notwrapped && !strcmp(lib->name, "libSDL_sound-1.0.so.1") && my_context->sdl2lib)
         notwrapped = 1;
@@ -419,12 +502,10 @@ library_t *NewLibrary(const char* path, box64context_t* context, elfheader_t* ve
     {
         box_free(lib->name);
         box_free(lib->path);
-        box_free(lib);
+        initDummyLib(lib);
+        //box_free(lib);
         return NULL;
     }
-    lib->gbridgemap = kh_init(bridgemap);
-    lib->wbridgemap = kh_init(bridgemap);
-    lib->lbridgemap = kh_init(bridgemap);
 
     return lib;
 }
@@ -435,37 +516,40 @@ int AddSymbolsLibrary(lib_t *maplib, library_t* lib, x64emu_t* emu)
     if(lib->type==LIB_EMULATED) {
         elfheader_t *elf_header = lib->e.elf;
         // add symbols
-        AddSymbols(maplib, GetMapSymbols(lib->e.elf), GetWeakSymbols(lib->e.elf), GetLocalSymbols(lib->e.elf), elf_header);
+        AddSymbols(maplib, elf_header);
     }
     return 0;
 }
-int FinalizeLibrary(library_t* lib, lib_t* local_maplib, int bindnow, x64emu_t* emu)
+int FinalizeLibrary(library_t* lib, lib_t* local_maplib, int bindnow, int deepbind, x64emu_t* emu)
 {
     if(!lib)
         return 0;
+    lib->deepbind = deepbind;
     if(lib->type==LIB_EMULATED) {
         if(lib->e.finalized)
             return 0;
         lib->e.finalized = 1;
         elfheader_t *elf_header = my_context->elfs[lib->e.elf_index];
         // finalize relocations
-        if(RelocateElf(my_context->maplib, local_maplib, bindnow, elf_header)) {
+        if(RelocateElf(my_context->maplib, local_maplib, bindnow, deepbind, elf_header)) {
             printf_log(LOG_NONE, "Error: relocating symbols in elf %s\n", lib->name);
             return 1;
         }
-        if(RelocateElfPlt(my_context->maplib, local_maplib, bindnow, elf_header)) {
+        if(RelocateElfPlt(my_context->maplib, local_maplib, bindnow, deepbind, elf_header)) {
             printf_log(LOG_NONE, "Error: relocating Plt symbols in elf %s\n", lib->name);
             return 1;
         }
 #ifdef HAVE_TRACE
         if(trace_func) {
             int weak;
-            if (GetGlobalSymbolStartEnd(local_maplib, trace_func, &trace_start, &trace_end, elf_header, -1, NULL, NULL, NULL)) {
+            int ver = -1, veropt = 0;
+            const char* vername = NULL;
+            if (GetGlobalSymbolStartEnd(local_maplib, trace_func, &trace_start, &trace_end, elf_header, ver, vername, veropt, NULL)) {
                 SetTraceEmu(trace_start, trace_end);
                 printf_dump(LOG_INFO, "TRACE on %s only (%p-%p)\n", trace_func, (void*)trace_start, (void*)trace_end);
                 box_free(trace_func);
                 trace_func = NULL;
-            } else if(GetLibLocalSymbolStartEnd(lib, trace_func, &trace_start, &trace_end, 0, &weak, -1, NULL, 0, NULL)) {
+            } else if(GetLibLocalSymbolStartEnd(lib, trace_func, &trace_start, &trace_end, 0, &weak, &ver, &vername, 0, &veropt, NULL)) {
                 SetTraceEmu(trace_start, trace_end);
                 printf_dump(LOG_INFO, "TRACE on %s only (%p-%p)\n", trace_func, (void*)trace_start, (void*)trace_end);
                 box_free(trace_func);
@@ -496,49 +580,40 @@ void Free1Library(library_t **the_lib, x64emu_t* emu)
 
     library_t* lib = *the_lib;
 
-    printf_dump(LOG_DEBUG, "Free1Library %s\n", lib->name);
+    FiniLibrary(lib, emu);
+
+    printf_dump(LOG_DEBUG, "Free1Library %s\n", lib->name?:"???");
     // remove lib from maplib/local_maplib...
     if(my_context) {
         MapLibRemoveLib(my_context->maplib, lib);
         MapLibRemoveLib(my_context->local_maplib, lib);
     }
     // free elf is relevant
-    if(lib->type==LIB_EMULATED) {
+    int lib_type = lib->type;
+    // Handle needed for cleaning
+    needed_libs_t* needed = (lib_type==LIB_EMULATED && lib->e.elf)?lib->e.elf->needed:((lib_type==LIB_WRAPPED)?lib->w.needed:NULL);
+    if(needed)
+        needed = copy_neededlib(needed);
+    // free elf
+    if(lib_type==LIB_EMULATED) {
         FreeElfHeader(&lib->e.elf);
     }
 
     // No "Fini" logic here, only memory handling
-    if(lib->maplib)
-        FreeLibrarian(&lib->maplib, emu);
-
-    if(lib->type!=LIB_UNNKNOW && lib->fini) {
+    if(lib->maplib && !lib->maplib_ref) {
+        lib_t* maplib = lib->maplib;
+        lib->maplib = NULL;
+        FreeLibrarian(&maplib, emu);
+    }
+    lib->maplib = NULL;
+    lib->maplib_ref = 0;
+    if(lib_type!=LIB_UNNKNOW && lib->fini) {
         lib->fini(lib);
     }
     box_free(lib->name);
     box_free(lib->path);
 
-    if(lib->gbridgemap) {
-        bridged_t *br;
-        kh_foreach_value_ref(lib->gbridgemap, br,
-            box_free(br->name);
-        );
-        kh_destroy(bridgemap, lib->gbridgemap);
-    }
-    if(lib->wbridgemap) {
-        bridged_t *br;
-        kh_foreach_value_ref(lib->wbridgemap, br,
-            box_free(br->name);
-        );
-        kh_destroy(bridgemap, lib->wbridgemap);
-    }
-    if(lib->lbridgemap) {
-        bridged_t *br;
-        kh_foreach_value_ref(lib->lbridgemap, br,
-            box_free(br->name);
-        );
-        kh_destroy(bridgemap, lib->lbridgemap);
-    }
-    if(lib->type == LIB_WRAPPED) {
+    if(lib_type == LIB_WRAPPED) {
         if(lib->w.symbolmap)
             kh_destroy(symbolmap, lib->w.symbolmap);
         if(lib->w.wsymbolmap)
@@ -559,12 +634,19 @@ void Free1Library(library_t **the_lib, x64emu_t* emu)
             kh_destroy(symbol2map, lib->w.symbol2map);
     }
 
-    // remove frim the dlopen collection
+    // remove from the dlopen collection
     if(lib->dlopen)
         RemoveDlopen(the_lib, lib->dlopen);
-    box_free(lib);
+    initDummyLib(lib);
+    //box_free(lib);
     if(*the_lib == lib)
         *the_lib = NULL;
+    // handle needed libs now
+    if(needed) {
+        for(int i=0; i<needed->size; ++i)
+            DecRefCount(&needed->libs[i], emu);
+        free_neededlib(needed);
+    }
 }
 
 char* GetNameLib(library_t* lib)
@@ -608,81 +690,37 @@ int IsSameLib(library_t* lib, const char* path)
     box_free(name);
     return ret;
 }
-int GetLibWeakSymbolStartEnd(library_t* lib, const char* name, uintptr_t* start, uintptr_t* end, size_t size, int* weak, int version, const char* vername, int local, const char* defver)
+int GetLibWeakSymbolStartEnd(library_t* lib, const char* name, uintptr_t* start, uintptr_t* end, size_t size, int* weak, int* version, const char** vername, int local, int* veropt, void** elfsym)
 {
-    if(!lib || !name[0])
+    if(!name[0] || !lib || (lib->type==LIB_UNNKNOW))
         return 0;
-    khint_t k;
-    kh_bridgemap_t *map = local?lib->lbridgemap:((*weak)?lib->wbridgemap:lib->gbridgemap);
-    // check first if already in the map
-    k = kh_get(bridgemap, map, VersionedName(name, version, vername));
-    if(k!=kh_end(map)) {
-        *start = kh_value(map, k).start;
-        *end = kh_value(map, k).end;
-        return 1;
-    }
     // get a new symbol
-    if(lib->getweak(lib, name, start, end, size, weak, version, vername, local, defver)) {
+    if(lib->getweak(lib, name, start, end, size, weak, version, vername, local, veropt, elfsym)) {
         *end += *start;     // lib->get(...) gives size, not end
-        char* symbol = box_strdup(VersionedName(name, version, vername));
-        int ret;
-        k = kh_put(bridgemap, map, symbol, &ret);
-        kh_value(map, k).name = symbol;
-        kh_value(map, k).start = *start;
-        kh_value(map, k).end = *end;
         return 1;
     }
     // nope
     return 0;
 }
-int GetLibGlobalSymbolStartEnd(library_t* lib, const char* name, uintptr_t* start, uintptr_t* end, size_t size, int* weak, int version, const char* vername, int local, const char* defver)
+int GetLibGlobalSymbolStartEnd(library_t* lib, const char* name, uintptr_t* start, uintptr_t* end, size_t size, int* weak, int* version, const char** vername, int local, int* veropt, void** elfsym)
 {
-    if(!name[0] || !lib)
+    if(!name[0] || !lib || (lib->type==LIB_UNNKNOW))
         return 0;
-    khint_t k;
-    kh_bridgemap_t *map = local?lib->lbridgemap:((*weak)?lib->wbridgemap:lib->gbridgemap);
-    // check if already in the map
-    k = kh_get(bridgemap, map, VersionedName(name, version, vername));
-    if(k!=kh_end(map)) {
-        *start = kh_value(map, k).start;
-        *end = kh_value(map, k).end;
-        return 1;
-    }
     // get a new symbol
-    if(lib->getglobal(lib, name, start, end, size, weak, version, vername, local, defver)) {
+    if(lib->getglobal(lib, name, start, end, size, weak, version, vername, local, veropt, elfsym)) {
         *end += *start;     // lib->get(...) gives size, not end
-        char* symbol = box_strdup(VersionedName(name, version, vername));
-        int ret;
-        k = kh_put(bridgemap, map, symbol, &ret);
-        kh_value(map, k).name = symbol;
-        kh_value(map, k).start = *start;
-        kh_value(map, k).end = *end;
         return 1;
     }
     // nope
     return 0;
 }
-int GetLibLocalSymbolStartEnd(library_t* lib, const char* name, uintptr_t* start, uintptr_t* end, size_t size, int* weak, int version, const char* vername, int local, const char* defver)
+int GetLibLocalSymbolStartEnd(library_t* lib, const char* name, uintptr_t* start, uintptr_t* end, size_t size, int* weak, int* version, const char** vername, int local, int* veropt, void** elfsym)
 {
-    if(!name[0])
+    if(!name[0] || !lib || (lib->type==LIB_UNNKNOW))
         return 0;
-    khint_t k;
-    // check first if already in the map
-    k = kh_get(bridgemap, lib->lbridgemap, VersionedName(name, version, vername));
-    if(k!=kh_end(lib->lbridgemap)) {
-        *start = kh_value(lib->lbridgemap, k).start;
-        *end = kh_value(lib->lbridgemap, k).end;
-        return 1;
-    }
     // get a new symbol
-    if(lib->getlocal(lib, name, start, end, size, weak, version, vername, local, defver)) {
+    if(lib->getlocal(lib, name, start, end, size, weak, version, vername, local, veropt, elfsym)) {
         *end += *start;     // lib->get(...) gives size, not end
-        char* symbol = box_strdup(VersionedName(name, version, vername));
-        int ret;
-        k = kh_put(bridgemap, lib->lbridgemap, symbol, &ret);
-        kh_value(lib->lbridgemap, k).name = symbol;
-        kh_value(lib->lbridgemap, k).start = *start;
-        kh_value(lib->lbridgemap, k).end = *end;
         return 1;
     }
     // nope
@@ -706,11 +744,19 @@ static int getSymbolInDataMaps(library_t*lib, const char* name, int noweak, uint
     void* symbol;
     khint_t k = kh_get(datamap, lib->w.datamap, name);
     if (k!=kh_end(lib->w.datamap)) {
+        #ifdef STATICBUILD
+        symbol = (void*)(kh_value(lib->w.datamap, k).addr);
+        #else
         symbol = dlsym(lib->w.lib, kh_key(lib->w.datamap, k));
+        #endif
         if(symbol) {
             // found!
             *addr = (uintptr_t)symbol;
+            #ifdef STATICBUILD
+            *size = kh_value(lib->w.datamap, k).size;
+            #else
             *size = kh_value(lib->w.datamap, k);
+            #endif
             *weak = 0;
             return 1;
         }
@@ -718,11 +764,19 @@ static int getSymbolInDataMaps(library_t*lib, const char* name, int noweak, uint
     if(!noweak) {
         k = kh_get(datamap, lib->w.wdatamap, name);
         if (k!=kh_end(lib->w.wdatamap)) {
+            #ifdef STATICBUILD
+            symbol = (void*)(kh_value(lib->w.wdatamap, k).addr);
+            #else
             symbol = dlsym(lib->w.lib, kh_key(lib->w.wdatamap, k));
+            #endif
             if(symbol) {
                 // found!
                 *addr = (uintptr_t)symbol;
+                #ifdef STATICBUILD
+                *size = kh_value(lib->w.wdatamap, k).size;
+                #else
                 *size = kh_value(lib->w.wdatamap, k);
+                #endif
                 *weak = 1;
                 return 1;
             }
@@ -737,13 +791,21 @@ static int getSymbolInDataMaps(library_t*lib, const char* name, int noweak, uint
         else
             strcpy(buff, "my_");
         strcat(buff, name);
+        #ifdef STATICBUILD
+        symbol = kh_value(lib->w.mydatamap, k).addr;
+        #else
         symbol = dlsym(my_context->box64lib, buff);
+        #endif
         if(!symbol)
             printf_log(LOG_NONE, "Warning, data %s not found\n", buff);
         if(symbol) {
             // found!
             *addr = (uintptr_t)symbol;
+            #ifdef STATICBUILD
+            *size = kh_value(lib->w.mydatamap, k).size;
+            #else
             *size = kh_value(lib->w.mydatamap, k);
+            #endif
             *weak = 0;
             return 1;
         }
@@ -764,7 +826,11 @@ static int getSymbolInSymbolMaps(library_t*lib, const char* name, int noweak, ui
             else
                 strcpy(buff, "my_");
             strcat(buff, name);
+            #ifdef STATICBUILD
+            symbol = (void*)s->addr;
+            #else
             symbol = dlsym(my_context->box64lib, buff);
+            #endif
             if(!symbol) {
                 printf_log(LOG_NONE, "Warning, function %s not found\n", buff);
                 return 0;
@@ -788,7 +854,11 @@ static int getSymbolInSymbolMaps(library_t*lib, const char* name, int noweak, ui
             else
                 strcpy(buff, "my_");
             strcat(buff, name);
+            #ifdef STATICBUILD
+            symbol = (void*)s->addr;
+            #else
             symbol = dlsym(my_context->box64lib, buff);
+            #endif
             if(!symbol) {
                 printf_log(LOG_NONE, "Warning, function %s not found\n", buff);
                 return 0;
@@ -806,6 +876,9 @@ static int getSymbolInSymbolMaps(library_t*lib, const char* name, int noweak, ui
     if (k!=kh_end(lib->w.symbolmap)) {
         symbol1_t *s = &kh_value(lib->w.symbolmap, k);
         if(!s->resolved) {
+            #ifdef STATICBUILD
+            symbol = (void*)s->addr;
+            #else
             symbol = dlsym(lib->w.lib, name);
             if(!symbol && lib->w.altprefix) {
                 char newname[200];
@@ -821,6 +894,7 @@ static int getSymbolInSymbolMaps(library_t*lib, const char* name, int noweak, ui
                 strcat(newname, name);
                 symbol = GetNativeSymbolUnversioned(lib->w.lib, newname);
             }
+            #endif
             if(!symbol) {
                 printf_dump(LOG_INFO, "Warning, function %s not found in lib %s\n", name, lib->name);
                 return 0;
@@ -845,7 +919,11 @@ static int getSymbolInSymbolMaps(library_t*lib, const char* name, int noweak, ui
                 else
                     strcpy(buff, "my_");
                 strcat(buff, name);
+                #ifdef STATICBUILD
+                symbol = (void*)s->addr;
+                #else
                 symbol = dlsym(my_context->box64lib, buff);
+                #endif
                 if(!symbol) {
                     printf_log(LOG_NONE, "Warning, function %s not found\n", buff);
                     return 0;
@@ -862,6 +940,9 @@ static int getSymbolInSymbolMaps(library_t*lib, const char* name, int noweak, ui
         if (k!=kh_end(lib->w.wsymbolmap)) {
             symbol1_t *s = &kh_value(lib->w.wsymbolmap, k);
             if(!s->resolved) {
+                #ifdef STATICBUILD
+                symbol = (void*)s->addr;
+                #else
                 symbol = dlsym(lib->w.lib, name);
                 if(!symbol && lib->w.altprefix) {
                     char newname[200];
@@ -877,6 +958,7 @@ static int getSymbolInSymbolMaps(library_t*lib, const char* name, int noweak, ui
                     strcat(newname, name);
                     symbol = GetNativeSymbolUnversioned(lib->w.lib, newname);
                 }
+                #endif
                 if(!symbol) {
                     printf_dump(LOG_INFO, "Warning, function %s not found in lib %s\n", name, lib->name);
                     return 0;
@@ -897,11 +979,15 @@ static int getSymbolInSymbolMaps(library_t*lib, const char* name, int noweak, ui
         if(!noweak || !s->weak)
         {
             if(!s->resolved) {
+                #ifdef STATICBUILD
+                symbol = (void*)s->addr;
+                #else
                 symbol = dlsym(lib->w.lib, kh_value(lib->w.symbol2map, k).name);
                 if(!symbol)
                     symbol = dlsym(RTLD_DEFAULT, kh_value(lib->w.symbol2map, k).name);    // search globaly maybe
                 if(!symbol)
                     symbol = GetNativeSymbolUnversioned(lib->w.lib, kh_value(lib->w.symbol2map, k).name);
+                #endif
                 if(!symbol) {
                     printf_dump(LOG_INFO, "Warning, function %s not found in lib %s\n", kh_value(lib->w.symbol2map, k).name, lib->name);
                     return 0;
@@ -918,7 +1004,7 @@ static int getSymbolInSymbolMaps(library_t*lib, const char* name, int noweak, ui
     return 0;
 }
 
-int getSymbolInMaps(library_t *lib, const char* name, int noweak, uintptr_t *addr, uintptr_t *size, int* weak, int version, const char* vername, int local)
+int getSymbolInMaps(library_t *lib, const char* name, int noweak, uintptr_t *addr, uintptr_t *size, int* weak, int version, const char* vername, int local, int veropt)
 {
     if(version==-2) // don't send global native symbol for a version==-2 search
         return 0;
@@ -981,6 +1067,13 @@ lib_t* GetMaplib(library_t* lib)
     return lib->maplib;
 }
 
+int GetDeepBind(library_t* lib)
+{
+    if(!lib)
+        return 0;
+    return lib->deepbind;
+}
+
 linkmap_t* getLinkMapLib(library_t* lib)
 {
     linkmap_t* lm = my_context->linkmap;
@@ -1040,6 +1133,7 @@ needed_libs_t* new_neededlib(int n)
 {
     needed_libs_t* ret = (needed_libs_t*)calloc(1, sizeof(needed_libs_t));
     ret->cap = ret->size = n;
+    ret->init_size = n;
     ret->libs = (library_t**)calloc(n, sizeof(library_t*));
     ret->names = (char**)calloc(n, sizeof(char*));
     return ret;
@@ -1065,6 +1159,7 @@ void add1_neededlib(needed_libs_t* needed)
     needed->libs = (library_t**)realloc(needed->libs, needed->cap*sizeof(library_t*));
     needed->names = (char**)realloc(needed->names, needed->cap*sizeof(char*));
     needed->size++;
+    needed->init_size++;
 }
 void add1lib_neededlib(needed_libs_t* needed, library_t* lib, const char* name)
 {
@@ -1075,14 +1170,53 @@ void add1lib_neededlib(needed_libs_t* needed, library_t* lib, const char* name)
         if(needed->libs[i]==lib)
             return;
     // add it
-    if(needed->size+1<=needed->cap)
-        return;
-    needed->cap = needed->size+1;
-    needed->libs = (library_t**)realloc(needed->libs, needed->cap*sizeof(library_t*));
-    needed->names = (char**)realloc(needed->names, needed->cap*sizeof(char*));
+    if(needed->size==needed->cap) {
+        needed->cap = needed->size+1;
+        needed->libs = (library_t**)realloc(needed->libs, needed->cap*sizeof(library_t*));
+        needed->names = (char**)realloc(needed->names, needed->cap*sizeof(char*));
+    }
     needed->libs[needed->size] = lib;
     needed->names[needed->size] = (char*)name;
     needed->size++;
+    needed->init_size++;
+}
+void add1lib_neededlib_name(needed_libs_t* needed, library_t* lib, const char* name)
+{
+    if(!needed || !name)
+        return;
+    // check if lib is already present
+    for (int i=0; i<needed->size; ++i)
+        if(!strcmp(needed->names[i], name))
+            return;
+    // add it
+    if(needed->size==needed->cap) {
+        needed->cap = needed->size+1;
+        needed->libs = (library_t**)realloc(needed->libs, needed->cap*sizeof(library_t*));
+        needed->names = (char**)realloc(needed->names, needed->cap*sizeof(char*));
+    }
+    needed->libs[needed->size] = lib;
+    needed->names[needed->size] = (char*)name;
+    needed->size++;
+    needed->init_size++;
+}
+void add1libref_neededlib(needed_libs_t* needed, library_t* lib)
+{
+    if(!needed || !lib)
+        return;
+    // check if lib is already present
+    for (int i=0; i<needed->size; ++i)
+        if(needed->libs[i]==lib)
+            return;
+    // add it
+    if(needed->size==needed->cap) {
+        needed->cap = needed->size+4;
+        needed->libs = (library_t**)realloc(needed->libs, needed->cap*sizeof(library_t*));
+        needed->names = (char**)realloc(needed->names, needed->cap*sizeof(char*));
+    }
+    needed->libs[needed->size] = lib;
+    needed->names[needed->size] = (char*)lib->name;
+    needed->size++;
+    IncRefCount(lib, NULL);
 }
 needed_libs_t* copy_neededlib(needed_libs_t* needed)
 {
@@ -1091,6 +1225,7 @@ needed_libs_t* copy_neededlib(needed_libs_t* needed)
     needed_libs_t* ret = (needed_libs_t*)calloc(1, sizeof(needed_libs_t));
     ret->cap = needed->cap;
     ret->size = needed->size;
+    ret->init_size = needed->init_size;
     ret->libs = (library_t**)calloc(ret->cap, sizeof(library_t*));
     ret->names = (char**)calloc(ret->cap, sizeof(char*));
     memcpy(ret->libs, needed->libs, ret->size*sizeof(library_t*));
@@ -1111,23 +1246,23 @@ void setNeededLibs(library_t* lib, int n, ...)
     va_end (va);
 }
 
+#define LIB_MAXCNT 255
 void IncRefCount(library_t* lib, x64emu_t* emu)
 {
-    if(!lib || lib->type==LIB_UNNKNOW)
+    if(!lib || lib->type==LIB_UNNKNOW) {
+        printf_log(LOG_NONE, "Warning, IncRefCount of a LIB_UNKNOWN library\n");
         return;
+    }
     switch (lib->type) {
         case LIB_WRAPPED:
+            if(lib->w.refcnt==LIB_MAXCNT)
+                return;
             ++lib->w.refcnt;
-            /*if(lib->w.needed)
-                for(int i=0; i<lib->w.needed->size; ++i) {
-                    IncRefCount(lib->w.needed->libs[i], emu);
-                }*/
             break;
         case LIB_EMULATED:
+            if(lib->e.elf->refcnt==LIB_MAXCNT)
+                return;
             ++lib->e.elf->refcnt;
-            /*if(lib->e.elf->needed)
-                for(int i=0; i<lib->e.elf->needed->size; ++i)   // some libs may not be loaded yet
-                    IncRefCount(lib->e.elf->needed->libs[i], emu);*/
     }
 }
 
@@ -1140,35 +1275,28 @@ int DecRefCount(library_t** lib, x64emu_t* emu)
         return 0;
     }
     int ret = 1;
-    needed_libs_t* needed = NULL;
-    int freed = 0;
     switch ((*lib)->type) {
         case LIB_WRAPPED:
-            needed = (*lib)->w.needed;
+            if((*lib)->w.refcnt==LIB_MAXCNT)
+                return ret;
             ret=--(*lib)->w.refcnt;
             if(!ret) {
-                needed = copy_neededlib(needed);
-                freed=1;
                 Free1Library(lib, emu);
             }
             break;
         case LIB_EMULATED:
-            needed = (*lib)->e.elf->needed;
-            ret=--(*lib)->e.elf->refcnt;
+            if(!(*lib)->e.elf)
+                ret = 0;
+            else {
+                if((*lib)->e.elf->refcnt==LIB_MAXCNT)
+                    return ret;
+                ret=--(*lib)->e.elf->refcnt;
+            }
             if(!ret) {
-                needed = copy_neededlib(needed);
-                freed=1;
-                removeLinkMapLib(*lib);
-                FiniLibrary(*lib, emu);
                 Free1Library(lib, emu);
             }
             break;
     }
-    /*if(needed)
-        for(int i=0; i<needed->size; ++i)
-            DecRefCount(&needed->libs[i], emu);*/
-    if(freed)
-        free_neededlib(needed);
     return ret;
 }
 
