@@ -3,16 +3,18 @@
 #endif
 
 GO("libc.musl-x86_64.so.1", libcmusl)
-GO("libpthread.so.0", libpthread)
 GO("librt.so.1", librt)
 GO("libbsd.so.0", libbsd)
 GO("libEGL.so.1", libegl)
+GO("libEGL.so", libegl)
 GO("libGL.so.1", libgl)
 GO("libGL.so", libgl)
 GO("libOpenGL.so.0", libgl)
 GO("libGLU.so.1", libglu)
 GO("libGLX.so.0", libglx)
 GO("libGLX.so", libglx)
+GO("libGLESv2.so.2", glesv2)
+GO("libGLESv2.so", glesv2)
 GO("libX11.so.6", libx11)
 GO("libasound.so.2", libasound)
 GO("libasound.so", libasound)
@@ -55,6 +57,7 @@ GO("libXau.so.6", libxau)
 GO("libXdmcp.so.6", libxdmcp)
 GO("libX11-xcb.so.1", libx11xcb)
 GO("libxcb.so.1", libxcb)
+GO("libxcb-cursor.so.0", libxcbcursor)
 GO("libxcb-xfixes.so.0", libxcbxfixes)
 GO("libxcb-shape.so.0", libxcbshape)
 GO("libxcb-shm.so.0", libxcbshm)
@@ -72,6 +75,7 @@ GO("libxcb-render-util.so.0", libxcbrenderutil)
 GO("libxcb-render.so.0", libxcbrender)
 GO("libxcb-sync.so.1", libxcbsync)
 GO("libxcb-xinerama.so.0", libxcbxinerama)
+GO("libxcb-xinput.so.0", libxcbxinput)
 GO("libxcb-xkb.so.1", libxcbxkb)
 //GO("libxcb-present.so.0", libxcbpresent)
 GO("libXtst.so.6", libxtst)
@@ -81,6 +85,7 @@ GO("libXdamage.so.1", libxdamage)
 GO("libXpresent.so.1", libxpresent)
 GO("libXmu.so.6", libxmu)
 GO("libxkbcommon.so.0", xkbcommon)
+GO("libxkbregistry.so.0", xkbregistry)
 GO("libxkbcommon-x11.so.0", xkbcommonx11)
 GO("libz.so.1", libz)
 GO("libopenal.so.1", openal)
@@ -100,6 +105,7 @@ GO("libudev.so.0", udev0)
 GO("libudev.so.1", udev1)
 GO("libudev.so", udev1)
 GO("libdbus-1.so.3", dbus)
+GO("libdecor-0.so.0", decor0)
 //GO("libiconv.so.2", iconv)
 GO("libdrm.so.2", libdrm)
 GO("libexpat.so.1", expat)
@@ -113,7 +119,9 @@ GO("libgcrypt.so.20", gcrypt)
 GO("libutil.so.1", util)
 GO("libuuid.so.1", libuuid)
 GO("libresolv.so.2", libresolv)
+GO("libresolv.so", libresolv)
 GO("libpulse-simple.so.0", pulsesimple)
+GO("libpulse-mainloop-glib.so.0", pulsemainloopglib)
 GO("libpulse.so.0", pulse)
 GO("libsndfile.so.1", libsndfile)
 GO("libgtk-x11-2.0.so.0", gtkx112)
@@ -176,7 +184,6 @@ GO("libtcmalloc_minimal.so.4", tcmallocminimal)
 GO("libpcre.so.3", libpcre)
 GO("libvulkan.so.1", vulkan)
 GO("libvulkan.so", vulkan)
-//GO("libwayland-client.so.0", waylandclient)
 GO("libgbm.so.1", gbm)
 GO("libgomp.so.1", gomp)
 GO("libcap.so.2", cap)
@@ -232,18 +239,36 @@ GO("libvdpau.so.1", libvdpau)
 GO("libnuma.so.1", libnuma)
 GO("libnuma.so", libnuma)
 GO("d3dadapter9.so.1", d3dadapter9)
+GO("libicuuc.so.64", icuuc64)
+GO("libicui18n.so.64", icui18n64)
 GO("libicuuc.so.66", icuuc66)
 GO("libicui18n.so.66", icui18n66)
 GO("libicuuc.so.67", icuuc67)
 GO("libicui18n.so.67", icui18n67)
 GO("libicuuc.so.72", icuuc72)
 GO("libicui18n.so.72", icui18n72)
+GO("libicuuc.so.73", icuuc73)
+GO("libicui18n.so.73", icui18n73)
+GO("libicuuc.so.74", icuuc74)
+GO("libicui18n.so.74", icui18n74)
+GO("libidn2.so.0", idn2)
+GO("libpam.so.0", pam)
+GO("libanl.so.1", anl)
+GO("libpsl.so.5", psl5)
+GO("libssh2.so.1", libssh2)
+GO("libp11-kit.so.0", p11kit)
+GO("libtasn1.so.6", libtasn1)
+GO("libnettle.so.8", libnettle8)
+GO("libunistring.so.2", libunistring2)
+GO("libhogweed.so.6", libhogweed6)
+GO("libsqlite3.so.0", libsqlite3)
 
 #ifdef ANDROID
 GO("libc.so", libc)
 GO("libm.so", libm)
 GO("libbsd.so", libbsd)
 GO("libdl.so", libdl)
+GO("libpthread.so", libpthread)
 GO("libcups.so", libcups)
 GO("liblber-2.4.so", lber)
 GO("libldap_r-2.4.so", ldapr)
@@ -301,6 +326,7 @@ GO("libXdamage.so", libxdamage)
 GO("libXpresent.so", libxpresent)
 GO("libXmu.so", libxmu)
 GO("libxkbcommon.so", xkbcommon)
+GO("libxkbregistry.so", xkbregistry)
 GO("libxkbcommon-x11.so", xkbcommonx11)
 GO("libpulse-simple.so", pulsesimple)
 GO("libpulse.so", pulse)
@@ -330,10 +356,21 @@ GO("libogg.so", libogg)
 GO("libFLAC.so", flac)
 GO("libz.so", libz)
 GO("libandroid-shmem.so", androidshmem)
+GO("libgobject-2.0.so", gobject2)
+GO("libglib-2.0.so", glib2)
+GO("libgtk-3.so", gtk3)
+GO("libgdk-3.so", gdk3)
+GO("libgdk_pixbuf-2.0.so", gdkpixbuf2)
+GO("libgio-2.0.so", gio2)
+GO("libgmodule-2.0.so", gmodule2)
+GO("libgtk-x11-2.0.so", gtkx112)
+GO("libgdk-x11-2.0.so", gdkx112)
+GO("libpangocairo-1.0.so", pangocairo)
 #else
 GO("libc.so.6", libc)
 GO("libm.so.6", libm)
 GO("libdl.so.2", libdl)
+GO("libpthread.so.0", libpthread)
 GO("libcups.so.2", libcups)
 GO("liblber-2.4.so.2", lber)
 GO("libldap_r-2.4.so.2", ldapr)
