@@ -46,9 +46,6 @@ void _pthread_cleanup_pop(void* buffer, int exec);
 // it will be pthread_kill@GLIBC_2.17 on aarch64, but it's GLIBC_2.2.5 on x86_64
 static iFli_t real_phtread_kill_old = NULL;
 
-// from src/wrapped/wrappedlibc.c
-void* my_mmap(x64emu_t* emu, void* addr, unsigned long length, int prot, int flags, int fd, int64_t offset);
-
 typedef struct threadstack_s {
 	void* 	stack;
 	size_t 	stacksize;
@@ -56,7 +53,7 @@ typedef struct threadstack_s {
 
 typedef struct x64_unwind_buff_s {
 	struct {
-		jump_buff_x64_t		__cancel_jmp_buf;	
+		jump_buff_x64_t		__cancel_jmp_buf;
 		int					__mask_was_saved;
 	} __cancel_jmp_buf[1];
 	void *__pad[4];
@@ -142,9 +139,9 @@ static void emuthread_destroy(void* p)
 	emuthread_t *et = (emuthread_t*)p;
 	if(!et)
 		return;
-	void* ptr;
 	// check tlsdata
-	/*if (my_context && (ptr = pthread_getspecific(my_context->tlskey)) != NULL)
+	/*void* ptr;
+	if (my_context && (ptr = pthread_getspecific(my_context->tlskey)) != NULL)
         free_tlsdatasize(ptr);*/
 	// free x64emu
 	if(et) {
@@ -203,7 +200,9 @@ x64emu_t* thread_get_emu()
 					stacksize = stack_size;
 			pthread_attr_destroy(&attr);
 		}
-		void* stack = my_mmap(NULL, NULL, stacksize, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_GROWSDOWN, -1, 0);
+		void* stack = internal_mmap(NULL, stacksize, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_GROWSDOWN, -1, 0);
+		if(stack!=MAP_FAILED)
+			setProtection((uintptr_t)stack, stacksize, PROT_READ|PROT_WRITE);
 		x64emu_t *emu = NewX64Emu(my_context, 0, (uintptr_t)stack, stacksize, 1);
 		SetupX64Emu(emu, NULL);
 		thread_set_emu(emu);
@@ -248,61 +247,30 @@ static void* pthread_routine(void* p)
 }
 
 #ifdef NOALIGN
-pthread_attr_t* getAlignedAttr(pthread_attr_t* m) {
-	return m;
-}
-void freeAlignedAttr(void* attr) {
-	(void)attr;
-}
+#define PTHREAD_ATTR_ALIGN(A)
+#define PTHREAD_ATTR_UNALIGN(A)
+#define PTHREAD_ATTR(A) 	A
 #else
-typedef struct aligned_attr_s {
-	uint64_t sign;
-	pthread_attr_t *at;
-} aligned_attr_t;
-#define SIGN_ATTR *(uint64_t*)"BOX64ATT"
-
-pthread_attr_t* getAlignedAttrWithInit(pthread_attr_t* attr, int init)
-{
-	if(!attr)
-		return attr;
-	aligned_attr_t* at = (aligned_attr_t*)attr;
-	if(init && at->sign==SIGN_ATTR)
-		return at->at;
-	pthread_attr_t* ret = (pthread_attr_t*)box_calloc(1, sizeof(pthread_attr_t));
-	at->sign = SIGN_ATTR;
-	at->at = ret;
-	if(init)
-		pthread_attr_init(ret);	// init?
-	return ret;
-}
-pthread_attr_t* getAlignedAttr(pthread_attr_t* attr)
-{
-	return getAlignedAttrWithInit(attr, 1);
-}
-void freeAlignedAttr(void* attr)
-{
-	if(!attr)
-		return;
-	aligned_attr_t* at = (aligned_attr_t*)attr;
-	if(at->sign==SIGN_ATTR) {
-		box_free(at->at);
-		at->sign = 0LL;
-	}
-}
+#define PTHREAD_ATTR_ALIGN(A) pthread_attr_t aligned_attr = {0}; if(A) memcpy(&aligned_attr, A, 56)
+#define PTHREAD_ATTR_UNALIGN(A) if(A) memcpy(A, &aligned_attr, 56)
+#define PTHREAD_ATTR(A)		(A)?&aligned_attr:NULL
 #endif
 
 EXPORT int my_pthread_attr_destroy(x64emu_t* emu, void* attr)
 {
 	if(emu->context->stacksizes)
 		FreeStackSize(emu->context->stacksizes, (uintptr_t)attr);
-	int ret = pthread_attr_destroy(getAlignedAttr(attr));
-	freeAlignedAttr(attr);
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_destroy(PTHREAD_ATTR(attr));
+	// no unaligned, it's destroyed
 	return ret;
 }
 
 EXPORT int my_pthread_attr_getstack(x64emu_t* emu, void* attr, void** stackaddr, size_t* stacksize)
 {
-	int ret = pthread_attr_getstack(getAlignedAttr(attr), stackaddr, stacksize);
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_getstack(PTHREAD_ATTR(attr), stackaddr, stacksize);
+	// no need to unalign, it's const for attr
 	if (ret==0)
 		GetStackSize(emu, (uintptr_t)attr, stackaddr, stacksize);
 	return ret;
@@ -316,63 +284,77 @@ EXPORT int my_pthread_attr_setstack(x64emu_t* emu, void* attr, void* stackaddr,
 	AddStackSize(emu->context->stacksizes, (uintptr_t)attr, stackaddr, stacksize);
 	//Don't call actual setstack...
 	//return pthread_attr_setstack(attr, stackaddr, stacksize);
-	return pthread_attr_setstacksize(getAlignedAttr(attr), stacksize);
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_setstacksize(PTHREAD_ATTR(attr), stacksize);
+	PTHREAD_ATTR_UNALIGN(attr);
+	return ret;
 }
 
 EXPORT int my_pthread_attr_setstacksize(x64emu_t* emu, void* attr, size_t stacksize)
 {
 	(void)emu;
 	//aarch64 have an PTHREAD_STACK_MIN of 131072 instead of 16384 on x86_64!
-	if(stacksize<PTHREAD_STACK_MIN)
+	if(stacksize<(size_t)PTHREAD_STACK_MIN)
 		stacksize = PTHREAD_STACK_MIN;
-	return pthread_attr_setstacksize(getAlignedAttr(attr), stacksize);
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_setstacksize(PTHREAD_ATTR(attr), stacksize);
+	PTHREAD_ATTR_UNALIGN(attr);
+	return ret;
 }
 
 #ifndef NOALIGN
 EXPORT int my_pthread_attr_getdetachstate(x64emu_t* emu, pthread_attr_t* attr, int *state)
 {
 	(void)emu;
-	return pthread_attr_getdetachstate(getAlignedAttr(attr), state);
+	PTHREAD_ATTR_ALIGN(attr);
+	return pthread_attr_getdetachstate(PTHREAD_ATTR(attr), state);
 }
 EXPORT int my_pthread_attr_getguardsize(x64emu_t* emu, pthread_attr_t* attr, size_t* size)
 {
 	(void)emu;
-	return pthread_attr_getguardsize(getAlignedAttr(attr), size);
+	PTHREAD_ATTR_ALIGN(attr);
+	return pthread_attr_getguardsize(PTHREAD_ATTR(attr), size);
 }
 #ifndef TERMUX
 EXPORT int my_pthread_attr_getinheritsched(x64emu_t* emu, pthread_attr_t* attr, int* sched)
 {
 	(void)emu;
-	return pthread_attr_getinheritsched(getAlignedAttr(attr), sched);
+	PTHREAD_ATTR_ALIGN(attr);
+	return pthread_attr_getinheritsched(PTHREAD_ATTR(attr), sched);
 }
 #endif
 EXPORT int my_pthread_attr_getschedparam(x64emu_t* emu, pthread_attr_t* attr, void* param)
 {
 	(void)emu;
-	return pthread_attr_getschedparam(getAlignedAttr(attr), param);
+	PTHREAD_ATTR_ALIGN(attr);
+	return pthread_attr_getschedparam(PTHREAD_ATTR(attr), param);
 }
 EXPORT int my_pthread_attr_getschedpolicy(x64emu_t* emu, pthread_attr_t* attr, int* policy)
 {
 	(void)emu;
-	return pthread_attr_getschedpolicy(getAlignedAttr(attr), policy);
+	PTHREAD_ATTR_ALIGN(attr);
+	return pthread_attr_getschedpolicy(PTHREAD_ATTR(attr), policy);
 }
 EXPORT int my_pthread_attr_getscope(x64emu_t* emu, pthread_attr_t* attr, int* scope)
 {
 	(void)emu;
-	return pthread_attr_getscope(getAlignedAttr(attr), scope);
+	PTHREAD_ATTR_ALIGN(attr);
+	return pthread_attr_getscope(PTHREAD_ATTR(attr), scope);
 }
 EXPORT int my_pthread_attr_getstackaddr(x64emu_t* emu, pthread_attr_t* attr, void* addr)
 {
 	(void)emu;
 	size_t size;
-	return pthread_attr_getstack(getAlignedAttr(attr), addr, &size);
+	PTHREAD_ATTR_ALIGN(attr);
+	return pthread_attr_getstack(PTHREAD_ATTR(attr), addr, &size);
 	//return pthread_attr_getstackaddr(getAlignedAttr(attr), addr);
 }
 EXPORT int my_pthread_attr_getstacksize(x64emu_t* emu, pthread_attr_t* attr, size_t* size)
 {
 	(void)emu;
 	void* addr;
-	int ret = pthread_attr_getstack(getAlignedAttr(attr), &addr, size);
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_getstack(PTHREAD_ATTR(attr), &addr, size);
 	if(!*size)
 		*size = 2*1024*1024;
 	//return pthread_attr_getstacksize(getAlignedAttr(attr), size);
@@ -381,59 +363,88 @@ EXPORT int my_pthread_attr_getstacksize(x64emu_t* emu, pthread_attr_t* attr, siz
 EXPORT int my_pthread_attr_init(x64emu_t* emu, pthread_attr_t* attr)
 {
 	(void)emu;
-	return pthread_attr_init(getAlignedAttrWithInit(attr, 0));
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_init(PTHREAD_ATTR(attr));
+	PTHREAD_ATTR_UNALIGN(attr);
+	return ret;
 }
 #ifndef ANDROID
 EXPORT int my_pthread_attr_setaffinity_np(x64emu_t* emu, pthread_attr_t* attr, size_t cpusize, void* cpuset)
 {
 	(void)emu;
-	return pthread_attr_setaffinity_np(getAlignedAttr(attr), cpusize, cpuset);
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_setaffinity_np(PTHREAD_ATTR(attr), cpusize, cpuset);
+	PTHREAD_ATTR_UNALIGN(attr);
+	return ret;
 }
 #endif
 EXPORT int my_pthread_attr_setdetachstate(x64emu_t* emu, pthread_attr_t* attr, int state)
 {
 	(void)emu;
-	return pthread_attr_setdetachstate(getAlignedAttr(attr), state);
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_setdetachstate(PTHREAD_ATTR(attr), state);
+	PTHREAD_ATTR_UNALIGN(attr);
+	return ret;
 }
 EXPORT int my_pthread_attr_setguardsize(x64emu_t* emu, pthread_attr_t* attr, size_t size)
 {
 	(void)emu;
-	return pthread_attr_setguardsize(getAlignedAttr(attr), size);
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_setguardsize(PTHREAD_ATTR(attr), size);
+	PTHREAD_ATTR_UNALIGN(attr);
+	return ret;
 }
 #ifndef TERMUX
 EXPORT int my_pthread_attr_setinheritsched(x64emu_t* emu, pthread_attr_t* attr, int sched)
 {
 	(void)emu;
-	return pthread_attr_setinheritsched(getAlignedAttr(attr), sched);
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_setinheritsched(PTHREAD_ATTR(attr), sched);
+	PTHREAD_ATTR_UNALIGN(attr);
+	return ret;
 }
 #endif
 EXPORT int my_pthread_attr_setschedparam(x64emu_t* emu, pthread_attr_t* attr, void* param)
 {
 	(void)emu;
-	return pthread_attr_setschedparam(getAlignedAttr(attr), param);
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_setschedparam(PTHREAD_ATTR(attr), param);
+	PTHREAD_ATTR_UNALIGN(attr);
+	return ret;
 }
 EXPORT int my_pthread_attr_setschedpolicy(x64emu_t* emu, pthread_attr_t* attr, int policy)
 {
 	(void)emu;
-	return pthread_attr_setschedpolicy(getAlignedAttr(attr), policy);
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_setschedpolicy(PTHREAD_ATTR(attr), policy);
+	PTHREAD_ATTR_UNALIGN(attr);
+	return ret;
 }
 EXPORT int my_pthread_attr_setscope(x64emu_t* emu, pthread_attr_t* attr, int scope)
 {
 	(void)emu;
-	return pthread_attr_setscope(getAlignedAttr(attr), scope);
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_setscope(PTHREAD_ATTR(attr), scope);
+	PTHREAD_ATTR_UNALIGN(attr);
+	return ret;
 }
 EXPORT int my_pthread_attr_setstackaddr(x64emu_t* emu, pthread_attr_t* attr, void* addr)
 {
 	size_t size = 2*1024*1024;
 	my_pthread_attr_getstacksize(emu, attr, &size);
-	return pthread_attr_setstack(getAlignedAttr(attr), addr, size);
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_attr_setstack(PTHREAD_ATTR(attr), addr, size);
+	PTHREAD_ATTR_UNALIGN(attr);
+	return ret;
 	//return pthread_attr_setstackaddr(getAlignedAttr(attr), addr);
 }
 #ifndef ANDROID
 EXPORT int my_pthread_getattr_np(x64emu_t* emu, pthread_t thread_id, pthread_attr_t* attr)
 {
 	(void)emu;
-	int ret = pthread_getattr_np(thread_id, getAlignedAttrWithInit(attr, 0));
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_getattr_np(thread_id, PTHREAD_ATTR(attr));
+	PTHREAD_ATTR_UNALIGN(attr);
 	if(!ret && thread_id==pthread_self()) {
 		if(!emu->context->stacksizes) {
 			emu->context->stacksizes = kh_init(threadstack);
@@ -455,12 +466,18 @@ EXPORT int my_pthread_getattr_np(x64emu_t* emu, pthread_t thread_id, pthread_att
 EXPORT int my_pthread_getattr_default_np(x64emu_t* emu, pthread_attr_t* attr)
 {
 	(void)emu;
-	return pthread_getattr_default_np(getAlignedAttrWithInit(attr, 0));
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_getattr_default_np(PTHREAD_ATTR(attr));
+	PTHREAD_ATTR_UNALIGN(attr);
+	return ret;
 }
 EXPORT int my_pthread_setattr_default_np(x64emu_t* emu, pthread_attr_t* attr)
 {
 	(void)emu;
-	return pthread_setattr_default_np(getAlignedAttr(attr));
+	PTHREAD_ATTR_ALIGN(attr);
+	int ret = pthread_setattr_default_np(PTHREAD_ATTR(attr));
+	PTHREAD_ATTR_UNALIGN(attr);
+	return ret;
 }
 #endif	//!ANDROID
 #endif
@@ -475,7 +492,8 @@ EXPORT int my_pthread_create(x64emu_t *emu, void* t, void* attr, void* start_rou
 
 	if(attr) {
 		size_t stsize;
-		if(pthread_attr_getstacksize(getAlignedAttr(attr), &stsize)==0)
+		PTHREAD_ATTR_ALIGN(attr);
+		if(pthread_attr_getstacksize(PTHREAD_ATTR(attr), &stsize)==0)
 			stacksize = stsize;
 	}
 	if(GetStackSize(emu, (uintptr_t)attr, &attr_stack, &attr_stacksize))
@@ -484,7 +502,9 @@ EXPORT int my_pthread_create(x64emu_t *emu, void* t, void* attr, void* start_rou
 		stacksize = attr_stacksize;
 		own = 0;
 	} else {
-		stack = my_mmap(NULL, NULL, stacksize, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_GROWSDOWN, -1, 0);
+		stack = internal_mmap(NULL, stacksize, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_GROWSDOWN, -1, 0);
+		if(stack!=MAP_FAILED)
+	        setProtection((uintptr_t)stack, stacksize, PROT_READ|PROT_WRITE);
 		own = 1;
 	}
 
@@ -502,24 +522,30 @@ EXPORT int my_pthread_create(x64emu_t *emu, void* t, void* attr, void* start_rou
 	}
 	#endif
 	// create thread
-	return pthread_create((pthread_t*)t, getAlignedAttr(attr), 
+	PTHREAD_ATTR_ALIGN(attr);
+	return pthread_create((pthread_t*)t, PTHREAD_ATTR(attr),
 		pthread_routine, et);
+	// no need too unalign for attr, it's const
 }
 
 void* my_prepare_thread(x64emu_t *emu, void* f, void* arg, int ssize, void** pet)
 {
 	int stacksize = (ssize)?ssize:(2*1024*1024);	//default stack size is 2Mo
-	void* stack = my_mmap(NULL, NULL, stacksize, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_GROWSDOWN, -1, 0);
+	void* stack = internal_mmap(NULL, stacksize, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_GROWSDOWN, -1, 0);
+	if(stack!=MAP_FAILED)
+		setProtection((uintptr_t)stack, stacksize, PROT_READ|PROT_WRITE);
 	emuthread_t *et = (emuthread_t*)box_calloc(1, sizeof(emuthread_t));
-    x64emu_t *emuthread = NewX64Emu(emu->context, (uintptr_t)f, (uintptr_t)stack, stacksize, 1);
+	x64emu_t *emuthread = NewX64Emu(emu->context, (uintptr_t)f, (uintptr_t)stack, stacksize, 1);
 	SetupX64Emu(emuthread, emu					);
 	//SetFS(emuthread, GetFS(emu));
 	et->emu = emuthread;
 	et->fnc = (uintptr_t)f;
 	et->arg = arg;
 	#ifdef DYNAREC
-	// pre-creation of the JIT code for the entry point of the thread
-	DBGetBlock(emu, (uintptr_t)f, 1, 0);	// function wrapping are 64bits only on box64
+	if(box64_dynarec) {
+		// pre-creation of the JIT code for the entry point of the thread
+		DBGetBlock(emu, (uintptr_t)f, 1, 0);	// function wrapping are 64bits only on box64
+	}
 	#endif
 	*pet =  et;
 	return pthread_routine;
@@ -584,7 +610,7 @@ GO(25)			\
 GO(26)			\
 GO(27)			\
 GO(28)			\
-GO(29)			
+GO(29)
 
 // cleanup_routine
 #define GO(A)   \
@@ -635,7 +661,6 @@ static void* findkey_dtorFct(void* fct)
 #undef SUPER
 
 // custom implementation of pthread_once...
-static __thread uintptr_t my_once_callback_fct = 0;
 int EXPORT my_pthread_once(x64emu_t* emu, int* once, void* cb)
 {
 	if(*once)	// quick test first
@@ -652,7 +677,14 @@ int EXPORT my_pthread_once(x64emu_t* emu, int* once, void* cb)
 	#endif
 	if(old)
 		return 0;
+    // make some room and align R_RSP before doing the call (maybe it would be simpler to just use Callback functions)
+    Push64(emu, R_RBP); // push rbp
+    R_RBP = R_RSP;      // mov rbp, rsp
+    R_RSP -= 0x200;
+    R_RSP &= ~63LL;
 	DynaCall(emu, (uintptr_t)cb);  // using DynaCall, speedup wine 7.21 initialisation
+	R_RSP = R_RBP;          // mov rsp, rbp
+	R_RBP = Pop64(emu);     // pop rbp
 	return 0;
 }
 EXPORT int my___pthread_once(x64emu_t* emu, void* once, void* cb) __attribute__((alias("my_pthread_once")));
@@ -680,6 +712,12 @@ static pthread_cond_t* alignCond(pthread_cond_t* pc)
 	return pc;
 }
 
+EXPORT int my_pthread_cond_signal(x64emu_t* emu, pthread_cond_t* cond)
+{
+	(void)emu;
+	int ret = pthread_cond_signal(alignCond(cond));
+	return ret;
+}
 EXPORT int my_pthread_cond_timedwait(x64emu_t* emu, pthread_cond_t* cond, void* mutex, void* abstime)
 {
 	(void)emu;
