@@ -10,6 +10,7 @@
 #include "box64context.h"
 #include "fileutils.h"
 #include "pathcoll.h"
+#include "x64emu.h"
 #ifdef HAVE_TRACE
 #include "x64trace.h"
 #endif
@@ -40,12 +41,14 @@ static const char default_rcfile[] =
 "\n"
 "[heroic]\n"
 "BOX64_NOSANDBOX=1\n"
-"BOX64_INPROCESSGPU=1\n"
 "BOX64_MALLOC_HACK=2\n"
 "\n"
 "[LotCG.x86_64]\n"
 "BOX64_DYNAREC_FASTROUND=0\n"
 "\n"
+"[Mini Metro]\n"
+"BOX64_ADDLIBS=stdc++.so.6\n"
+"\n"
 "[pressure-vessel-wrap]\n"
 "BOX64_NOGTK=1\n"
 "\n"
@@ -58,13 +61,6 @@ static const char default_rcfile[] =
 "[streaming_client]\n"
 "BOX64_EMULATED_LIBS=libSDL2-2.0.so.0:libSDL2_ttf-2.0.so.0\n"
 "\n"
-"[steamwebhelper]\n"
-"BOX64_NOSANDBOX=1\n"
-"BOX64_INPROCESSGPU=1\n"
-"BOX64_MALLOC_HACK=2\n"
-"BOX64_LOG=0\n"
-"BOX64_DYNAREC_BIGBLOCK=0\n"
-"\n"
 "[steam-runtime-check-requirements]\n"
 "BOX64_EXIT=1\n"
 "\n"
@@ -86,6 +82,9 @@ CENTRYBOOL(BOX64_NOSIGSEGV, no_sigsegv)                 \
 CENTRYBOOL(BOX64_NOSIGILL, no_sigill)                   \
 ENTRYBOOL(BOX64_SHOWSEGV, box64_showsegv)               \
 ENTRYBOOL(BOX64_SHOWBT, box64_showbt)                   \
+ENTRYBOOL(BOX64_MMAP32, box64_mmap32)                   \
+ENTRYBOOL(BOX64_IGNOREINT3, box64_ignoreint3)           \
+IGNORE(BOX64_RDTSC)                                     \
 ENTRYBOOL(BOX64_X11THREADS, box64_x11threads)           \
 ENTRYBOOL(BOX64_X11GLX, box64_x11glx)                   \
 ENTRYDSTRING(BOX64_LIBGL, box64_libGL)                  \
@@ -100,16 +99,30 @@ ENTRYBOOL(BOX64_CRASHHANDLER, box64_dummy_crashhandler) \
 ENTRYBOOL(BOX64_NOPULSE, box64_nopulse)                 \
 ENTRYBOOL(BOX64_NOGTK, box64_nogtk)                     \
 ENTRYBOOL(BOX64_NOVULKAN, box64_novulkan)               \
+ENTRYBOOL(BOX64_RDTSC_1GHZ, box64_rdtsc_1ghz)           \
+ENTRYBOOL(BOX64_SSE42, box64_sse42)                     \
+ENTRYINT(BOX64_AVX, new_avx, 0, 2, 2)                   \
 ENTRYBOOL(BOX64_FUTEX_WAITV, box64_futex_waitv)         \
 ENTRYSTRING_(BOX64_BASH, bash)                          \
 ENTRYINT(BOX64_JITGDB, jit_gdb, 0, 3, 2)                \
 ENTRYBOOL(BOX64_NOSANDBOX, box64_nosandbox)             \
 ENTRYBOOL(BOX64_INPROCESSGPU, box64_inprocessgpu)       \
+ENTRYBOOL(BOX64_CEFDISABLEGPU, box64_cefdisablegpu)     \
+ENTRYBOOL(BOX64_CEFDISABLEGPUCOMPOSITOR, box64_cefdisablegpucompositor)\
 ENTRYBOOL(BOX64_EXIT, want_exit)                        \
 ENTRYBOOL(BOX64_LIBCEF, box64_libcef)                   \
+ENTRYBOOL(BOX64_JVM, box64_jvm)                         \
+ENTRYBOOL(BOX64_UNITYPLAYER, box64_unityplayer)         \
 ENTRYBOOL(BOX64_SDL2_JGUID, box64_sdl2_jguid)           \
 ENTRYINT(BOX64_MALLOC_HACK, box64_malloc_hack, 0, 2, 2) \
+ENTRYINTPOS(BOX64_MAXCPU, new_maxcpu)                   \
+ENTRYSTRING_(BOX64_ADDLIBS, new_addlibs)                \
 ENTRYSTRING_(BOX64_ENV, new_env)                        \
+ENTRYSTRING_(BOX64_ENV1, new_env1)                      \
+ENTRYSTRING_(BOX64_ENV2, new_env2)                      \
+ENTRYSTRING_(BOX64_ENV3, new_env3)                      \
+ENTRYSTRING_(BOX64_ENV4, new_env4)                      \
+ENTRYBOOL(BOX64_RESERVE_HIGH, new_reserve_high)         \
 
 #ifdef HAVE_TRACE
 #define SUPER2()                                        \
@@ -138,8 +151,9 @@ ENTRYINT(BOX64_DYNAREC_DUMP, box64_dynarec_dump, 0, 2, 2)           \
 ENTRYINT(BOX64_DYNAREC_LOG, box64_dynarec_log, 0, 3, 2)             \
 ENTRYINT(BOX64_DYNAREC_BIGBLOCK, box64_dynarec_bigblock, 0, 3, 2)   \
 ENTRYSTRING_(BOX64_DYNAREC_FORWARD, box64_dynarec_forward)          \
-ENTRYINT(BOX64_DYNAREC_STRONGMEM, box64_dynarec_strongmem, 0, 3, 2) \
+ENTRYINT(BOX64_DYNAREC_STRONGMEM, box64_dynarec_strongmem, 0, 4, 3) \
 ENTRYBOOL(BOX64_DYNAREC_X87DOUBLE, box64_dynarec_x87double)         \
+ENTRYBOOL(BOX64_DYNAREC_DIV0, box64_dynarec_div0)                   \
 ENTRYBOOL(BOX64_DYNAREC_FASTNAN, box64_dynarec_fastnan)             \
 ENTRYBOOL(BOX64_DYNAREC_FASTROUND, box64_dynarec_fastround)         \
 ENTRYINT(BOX64_DYNAREC_SAFEFLAGS, box64_dynarec_safeflags, 0, 2, 2) \
@@ -147,12 +161,12 @@ ENTRYBOOL(BOX64_DYNAREC_CALLRET, box64_dynarec_callret)             \
 ENTRYBOOL(BOX64_DYNAREC_BLEEDING_EDGE, box64_dynarec_bleeding_edge) \
 ENTRYBOOL(BOX64_DYNAREC_JVM, box64_dynarec_jvm)                     \
 ENTRYBOOL(BOX64_DYNAREC_TBB, box64_dynarec_tbb)                     \
-ENTRYINT(BOX64_DYNAREC_HOTPAGE, box64_dynarec_hotpage, 0, 255, 8)   \
-ENTRYBOOL(BOX64_DYNAREC_FASTPAGE, box64_dynarec_fastpage)           \
+IGNORE(BOX64_DYNAREC_HOTPAGE)                                       \
+IGNORE(BOX64_DYNAREC_FASTPAGE)                                      \
 ENTRYBOOL(BOX64_DYNAREC_ALIGNED_ATOMICS, box64_dynarec_aligned_atomics) \
 ENTRYBOOL(BOX64_DYNAREC_WAIT, box64_dynarec_wait)                   \
 ENTRYSTRING_(BOX64_NODYNAREC, box64_nodynarec)                      \
-ENTRYBOOL(BOX64_DYNAREC_TEST, box64_dynarec_test)                   \
+ENTRYSTRING_(BOX64_DYNAREC_TEST, box64_dynarec_test)                \
 ENTRYBOOL(BOX64_DYNAREC_MISSING, box64_dynarec_missing)             \
 
 #else
@@ -164,6 +178,7 @@ IGNORE(BOX64_DYNAREC_BIGBLOCK)                                      \
 IGNORE(BOX64_DYNAREC_FORWARD)                                       \
 IGNORE(BOX64_DYNAREC_STRONGMEM)                                     \
 IGNORE(BOX64_DYNAREC_X87DOUBLE)                                     \
+IGNORE(BOX64_DYNAREC_DIV0)                                          \
 IGNORE(BOX64_DYNAREC_FASTNAN)                                       \
 IGNORE(BOX64_DYNAREC_FASTROUND)                                     \
 IGNORE(BOX64_DYNAREC_SAFEFLAGS)                                     \
@@ -220,10 +235,30 @@ SUPER()
 #undef ENTRYDSTRING
 #undef ENTRYADDR
 #undef ENTRYULONG
-// the actual fields
+// the actual fields, in two steps to regroup bit fields together
 #define ENTRYBOOL(NAME, name) uint8_t name:1;
 #define CENTRYBOOL(NAME, name) uint8_t name:1;
 #define ENTRYINT(NAME, name, minval, maxval, bits) uint8_t name:bits;
+#define ENTRYINTPOS(NAME, name)
+#define ENTRYSTRING(NAME, name)
+#define ENTRYSTRING_(NAME, name)
+#define ENTRYDSTRING(NAME, name)
+#define ENTRYADDR(NAME, name)
+#define ENTRYULONG(NAME, name)
+SUPER()
+// done
+#undef ENTRYBOOL
+#undef CENTRYBOOL
+#undef ENTRYINT
+#undef ENTRYINTPOS
+#undef ENTRYSTRING
+#undef ENTRYSTRING_
+#undef ENTRYDSTRING
+#undef ENTRYADDR
+#undef ENTRYULONG
+#define ENTRYBOOL(NAME, name)
+#define CENTRYBOOL(NAME, name)
+#define ENTRYINT(NAME, name, minval, maxval, bits)
 #define ENTRYINTPOS(NAME, name) uint32_t name;
 #define ENTRYSTRING(NAME, name) char* name;
 #define ENTRYSTRING_(NAME, name) char* name;
@@ -276,7 +311,7 @@ static void addParam(const char* name, my_params_t* param)
     k = kh_get(params, params, name);
     if(k==kh_end(params)) {
         int ret;
-        k = kh_put(params, params, strdup(name), &ret);
+        k = kh_put(params, params, box_strdup(name), &ret);
     } else {
         clearParam(&kh_value(params, k));
     }
@@ -385,7 +420,7 @@ void LoadRCFile(const char* filename)
                 else if(!strcmp(key, #NAME)) {                          \
                     current_param.is_##name##_present = 1;              \
                     if(current_param.name) free(current_param.name);    \
-                    current_param.name = strdup(val);                   \
+                    current_param.name = box_strdup(val);               \
                 }
             #define ENTRYSTRING_(NAME, name) ENTRYSTRING(NAME, name)
             #define ENTRYDSTRING(NAME, name) ENTRYSTRING(NAME, name)
@@ -431,6 +466,7 @@ void LoadRCFile(const char* filename)
         free(current_name);
     }
     free(line);
+    fclose(f);
     printf_log(LOG_INFO, "Params database has %d entries\n", kh_size(params));
 }
 
@@ -455,6 +491,9 @@ extern FILE* ftrace;
 extern char* ftrace_name;
 void openFTrace(const char* newtrace);
 void addNewEnvVar(const char* s);
+void AddNewLibs(const char* libs);
+void computeRDTSC();
+void my_reserveHighMem();
 #ifdef DYNAREC
 void GatherDynarecExtensions();
 #endif
@@ -462,12 +501,20 @@ void GatherDynarecExtensions();
 void setupTraceInit();
 void setupTrace();
 #endif
+static char old_name[256] = "";
+const char* GetLastApplyName()
+{
+    return old_name;
+}
 void ApplyParams(const char* name)
 {
     if(!name || !params)
         return;
-    static char old_name[256] = "";
     int new_cycle_log = cycle_log;
+    int new_maxcpu = box64_maxcpu;
+    int new_avx = box64_avx2?2:box64_avx;
+    int box64_dynarec_jvm = box64_jvm;
+    int new_reserve_high = 0;
     if(!strcmp(name, old_name)) {
         return;
     }
@@ -492,7 +539,7 @@ void ApplyParams(const char* name)
     #define ENTRYINTPOS(NAME, name) if(param->is_##name##_present) {name = param->name; printf_log(LOG_INFO, "Applying %s=%d\n", #NAME, param->name);}
     #define ENTRYSTRING(NAME, name) if(param->is_##name##_present) {name = param->name; printf_log(LOG_INFO, "Applying %s=%s\n", #NAME, param->name);}
     #define ENTRYSTRING_(NAME, name)  
-    #define ENTRYDSTRING(NAME, name) if(param->is_##name##_present) {if(name) free(name); name = strdup(param->name); printf_log(LOG_INFO, "Applying %s=%s\n", #NAME, param->name);}
+    #define ENTRYDSTRING(NAME, name) if(param->is_##name##_present) {if(name) free(name); name = box_strdup(param->name); printf_log(LOG_INFO, "Applying %s=%s\n", #NAME, param->name);}
     #define ENTRYADDR(NAME, name) if(param->is_##name##_present) {name = param->name; printf_log(LOG_INFO, "Applying %s=%zd\n", #NAME, param->name);}
     #define ENTRYULONG(NAME, name) if(param->is_##name##_present) {name = param->name; printf_log(LOG_INFO, "Applying %s=%lld\n", #NAME, param->name);}
     SUPER()
@@ -515,6 +562,33 @@ void ApplyParams(const char* name)
         cycle_log = new_cycle_log;
         initCycleLog(my_context);
     }
+    if(new_reserve_high)
+        my_reserveHighMem();
+    if(param->is_new_avx_present) {
+        if(!new_avx) {
+            printf_log(LOG_INFO, "Hidding AVX extension\n");
+            box64_avx = 0; box64_avx2 = 0;
+        } else if(new_avx==1) {
+            printf_log(LOG_INFO, "Exposing AVX extension\n");
+            box64_avx = 1; box64_avx2 = 0;
+        } else if(new_avx==2) {
+            printf_log(LOG_INFO, "Exposing AVX/AVX2 extensions\n");
+            box64_avx = 1; box64_avx2 = 1;
+        }
+    }
+    if(param->is_box64_rdtsc_1ghz_present)
+        computeRDTSC();
+    #ifdef DYNAREC
+    if(param->is_box64_dynarec_jvm_present && !param->is_box64_jvm_present)
+        box64_jvm = box64_dynarec_jvm;
+    #endif
+    if(!box64_maxcpu_immutable) {
+        if(new_maxcpu!=box64_maxcpu && box64_maxcpu && box64_maxcpu<new_maxcpu) {
+        printf_log(LOG_INFO, "Not applying BOX64_MAXCPU=%d because a lesser value is already active: %d\n", new_maxcpu, box64_maxcpu);
+        } else
+            box64_maxcpu = new_maxcpu;
+    } else if(new_maxcpu!=box64_maxcpu)
+        printf_log(LOG_INFO, "Not applying BOX64_MAXCPU=%d because it's too late\n", new_maxcpu);
     if(param->is_ld_library_path_present) AppendList(&my_context->box64_ld_lib, param->ld_library_path, 1);
     if(param->is_box64_path_present) AppendList(&my_context->box64_path, param->box64_path, 1);
     if(param->is_trace_file_present) {
@@ -529,14 +603,33 @@ void ApplyParams(const char* name)
         AppendList(&my_context->box64_emulated_libs, param->emulated_libs, 0);
         printf_log(LOG_INFO, "Applying %s=%s\n", "BOX64_EMULATED_LIBS", param->emulated_libs);
     }
+    if(param->is_new_addlibs_present) {
+        AddNewLibs(param->new_addlibs);
+    }
     if(param->is_new_env_present) {
         addNewEnvVar(param->new_env);
         printf_log(LOG_INFO, "Applying %s=%s\n", "BOX64_ENV", param->new_env);
     }
+    if(param->is_new_env1_present) {
+        addNewEnvVar(param->new_env1);
+        printf_log(LOG_INFO, "Applying %s=%s\n", "BOX64_ENV1", param->new_env1);
+    }
+    if(param->is_new_env2_present) {
+        addNewEnvVar(param->new_env2);
+        printf_log(LOG_INFO, "Applying %s=%s\n", "BOX64_ENV2", param->new_env2);
+    }
+    if(param->is_new_env3_present) {
+        addNewEnvVar(param->new_env3);
+        printf_log(LOG_INFO, "Applying %s=%s\n", "BOX64_ENV3", param->new_env3);
+    }
+    if(param->is_new_env4_present) {
+        addNewEnvVar(param->new_env4);
+        printf_log(LOG_INFO, "Applying %s=%s\n", "BOX64_ENV4", param->new_env4);
+    }
     if(param->is_bash_present && FileIsX64ELF(param->bash)) {
         if(my_context->bashpath)
             free(my_context->bashpath);
-        my_context->bashpath = strdup(param->bash);
+        my_context->bashpath = box_strdup(param->bash);
         printf_log(LOG_INFO, "Applying %s=%s\n", "BOX64_BASH", param->bash);
     }
     #ifdef HAVE_TRACE
@@ -572,16 +665,44 @@ void ApplyParams(const char* name)
     #ifdef DYNAREC
     if(param->is_box64_nodynarec_present) {
         uintptr_t no_start = 0, no_end = 0;
-        char* p;
-        no_start = strtoul(param->box64_nodynarec, &p, 0);
-        if(p!=param->box64_nodynarec && p[0]=='-') {
-            char* p2;
-            ++p;
-            no_end = strtoul(p, &p2, 0);
-            if(p2!=p && no_end>no_start) {
-                box64_nodynarec_start = no_start;
-                box64_nodynarec_end = no_end;
-                printf_log(LOG_INFO, "Appling BOX64_NODYNAREC=%p-%p\n", (void*)box64_nodynarec_start, (void*)box64_nodynarec_end);
+        int ok = 0;
+        if(sscanf(param->box64_nodynarec, "0x%lX-0x%lX", &no_start, &no_end)==2)
+            ok = 1;
+        if(!ok && sscanf(param->box64_nodynarec, "%lx-%lx", &no_start, &no_end)==2)
+            ok = 1;
+        if(!ok && sscanf(param->box64_nodynarec, "%ld-%ld", &no_start, &no_end)==2)
+            ok = 1;
+        if(ok && no_end>no_start) {
+            box64_nodynarec_start = no_start;
+            box64_nodynarec_end = no_end;
+            printf_log(LOG_INFO, "Appling BOX64_NODYNAREC=%p-%p\n", (void*)box64_nodynarec_start, (void*)box64_nodynarec_end);
+        } else {
+            printf_log(LOG_INFO, "Ignoring BOX64_NODYNAREC=%s (%p-%p)\n", param->box64_nodynarec, (void*)box64_nodynarec_start, (void*)box64_nodynarec_end);
+        }
+    }
+    if(param->is_box64_dynarec_test_present) {
+        uintptr_t no_start = 0, no_end = 0;
+        if(strlen(param->box64_dynarec_test)==1) {
+            box64_dynarec_test = param->box64_dynarec_test[0]-'0';
+            box64_dynarec_test_start = 0x0;
+            box64_dynarec_test_end = 0x0;
+            if(box64_dynarec_test>2) box64_dynarec_test = 0;
+        } else {
+            int ok = 0;
+            if(sscanf(param->box64_dynarec_test, "0x%lX-0x%lX", &no_start, &no_end)==2)
+                ok = 1;
+            if(!ok && sscanf(param->box64_dynarec_test, "%lx-%lx", &no_start, &no_end)==2)
+                ok = 1;
+            if(!ok && sscanf(param->box64_dynarec_test, "%ld-%ld", &no_start, &no_end)==2)
+                ok = 1;
+            if(ok && no_end>no_start) {
+                box64_dynarec_test = 1;
+                box64_dynarec_test_start = no_start;
+                box64_dynarec_test_end = no_end;
+                printf_log(LOG_INFO, "Appling BOX64_DYNAREC_TEST=%p-%p\n", (void*)box64_dynarec_test_start, (void*)box64_dynarec_test_end);
+            } else {
+                box64_dynarec_test = 0;
+                printf_log(LOG_INFO, "Ignoring BOX64_DYNAREC_TEST=%s (%p-%p)\n", param->box64_dynarec_test, (void*)box64_dynarec_test_start, (void*)box64_dynarec_test_end);
             }
         }
     }
@@ -597,6 +718,8 @@ void ApplyParams(const char* name)
     if(param->is_box64_dynarec_test_present && box64_dynarec_test) {
         box64_dynarec_fastnan = 0;
         box64_dynarec_fastround = 0;
+        box64_dynarec_x87double = 1;
+        box64_dynarec_div0 = 1;
         box64_dynarec_callret = 0;
     }
     #endif
