@@ -59,6 +59,7 @@ static void add_no_overlap(void* addr, size_t size)
 
 void wine_prereserve(const char* reserve)
 {
+    init_custommem_helper(my_context);
     void* addr = NULL;
     size_t size = 0;
 
@@ -68,12 +69,8 @@ void wine_prereserve(const char* reserve)
 
     int idx = 0;
     while(my_wine_reserve[idx].addr && my_wine_reserve[idx].size) {
-        void* p = mmap(my_wine_reserve[idx].addr, my_wine_reserve[idx].size, 
-                    PROT_NONE, /*MAP_FIXED |*/ MAP_PRIVATE | MAP_ANON | MAP_NORESERVE, -1, 0);
-        if(p==(void*)-1 || p!=my_wine_reserve[idx].addr) {
-            printf_log(LOG_NONE, "Warning, prereserve of %p:0x%lx failed (%s)\n", my_wine_reserve[idx].addr, my_wine_reserve[idx].size, strerror(errno));
-            if(p!=(void*)-1)
-                munmap(p, my_wine_reserve[idx].size);
+        if(!isBlockFree(my_wine_reserve[idx].addr, my_wine_reserve[idx].size)) {
+            printf_log(LOG_NONE, "Warning, prereserve of %p:0x%lx is not free\n", my_wine_reserve[idx].addr, my_wine_reserve[idx].size);
             my_wine_reserve[idx].addr = NULL;
             my_wine_reserve[idx].size = 0;
         } else {
@@ -103,7 +100,7 @@ void dynarec_wine_prereserve()
     // don't reserve the initial arbritrary block as "with linker", it's not true
     for(int i=1; i<sizeof(my_wine_reserve)/sizeof(my_wine_reserve[0]); ++i)
         if(my_wine_reserve[i].addr && my_wine_reserve[i].size)
-            addDBFromAddressRange(my_context, (uintptr_t)my_wine_reserve[i].addr, my_wine_reserve[i].size, 0);  // prepare the prereserved area for exec, with linker
+            addDBFromAddressRange((uintptr_t)my_wine_reserve[i].addr, my_wine_reserve[i].size);  // prepare the prereserved area for exec, with linker
     #endif
 }
 #endif
