@@ -425,6 +425,7 @@ GO(gdk_pango_renderer_set_stipple, vFpip)
 GO(gdk_parse_args, vFpp)
 GO(gdk_pixbuf_get_from_drawable, pFpppiiiiii)
 GO(gdk_pixbuf_get_from_image, pFpppiiiiii)
+GO(gdk_pixbuf_get_from_window, pFpiiii)
 GO(gdk_pixbuf_render_pixmap_and_mask, vFpppi)
 GO(gdk_pixbuf_render_pixmap_and_mask_for_colormap, vFppppi)
 GO(gdk_pixbuf_render_threshold_alpha, vFppiiiiiii)
@@ -597,7 +598,7 @@ GOM(gdk_threads_add_timeout_full, uFEiuppp)
 GO(gdk_threads_enter, vFv)
 GO(gdk_threads_init, vFv)
 GO(gdk_threads_leave, vFv)
-//GOM(gdk_threads_set_lock_functions, vFEpp)
+GOM(gdk_threads_set_lock_functions, vFEpp)
 GO(gdk_toplevel_get_type, pFv)
 GO(gdk_unicode_to_keyval, uFu)
 GO(gdk_utf8_to_compound_text, iFppppp)
