@@ -20,7 +20,11 @@
 #include "gtkclass.h"
 #include "threads.h"
 
-const char* glib2Name = "libglib-2.0.so.0";
+#ifdef ANDROID
+    const char* glib2Name = "libglib-2.0.so";
+#else
+    const char* glib2Name = "libglib-2.0.so.0";
+#endif
 #define LIBNAME glib2
 
 typedef void  (*vFppip_t)(void*, void*, int, void*);
@@ -1342,20 +1346,25 @@ typedef struct my_GOptionEntry_s {
 EXPORT void my_g_option_context_add_main_entries(x64emu_t* emu, void* context, my_GOptionEntry_t* entries, void* domain)
 {
     my_GOptionEntry_t* p = entries;
-    while (p->long_name) {
+    int idx = 0;
+    while (p && p->long_name) {
         // wrap Callbacks
-        if (p->arg == 3)
-            p->arg_data = findGOptionArgFct(p->arg_data);
         ++p;
+        ++idx;
     }
-    my->g_option_context_add_main_entries(context, entries, domain);
     p = entries;
-    while (p->long_name) {
-        // unwrap Callbacks
+    my_GOptionEntry_t my_entries[idx+1];
+    idx = 0;
+    while (p && p->long_name) {
+        // wrap Callbacks
+        my_entries[idx] = *p;
         if (p->arg == 3)
-            p->arg_data = reverseGOptionArgFct(p->arg_data);
+            my_entries[idx].arg_data = findGOptionArgFct(p->arg_data);
         ++p;
+        ++idx;
     }
+    if(p) my_entries[idx] = *p;
+    my->g_option_context_add_main_entries(context, entries?my_entries:NULL, domain);
 }
 
 EXPORT void* my_g_strconcat(x64emu_t* emu, void* first, uintptr_t* data)
@@ -1435,16 +1444,13 @@ EXPORT void* my_g_once_impl(x64emu_t* emu, void* once, void* f, void* arg)
     return my->g_once_impl(once, findGThreadFuncFct(f), arg);
 }
 
+EXPORT void* my_g_bytes_new_with_free_func(x64emu_t* emu, void* data, unsigned long n, void* notify, void* user)
+{
+    return my->g_bytes_new_with_free_func(data, n, findGDestroyNotifyFct(notify), user);
+}
+
 #define PRE_INIT    \
     if(box64_nogtk) \
         return -1;
 
-#define CUSTOM_INIT \
-    my_lib = lib;       \
-    getMy(lib);
-
-#define CUSTOM_FINI \
-    freeMy();
-
 #include "wrappedlib_init.h"
-
