@@ -167,7 +167,7 @@ GO(g_bytes_new, pFpL)
 GO(g_bytes_new_from_bytes, pFpLL)
 GO(g_bytes_new_static, pFpL)
 GO(g_bytes_new_take, pFpL)
-//GOM(g_bytes_new_with_free_func, pFppLBp)
+GOM(g_bytes_new_with_free_func, pFEpLpp)
 GO(g_bytes_ref, pFp)
 GO(g_bytes_unref, vFp)
 GO(g_bytes_unref_to_array, pFp)
@@ -315,10 +315,10 @@ GO(g_date_time_new_from_timeval_utc, pFp)
 GO(g_date_time_new_from_unix_utc, pFl)
 //GO(g_date_time_new_local, 
 GO(g_date_time_new_now, pFp)
-//GO(g_date_time_new_now_local, 
+GO(g_date_time_new_now_local, pFv)
 GO(g_date_time_new_now_utc, pFv)
 GO(g_date_time_new_utc, pFiiiiid)
-//GO(g_date_time_ref, 
+GO(g_date_time_ref, pFp)
 //GO(g_date_time_to_local, 
 GO(g_date_time_to_timeval, iFpp)
 GO(g_date_time_to_timezone, pFpp)
@@ -613,7 +613,9 @@ GO(g_log_set_always_fatal, iFi)
 GOM(g_log_set_default_handler, pFEpp)
 GO(g_log_set_fatal_mask, iFpi)
 GOM(g_log_set_handler, uFEpipp)
+GO(g_log_writer_is_journald, iFi)
 GOM(g_logv, vFEpipA)
+GO(g_log_structured_array, vFipL)
 GO(g_lstat, iFpp)
 GO(g_main_context_acquire, iFp)
 GO(g_main_context_add_poll, vFppi)
@@ -754,7 +756,9 @@ GO(g_nullify_pointer, vFp)
 GOM(g_once_impl, pFEppp)
 GO(g_once_init_enter, iFp)
 GO(g_once_init_enter_impl, iFp)
+GO(g_once_init_enter_pointer, iFp)
 GO(g_once_init_leave, vFpL)
+GO(g_once_init_leave_pointer, vFpp)
 GO(g_on_error_query, vFp)
 GO(g_on_error_stack_trace, vFp)
 GO(g_open, iFpii)
@@ -1387,7 +1391,7 @@ GO(g_unicode_script_to_iso15924, uFi)
 //GO(g_unix_error_quark, 
 //GO(g_unix_fd_add, 
 //GO(g_unix_fd_add_full, 
-//GO(g_unix_fd_source_new, 
+GO(g_unix_fd_source_new, pFii)
 //GO(g_unix_open_pipe, 
 //GO(g_unix_set_fd_nonblocking, 
 //GO(g_unix_signal_add, 
