@@ -12,14 +12,21 @@
 #include "librarian/library_private.h"
 #include "x64emu.h"
 
-const char* gmodule2Name = "libgmodule-2.0.so.0";
+#ifdef ANDROID
+    const char* gmodule2Name = "libgmodule-2.0.so";
+#else
+    const char* gmodule2Name = "libgmodule-2.0.so.0";
+#endif
 #define LIBNAME gmodule2
 
 #define PRE_INIT    \
     if(box64_nogtk) \
         return -1;
 
-#define CUSTOM_INIT \
-    setNeededLibs(lib, 1, "libglib-2.0.so.0");
+#ifdef ANDROID
+#define NEEDED_LIBS "libglib-2.0.so"
+#else
+#define NEEDED_LIBS "libglib-2.0.so.0"
+#endif
 
 #include "wrappedlib_init.h"
