@@ -587,6 +587,75 @@ static void* findGstTypeFindFunctionFct(void* fct)
     printf_log(LOG_NONE, "Warning, no more slot for gstreamer GstTypeFindFunction callback\n");
     return NULL;
 }
+//GstPadIterIntLinkFunction
+#define GO(A)   \
+static uintptr_t my_GstPadIterIntLinkFunction_fct_##A = 0;                          \
+static void* my_GstPadIterIntLinkFunction_##A(void* a, void* b)                     \
+{                                                                                   \
+    return (void*)RunFunctionFmt(my_GstPadIterIntLinkFunction_fct_##A, "pp", a, b); \
+}
+SUPER()
+#undef GO
+static void* findGstPadIterIntLinkFunctionFct(void* fct)
+{
+    if(!fct) return fct;
+    void* p;
+    if((p = GetNativeFnc((uintptr_t)fct))) return p;
+    #define GO(A) if(my_GstPadIterIntLinkFunction_fct_##A == (uintptr_t)fct) return my_GstPadIterIntLinkFunction_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my_GstPadIterIntLinkFunction_fct_##A == 0) {my_GstPadIterIntLinkFunction_fct_##A = (uintptr_t)fct; return my_GstPadIterIntLinkFunction_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for gstreamer GstPadIterIntLinkFunction callback\n");
+    return NULL;
+}
+//GstPadStickyEventsForeachFunction
+#define GO(A)   \
+static uintptr_t my_GstPadStickyEventsForeachFunction_fct_##A = 0;                              \
+static int my_GstPadStickyEventsForeachFunction_##A(void* a, void* b, void* c)                  \
+{                                                                                               \
+    return (int)RunFunctionFmt(my_GstPadStickyEventsForeachFunction_fct_##A, "ppp", a, b, c);   \
+}
+SUPER()
+#undef GO
+static void* findGstPadStickyEventsForeachFunctionFct(void* fct)
+{
+    if(!fct) return fct;
+    void* p;
+    if((p = GetNativeFnc((uintptr_t)fct))) return p;
+    #define GO(A) if(my_GstPadStickyEventsForeachFunction_fct_##A == (uintptr_t)fct) return my_GstPadStickyEventsForeachFunction_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my_GstPadStickyEventsForeachFunction_fct_##A == 0) {my_GstPadStickyEventsForeachFunction_fct_##A = (uintptr_t)fct; return my_GstPadStickyEventsForeachFunction_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for gstreamer GstPadStickyEventsForeachFunction callback\n");
+    return NULL;
+}
+//GstBufferForeachMetaFunc
+#define GO(A)   \
+static uintptr_t my_GstBufferForeachMetaFunc_fct_##A = 0;                              \
+static int my_GstBufferForeachMetaFunc_##A(void* a, void* b, void* c)                  \
+{                                                                                      \
+    return (int)RunFunctionFmt(my_GstBufferForeachMetaFunc_fct_##A, "ppp", a, b, c);   \
+}
+SUPER()
+#undef GO
+static void* findGstBufferForeachMetaFuncFct(void* fct)
+{
+    if(!fct) return fct;
+    void* p;
+    if((p = GetNativeFnc((uintptr_t)fct))) return p;
+    #define GO(A) if(my_GstBufferForeachMetaFunc_fct_##A == (uintptr_t)fct) return my_GstBufferForeachMetaFunc_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my_GstBufferForeachMetaFunc_fct_##A == 0) {my_GstBufferForeachMetaFunc_fct_##A = (uintptr_t)fct; return my_GstBufferForeachMetaFunc_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for gstreamer GstBufferForeachMetaFunc callback\n");
+    return NULL;
+}
 
 #undef SUPER
 
@@ -908,7 +977,7 @@ EXPORT void* my_gst_structure_new_valist(x64emu_t* emu, void* name, void* first,
     #else
     CREATE_VALIST_FROM_VALIST(V, emu->scratch);
     #endif
-    my->gst_structure_new_valist(name, first, VARARGS);
+    return my->gst_structure_new_valist(name, first, VARARGS);
 }
 
 EXPORT void* my_gst_make_element_message_details(x64emu_t* emu, void* name, uintptr_t* b)
@@ -916,7 +985,7 @@ EXPORT void* my_gst_make_element_message_details(x64emu_t* emu, void* name, uint
     if(!name)
         return NULL;
     CREATE_VALIST_FROM_VAARG(b, emu->scratch, 1);
-    my->gst_structure_new_valist("detail", name, VARARGS);
+    return my->gst_structure_new_valist("detail", name, VARARGS);
 }
 
 EXPORT int my_gst_pad_start_task(x64emu_t* emu, void* pad, void* f, void* data, void* d)
@@ -1054,35 +1123,38 @@ EXPORT int my_gst_type_find_register(x64emu_t* emu, void* plugin, void* name, ui
     return my->gst_type_find_register(plugin, name, rank, findGstTypeFindFunctionFct(f), ext, caps, data, findDestroyFct(f));
 }
 
+EXPORT void my_gst_pad_set_iterate_internal_links_function_full(x64emu_t* emu, void* pad, void* f, void* data, void* d)
+{
+    my->gst_pad_set_iterate_internal_links_function_full(pad, findGstPadIterIntLinkFunctionFct(f), data, findDestroyFct(d));
+}
+
+EXPORT void my_gst_pad_sticky_events_foreach(x64emu_t* emu, void* pad, void* f, void* data)
+{
+    my->gst_pad_sticky_events_foreach(pad, findGstPadStickyEventsForeachFunctionFct(f), data);
+}
+
+EXPORT int my_gst_buffer_foreach_meta(x64emu_t* emu, void* buff, void* f, void* data)
+{
+    return my->gst_buffer_foreach_meta(buff, findGstBufferForeachMetaFuncFct(f), data);
+}
+
 #define PRE_INIT    \
     if(box64_nogtk) \
         return -1;
 
+#define CUSTOM_INIT \
+    SetGstObjectID(my->gst_object_get_type());                 \
+    SetGstAllocatorID(my->gst_allocator_get_type());           \
+    SetGstTaskPoolID(my->gst_task_pool_get_type());            \
+    SetGstElementID(my->gst_element_get_type());               \
+    SetGstBinID(my->gst_bin_get_type());                       \
+    SetGstPadID(my->gst_pad_get_type());                       \
+    SetGstURIHandlerID(my->gst_uri_handler_get_type());
+
 #ifdef ANDROID
-    #define CUSTOM_INIT \
-        getMy(lib);     \
-        SetGstObjectID(my->gst_object_get_type());                 \
-        SetGstAllocatorID(my->gst_allocator_get_type());           \
-        SetGstTaskPoolID(my->gst_task_pool_get_type());            \
-        SetGstElementID(my->gst_element_get_type());               \
-        SetGstBinID(my->gst_bin_get_type());                       \
-        SetGstPadID(my->gst_pad_get_type());                       \
-        SetGstURIHandlerID(my->gst_uri_handler_get_type());        \
-        setNeededLibs(lib, 1, "libgtk-3.so");
+#define NEEDED_LIBS "libgtk-3.so"
 #else
-    #define CUSTOM_INIT \
-        getMy(lib);     \
-        SetGstObjectID(my->gst_object_get_type());                 \
-        SetGstAllocatorID(my->gst_allocator_get_type());           \
-        SetGstTaskPoolID(my->gst_task_pool_get_type());            \
-        SetGstElementID(my->gst_element_get_type());               \
-        SetGstBinID(my->gst_bin_get_type());                       \
-        SetGstPadID(my->gst_pad_get_type());                       \
-        SetGstURIHandlerID(my->gst_uri_handler_get_type());        \
-        setNeededLibs(lib, 1, "libgtk-3.so.0");
+#define NEEDED_LIBS "libgtk-3.so.0"
 #endif
 
-#define CUSTOM_FINI \
-    freeMy();
-
 #include "wrappedlib_init.h"
