@@ -34,14 +34,25 @@ void stSetup(box64context_t* context)
     my___libc_stack_end = context->stack;   // is this the end, or should I add stasz?
 }
 
+#ifdef STATICBUILD
+#include <link.h>
+extern void* __libc_enable_secure;
+extern void* __stack_chk_guard;
+//extern void* __pointer_chk_guard;
+//extern void* _rtld_global;
+//extern void* _rtld_global_ro;
+#endif
+
 // don't try to load the actual ld-linux (because name is variable), just use box64 itself, as it's linked to ld-linux
 const char* ldlinuxName = "ld-linux.so.2";
 #define LIBNAME ldlinux
 
+#ifndef STATICBUILD
 #define PRE_INIT\
     if(1)                                                           \
         lib->w.lib = dlopen(NULL, RTLD_LAZY | RTLD_GLOBAL);    \
     else
+#endif
 
 #define CUSTOM_INIT         \
     stSetup(box64);         \
