@@ -15,9 +15,15 @@
 // _dl_tls_setup
 DATA(__libc_enable_secure, sizeof(void*))
 DATAM(__libc_stack_end, sizeof(void*))
-DATA(__pointer_chk_guard, sizeof(void*))
 DATAB(_r_debug, 40)
+#ifdef STATICBUILD
+//DATA(__pointer_chk_guard, sizeof(void*))
+//DATA(_rtld_global, sizeof(void*))
+//DATA(_rtld_global_ro, sizeof(void*))
+#else
+DATA(__pointer_chk_guard, sizeof(void*))
 DATA(_rtld_global, sizeof(void*))
 DATA(_rtld_global_ro, sizeof(void*))
+#endif
 DATA(__stack_chk_guard, sizeof(void*))
 GOM(__tls_get_addr, pFp)    //%NoE
