@@ -90,7 +90,7 @@ GOW(bindtextdomain, pFpp)
 GOW(bind_textdomain_codeset, pFpp)
 GOW(brk, iFp)
 //GO(__bsd_getpgrp, 
-//GOW(bsd_signal, 
+GO2(bsd_signal, pFEip, my_signal)
 GOM(bsearch, pFEppLLp)
 GOW(btowc, uFi)
 GO(__bzero, vFpL)
@@ -124,6 +124,21 @@ GO(chroot, iFp)
 GOW(clearenv, iFv)
 GO(clearerr, vFp)
 GO(clearerr_unlocked, vFp)
+#ifdef STATICBUILD
+//GO(clnt_broadcast, 
+//GO(clnt_create, !FpLLp)
+//GO(clnt_pcreateerror, vFp)
+//GO(clnt_perrno, vFu)
+//GO(clnt_perror, vF!p)
+//GO(clntraw_create, !FLL)
+//GO(clnt_spcreateerror, pFp)
+//GO(clnt_sperrno, pFu)
+//GO(clnt_sperror, pF!p)
+//GO(clnttcp_create, !F!LLpuu)
+//GO(clntudp_bufcreate, !F!LL?puu)
+//GO(clntudp_create, !F!LL?p)
+//GO(clntunix_create, !F!LLpuu)
+#else
 //GO(clnt_broadcast, 
 //GO(clnt_create, !FpLLp)
 GO(clnt_pcreateerror, vFp)
@@ -137,6 +152,7 @@ GO(clnt_sperrno, pFu)
 //GO(clntudp_bufcreate, !F!LL?puu)
 //GO(clntudp_create, !F!LL?p)
 //GO(clntunix_create, !F!LLpuu)
+#endif
 GO(clock, lFv)
 GO(clock_adjtime, iFip)
 //GO(__clock_getcpuclockid, 
@@ -156,9 +172,10 @@ GOW(close, iFi)
 GOW(closedir, iFp)
 GO(closelog, vFv)
 //GO(__close_nocancel, 
+GO(close_range, iFuuu)
 GO(__cmsg_nxthdr, pFpp)
 GO(confstr, LFipL)
-//GO(__confstr_chk, 
+GO(__confstr_chk, LFipLL)
 GOW(__connect, iFipu)
 GOW(connect, iFipu)
 GO(copy_file_range, lFipipLu)
@@ -175,7 +192,11 @@ GO(ctime_r, pFpp)
 //DATA(__ctype32_b, 
 //DATA(__ctype32_tolower, 
 //DATA(__ctype32_toupper, 
+#ifdef STATICBUILD
+//DATA(__ctype_b, sizeof(void*))
+#else
 DATA(__ctype_b, sizeof(void*))
+#endif
 GO(__ctype_b_loc, pFv)
 GO(__ctype_get_mb_cur_max, LFv)
 //GO(__ctype_init, 
@@ -276,7 +297,7 @@ GOW(erand48_r, iFppp)
 GOM(err, vFEipV)
 //DATAB(errno, 
 GO(__errno_location, pFv)
-//GOW(error, vFiipV)
+GOW(error, vFiipV)
 //GOW(error_at_line, vFiipupV)
 //DATAB(error_message_count, 
 //DATAB(error_one_per_line, 
@@ -297,10 +318,10 @@ GOM(execl, iFEpV) // First argument is also part of the variadic
 GOM(execle, iFEpV) // First argument is also part of the variadic
 GOM(execlp, iFEpV) // First argument is also part of the variadic
 GOM(execv, iFEpp)
-GOWM(execve, iFEppp)
+GOM(execve, iFEppp)
 GOM(execvp, iFEpp)
 GOW(execvpe, iFppp)
-GOM(_exit, vFEi)
+GO(_exit, vFi)  // immediate exit
 GOM(exit, vFEi)
 GOW(_Exit, vFi)
 GO(explicit_bzero, vFpL)
@@ -475,13 +496,17 @@ GOM(__fxstat64, iFEiip)
 GOM(__fxstatat, iFEiippi)
 GOM(__fxstatat64, iFEiippi)
 //GO(__gai_sigqueue, 
+GO(gai_cancel, iFp)
+GO(gai_error, iFp)
 GO(gai_strerror, pFi)
+GO(gai_suspend, iFpip)
 //GO(__gconv_get_alias_db, 
 //GO(__gconv_get_cache, 
 //GO(__gconv_get_modules_db, 
 //GO(__gconv_transliterate, 
 GO(gcvt, pFdip)
 GO(getaddrinfo, iFpppp)
+GO(getaddrinfo_a, iFipip)
 GO(getaliasbyname, pFp)
 GO(getaliasbyname_r, iFpppLp)
 GO(getaliasent, pFv)
@@ -560,7 +585,11 @@ GO(getnameinfo, iFpupupui)
 //GO(getnetent_r, iF!pL!p)
 GO(getnetgrent, iFppp)
 GOW(getnetgrent_r, iFppppL)
+#ifdef STATICBUILD
+//GO(getnetname, iFp)
+#else
 GO(getnetname, iFp)
+#endif
 GOW(get_nprocs, iFv)
 GOW(get_nprocs_conf, iFv)
 GOM(getopt, iFipp)
@@ -574,7 +603,7 @@ GO(__getpgid, iFi)
 GOW(getpgid, iFi)
 GO(getpgrp, iFv)
 GOW(get_phys_pages, lFv)
-GO(__getpid, uFv)
+GO(__getpid, iFv)
 GO(getpid, iFv)
 //GO(getpmsg, // Deprecated
 GOW(getppid, iFv)
@@ -586,7 +615,11 @@ GO(getprotobynumber_r, iFippLp)
 GO(getprotoent, pFv)
 GO(getprotoent_r, iFppLp)
 GOW(getpt, iFv)
+#ifdef STATICBUILD
+//GO(getpublickey, iFpp)
+#else
 GO(getpublickey, iFpp)
+#endif
 GOW(getpw, iFup)
 GO(getpwent, pFv)
 GO(getpwent_r, iFppLp)
@@ -606,13 +639,21 @@ GO(getrpcbynumber, pFi)
 GO(getrpcbynumber_r, iFippLp)
 GO(getrpcent, pFv)
 GO(getrpcent_r, iFppLp)
+#ifdef STATICBUILD
+//GO(getrpcport, 
+#else
 GO(getrpcport, iFpLLu)
+#endif
 GOW(getrusage, iFip)
 //GOW(gets, // Deprecated
 //GO(__gets_chk, 
+#ifdef STATICBUILD
+//GO(getsecretkey, 
+#else
 GO(getsecretkey, iFppp)
+#endif
 GO(getservbyname, pFpp)
-//GO(getservbyname_r, iFpp!pL!)
+GO(getservbyname_r, iFppppLp)
 GO(getservbyport, pFip)
 GO(getservbyport_r, iFipppLp)
 GO(getservent, pFv)
@@ -683,7 +724,11 @@ GOW(hdestroy, vFv)
 //DATAB(__h_errno, 
 GO(__h_errno_location, pFv)
 GO(herror, vFp)
+#ifdef STATICBUILD
+//GO(host2netname, 
+#else
 GO(host2netname, iFppp)
+#endif
 //GO(hsearch, pF?u)
 //GOW(hsearch_r, iF?up!)
 GO(hstrerror, pFi)
@@ -743,7 +788,7 @@ GO(initstate_r, iFupLp)
 GO(innetgr, iFpppp)
 GO(inotify_add_watch, iFipu)
 GO(inotify_init, iFv)
-GO(inotify_init1, iFi)
+GO(inotify_init1, iFO)
 GO(inotify_rm_watch, iFii)
 GO(insque, vFpp)
 //GO(__internal_endnetgrent, 
@@ -827,7 +872,11 @@ DATA(_IO_list_all, sizeof(void*))
 //GO(_IO_marker_difference, 
 //GO(_IO_padn, 
 //GO(_IO_peekc_locked, 
+#ifdef STATICBUILD
+//GO(ioperm, 
+#else
 GO(ioperm, iFLLi)
+#endif
 GOM(iopl, iFEi)     //Not always present
 //GO(_IO_popen, 
 //GO(_IO_printf, 
@@ -928,7 +977,10 @@ GOW(isnanl, iFD)
 GO2(__isoc23_fscanf, iFEppV, my___isoc99_fscanf)
 GO2(__isoc23_sscanf, iFEppV, my___isoc99_sscanf)
 GO2(__isoc23_strtol, lFppi, strtol)
+GO2(__isoc23_strtoll, IFppi, strtoll)
 GO2(__isoc23_strtoul, LFppi, strtoul)
+GO2(__isoc23_strtoull, UFppi, strtoull)
+GO2(__isoc23_wcstol, LFppi, wcstol)
 GOM(__isoc99_fscanf, iFEppV)
 //GO(__isoc99_fwscanf, iFppV)
 GOM(__isoc99_scanf, iFEpV)
@@ -936,7 +988,7 @@ GOM(__isoc99_sscanf, iFEppV)
 GOM(__isoc99_swscanf, iFEppV)
 GOM(__isoc99_vfscanf, iFEppA)
 //GO(__isoc99_vfwscanf, iFppA)
-//GO(__isoc99_vscanf, iFpA)
+GOM(__isoc99_vscanf, iFEpA)
 GOM(__isoc99_vsscanf, iFEppA)
 GOM(__isoc99_vswscanf, iFEppA)
 //GO(__isoc99_vwscanf, iFpA)
@@ -1008,9 +1060,15 @@ GOW(jrand48_r, iFppp)
 //GO(key_gendes, iF!)
 //DATAB(__key_gendes_LOCAL, 
 //GO(key_get_conv, iFp!)
+#ifdef STATICBUILD
+//GO(key_secretkey_is_set, 
+//GO(key_setnet, 
+//GO(key_setsecret, 
+#else
 GO(key_secretkey_is_set, iFv)
 //GO(key_setnet, 
 GO(key_setsecret, iFp)
+#endif
 GOW(kill, iFii)
 GO(killpg, iFii)
 GO(klogctl, iFipi)
@@ -1228,8 +1286,13 @@ GO(__nanosleep, iFpp)
 GOW(nanosleep, iFpp)
 //GO(__nanosleep_nocancel, 
 //GO(__netlink_assert_response, 
+#ifdef STATICBUILD
+//GO(netname2host, iFppi)
+//GO(netname2user, iFppppp)
+#else
 GO(netname2host, iFppi)
 GO(netname2user, iFppppp)
+#endif
 GO(__newlocale, pFipp)
 GOW(newlocale, pFipp)
 //GO(nfsservctl, // Deprecated
@@ -1318,7 +1381,7 @@ GO(perror, vFp)
 GOW(personality, iFL)
 GO(__pipe, iFp)
 GOW(pipe, iFp)
-GO(pipe2, iFpi)
+GO(pipe2, iFpO)
 //GO(pivot_root, // Deprecated
 GO(pkey_alloc, iFuu)
 GO(pkey_free, iFi)
@@ -1328,8 +1391,13 @@ GO(pkey_set, iFiu)
 //GO(pmap_getmaps, pF!)
 //GO(pmap_getport, WF!LLu)
 //GO(pmap_rmtcall, uF!LLL@p@p?p)
+#ifdef STATICBUILD
+//GO(pmap_set, iFLLiW)
+//GO(pmap_unset, iFLL)
+#else
 GO(pmap_set, iFLLiW)
 GO(pmap_unset, iFLL)
+#endif
 GO(__poll, iFpLi)
 GO(poll, iFpLi)
 GO(__poll_chk, iFpuiL)
@@ -1393,131 +1461,13 @@ DATA(__progname_full, sizeof(void))
 GOW(pselect, iFippppp)
 GO(psiginfo, vFpp)
 GO(psignal, vFip)
+#ifdef STATICBUILD
+//GO(__sF, vFip)
+//GO(__assert2, vFip)
+#else
 GO(__sF, vFip)
 GO(__assert2, vFip)
-
-GOM(pthread_atfork, iFEppp)
-GOM(pthread_attr_destroy, iFEp)
-GOM(pthread_attr_getdetachstate, iFEpp)
-GOM(pthread_attr_getguardsize, iFEpp)
-GOM(pthread_attr_getinheritsched, iFEpp)
-GOM(pthread_attr_getschedparam, iFEpp)
-GOM(pthread_attr_getschedpolicy, iFEpp)
-GOM(pthread_attr_getscope, iFEpp)
-GOM(pthread_attr_getstack, iFEppp)
-GOM(pthread_attr_getstackaddr, iFEpp)
-GOM(pthread_attr_getstacksize, iFEpp)
-GOM(pthread_attr_init, iFEp)
-GOM(pthread_attr_setaffinity_np, iFEpLp)
-GOM(pthread_attr_setdetachstate, iFEpi)
-GOM(pthread_attr_setguardsize, iFEpL)
-GOM(pthread_attr_setinheritsched, iFEpi)
-GOM(pthread_attr_setschedparam, iFEpp)
-GOM(pthread_attr_setschedpolicy, iFEpi)
-GOM(pthread_attr_setscope, iFEpi)
-GOM(pthread_attr_setstackaddr, iFEpp)
-GOM(pthread_attr_setstack, iFEppL)
-GOM(pthread_attr_setstacksize, iFEpL)
-GOM(pthread_barrierattr_destroy, iFEp)
-GOM(pthread_barrierattr_getpshared, iFEpp)
-GOM(pthread_barrierattr_init, iFEp)
-GOM(pthread_barrierattr_setpshared, iFEpi)
-GO(pthread_barrier_destroy, iFp)
-GOM(pthread_barrier_init, iFEppu)
-GO(pthread_barrier_wait, iFp)
-GO(pthread_cancel, iFL)
-GOM(pthread_condattr_destroy, iFEp)
-GOM(pthread_condattr_getclock, iFEpp)
-GOM(pthread_condattr_getpshared, iFEpp)
-GOM(pthread_condattr_init, iFEp)
-GOM(pthread_condattr_setclock, iFEpi)
-GOM(pthread_condattr_setpshared, iFEpi)
-GOM(pthread_cond_broadcast, iFEp)
-GOM(pthread_cond_destroy, iFEp)
-GOM(pthread_cond_init, iFEpp)
-GO(pthread_cond_signal, iFp)
-GOM(pthread_cond_timedwait, iFEppp)
-GOM(pthread_cond_wait, iFEpp)
-GOM(pthread_create, iFEpppp)
-GOM(pthread_cond_clockwait, iFEppip)
-GO(pthread_detach, iFL)
-GO(pthread_equal, iFLL)
-GO(pthread_exit, vFp)
-GOM(pthread_getaffinity_np, iFEpLp)
-GOM(pthread_getattr_np, iFELp)
-GOM(pthread_getattr_default_np, iFEp)
-GOM(pthread_setattr_default_np, iFEp)
-GO(pthread_getcpuclockid, iFLp)
-GO(pthread_getschedparam, iFLpp)
-GO(pthread_getspecific, pFL)
-GO(pthread_getname_np, iFppL)
-GO(pthread_join, iFLp)
-GOM(pthread_key_create, iFEpp)
-GO(pthread_key_delete, iFL)
-GO2(pthread_kill@GLIBC_2.2.5, iFEpi, my_pthread_kill_old)
-GOM(pthread_kill, iFEpi)
-GO(pthread_kill_other_threads_np, vFv)
-GOM(pthread_mutexattr_destroy, iFEp)
-GOM(pthread_mutexattr_getkind_np, iFEpp)
-GOM(pthread_mutexattr_getprotocol, iFEpp)
-GOM(pthread_mutexattr_getrobust, iFEpp)
-GOM(pthread_mutexattr_gettype, iFEpp)
-GOM(pthread_mutexattr_init, iFEp)
-GOM(pthread_mutexattr_setkind_np, iFEpi)
-GOM(pthread_mutexattr_setprotocol, iFEpi)
-GOM(pthread_mutexattr_setpshared, iFEpi)
-GOM(pthread_mutexattr_setrobust, iFEpi)
-GOM(pthread_mutexattr_settype, iFEpi)
-GO(pthread_mutex_consistent, iFp)
-GO(pthread_mutex_destroy, iFp)
-// phtread_mutex_t is 40 bytes on x86_64, but 48bytes on ARM64
-GOM(pthread_mutex_init, iFpp)
-GO(pthread_mutex_lock, iFp)
-GO(pthread_mutex_timedlock, iFpp)
-GO(pthread_mutex_trylock, iFp)
-GO(pthread_mutex_unlock, iFp)
-GOM(pthread_once, iFEpp)
-GO(pthread_rwlockattr_destroy, vFp)
-GO(pthread_rwlockattr_getkind_np, iFpp)
-GO(pthread_rwlockattr_init, iFp)
-GO(pthread_rwlockattr_setkind_np, iFpi)
-GO(pthread_rwlock_destroy, iFp)
-GO(pthread_rwlock_init, iFpp)
-GO(pthread_rwlock_rdlock, iFp)
-GO(pthread_rwlock_tryrdlock, iFp)
-GO(pthread_rwlock_trywrlock, iFp)
-GO(pthread_rwlock_unlock, iFp)
-GO(pthread_rwlock_wrlock, iFp)
-GO(pthread_self, LFv)
-GOM(pthread_setaffinity_np, iFEpLp)
-GO(pthread_setcancelstate, iFip)
-GO(pthread_setcanceltype, iFip)
-GO(pthread_setconcurrency, iFi)
-GO(pthread_setname_np, iFpp)
-GO(pthread_setschedparam, iFLip)
-GO(pthread_setschedprio, iFpi)
-GO(pthread_setspecific, iFLp)
-GO(pthread_sigmask, iFipp)
-GO(pthread_spin_destroy, iFp)
-GO(pthread_spin_init, iFpi)
-GO(pthread_spin_lock, iFp)
-GO(pthread_spin_trylock, iFp)
-GO(pthread_spin_unlock, iFp)
-GO(pthread_testcancel, vFv)
-GO(pthread_timedjoin_np, iFppp)
-GO(pthread_tryjoin_np, iFpp)
-GO(pthread_yield, iFv)
-GO(sem_close, iFp)
-GO(sem_clockwait, iFppp)
-GO(sem_destroy, iFp)
-GO(sem_getvalue, iFpp)
-GO(sem_init, iFpiu)
-GO(sem_open, pFpOM)
-GO(sem_post, iFp)
-GO(sem_timedwait, iFpp)
-GO(sem_trywait, iFp)
-GO(sem_unlink, iFp)
-GO(sem_wait, iFp)
+#endif
 
 GOM(ptrace, lFEuipp)
 GO(ptsname, pFi)
@@ -1611,7 +1561,7 @@ GOWM(register_printf_specifier, iFEipp)
 GOWM(register_printf_type, iFEp)
 //GO(registerrpc, // Deprecated?
 GO(remap_file_pages, iFpLiLi)
-//GOW(re_match, iFppii!)
+GOW(re_match, lFppllp)
 //GOW(re_match_2, iFppipii!i)
 //DATA(re_max_failures, 
 GO(remove, iFp)
@@ -1621,7 +1571,7 @@ GO(rename, iFpp)
 GOW(renameat, iFipip)
 GOW(renameat2, iFipipu)
 //DATAB(_res, 
-//GOW(re_search, iFppiii!)
+GOW(re_search, lFpplllp)
 //GOW(re_search_2, iFppipiii!i)
 //GOW(re_set_registers, vFp!upp)
 GOW(re_set_syntax, LFL)
@@ -1630,6 +1580,7 @@ GO(__res_iclose, vFpi)
 GO(__res_init, iFv)
 GO(__res_nclose, vFp)
 GO(__res_ninit, iFp)
+GO(res_nsearch, iFppiipi)
 //GO(__resolv_context_get, 
 //GO(__resolv_context_get_override, 
 //GO(__resolv_context_get_preinit, 
@@ -1646,12 +1597,21 @@ GO(rexec_af, iFpippppW)
 //DATAB(rexecoptions, 
 GO(rindex, pFpi)
 GOW(rmdir, iFp)
+#ifdef STATICBUILD
+//DATAB(rpc_createerr, 
+//GO(_rpc_dtablesize, iFv)
+//GO(__rpc_thread_createerr, !Fv)
+//GO(__rpc_thread_svc_fdset, !Fv)
+//GO(__rpc_thread_svc_max_pollfd, pFv)
+//GO(__rpc_thread_svc_pollfd, pFv)
+#else
 //DATAB(rpc_createerr, 
 GO(_rpc_dtablesize, iFv)
 //GO(__rpc_thread_createerr, !Fv)
 //GO(__rpc_thread_svc_fdset, !Fv)
 GO(__rpc_thread_svc_max_pollfd, pFv)
 GO(__rpc_thread_svc_pollfd, pFv)
+#endif
 GO(rpmatch, iFp)
 GO(rresvport, iFp)
 GO(rresvport_af, iFpW)
@@ -1891,7 +1851,7 @@ GO(__strerror_r, pFipL)
 GO(strerror_r, pFipL)
 //GO(strfmon, lFpLpV)
 //GO(__strfmon_l, 
-//GOW(strfmon_l, lFpLppV)
+GOW(strfmon_l, lFpLppdddddd)    // should be V, but only double are allowed...
 GO(strfromd, iFpLpd)
 GO(strfromf, iFpLpf)
 GO(strfromf128, iFpLpD)
@@ -1956,6 +1916,7 @@ GO(__strtof_l, fFppL)
 GOW(strtof_l, fFppp)
 //GO(__strtof_nan, 
 GO(strtoimax, lFppi)
+GO2(__isoc23_strtoimax, lFppi, strtoimax)
 GO(strtok, pFpp)
 GO(__strtok_r, pFppp)
 GOW(strtok_r, pFppp)
@@ -1983,6 +1944,7 @@ GO(__strtoull_internal, UFppii)
 GOW(__strtoull_l, LFppip)
 GOW(strtoull_l, UFppip)
 GO(strtoumax, LFppi)
+GO2(__isoc23_strtoumax, LFppi, strtoumax)
 GOW(strtouq, UFppi)
 //GO(__strverscmp, 
 GOW(strverscmp, iFpp)
@@ -1990,6 +1952,36 @@ GO(strxfrm, LFppL)
 GO(__strxfrm_l, LFppLL)
 GO(strxfrm_l, LFppLp)
 //GO(stty, // Deprecated
+#ifdef STATICBUILD
+//DATAB(svcauthdes_stats, 
+//GO(svcerr_auth, vF!u)
+//GO(svcerr_decode, vF!)
+//GO(svcerr_noproc, vF!)
+//GO(svcerr_noprog, vF!)
+//GO(svcerr_progvers, vF!LL)
+//GO(svcerr_systemerr, vF!)
+//GO(svcerr_weakauth, vF!)
+//GO(svc_exit, vFv)
+//GO(svcfd_create, !Fiuu)
+//DATAB(svc_fdset, 
+//GO(svc_getreq, vFi)
+//GO(svc_getreq_common, vFi)
+//GO(svc_getreq_poll, vFpi)
+//GO(svc_getreqset, vF!)
+//DATAB(svc_max_pollfd, 
+//DATAB(svc_pollfd, 
+//GO(svcraw_create, !Fv)
+//GO(svc_register, iF!LL@L)
+//GO(svc_run, vFv)
+//GO(svc_sendreply, iF!@p)
+//GO(svctcp_create, !Fiuu)
+//GO(svcudp_bufcreate, !Fiuu)
+//GO(svcudp_create, !Fi)
+//GO(svcudp_enablecache, 
+//GO(svcunix_create, !Fiuup)
+//GO(svcunixfd_create, 
+//GO(svc_unregister, vFLL)
+#else
 //DATAB(svcauthdes_stats, 
 //GO(svcerr_auth, vF!u)
 //GO(svcerr_decode, vF!)
@@ -2018,6 +2010,7 @@ GO(svc_run, vFv)
 //GO(svcunix_create, !Fiuup)
 //GO(svcunixfd_create, 
 GO(svc_unregister, vFLL)
+#endif
 GO(swab, vFppl)
 GOWM(swapcontext, iFEpp)
 //GOW(swapoff, 
@@ -2033,10 +2026,20 @@ GO(syncfs, iFi)
 GOM(syscall, lFEv)
 GOM(__sysconf, lFEi)
 GOM(sysconf, IFEi)
+#ifdef STATICBUILD
+//GO(__sysctl, 
+//GO(sysctl, 
+#else
 GO(__sysctl, iFpipppL)
 GOW(sysctl, iFpipppL) // Deprecated
+#endif
+#ifdef STATICBUILD
+//DATA(_sys_errlist, sizeof(void*))
+//DATA(sys_errlist, sizeof(void*))
+#else
 DATA(_sys_errlist, sizeof(void*))
 DATA(sys_errlist, sizeof(void*))
+#endif
 GOW(sysinfo, iFp)
 GOM(syslog, vFEipV)
 GOM(__syslog_chk, vFEiipV)
@@ -2066,6 +2069,7 @@ GOW(textdomain, pFp)
 GOWM(tfind, pFEppp)
 GOW(tgkill, iFiii)
 //GO(thrd_current, 
+GO(thrd_exit, vFp)
 //GO(thrd_equal, 
 //GO(thrd_sleep, 
 //GO(thrd_yield, 
@@ -2113,7 +2117,7 @@ GO(ttyname_r, iFipL)
 //GO(__ttyname_r_chk, 
 GO(ttyslot, iFv)
 //GO(__twalk, 
-//GOW(twalk, vFp@)
+GOWM(twalk, vFEpp)
 //GOW(twalk_r, vFp@p)
 DATA(__tzname, sizeof(void*))
 DATA(tzname, sizeof(void*)) //type V
@@ -2139,7 +2143,11 @@ GO(updwtmpx, vFpp)
 //GO(uselib, // Deprecated
 GO(__uselocale, pFp)
 GOW(uselocale, pFp)
+#ifdef STATICBUILD
+//GO(user2netname, 
+#else
 GO(user2netname, iFpup)
+#endif
 GO(usleep, iFu)
 //GO(ustat, // Deprecated
 GO(utime, iFpp)
@@ -2170,7 +2178,7 @@ GO(vhangup, iFv)
 GO(vmsplice, lFipLu)
 GOM(vprintf, iFEpA)
 GOM(__vprintf_chk, iFEvpp)
-//GOW(vscanf, iFpA)
+GOWM(vscanf, iFEpA)
 GOWM(__vsnprintf, iFEpLpA)
 GOWM(vsnprintf, iFEpLpA)
 GOM(__vsnprintf_chk, iFEpLiLpp)
@@ -2203,7 +2211,7 @@ GOW(wcpcpy, pFpp)
 GO(wcpncpy, pFppL)
 //GO(__wcpncpy_chk, 
 GO(wcrtomb, LFpup)
-//GO(__wcrtomb_chk, 
+GO(__wcrtomb_chk, LFpupL)
 GOW(wcscasecmp, iFpp)
 GO(__wcscasecmp_l, iFppp)
 GOW(wcscasecmp_l, iFppp)
@@ -2237,13 +2245,13 @@ GO(wcsnrtombs, LFppLLp)
 GO(wcspbrk, pFpp)
 GO(wcsrchr, pFpu)
 GO(wcsrtombs, LFppLp)
-//GO(__wcsrtombs_chk, 
+GO(__wcsrtombs_chk, LFppLpL)
 GO(wcsspn, LFpp)
 GO(wcsstr, pFpp)
 GO(wcstod, dFpp)
 //GO(__wcstod_internal, 
 //GO(__wcstod_l, 
-//GOW(wcstod_l, dFpp!)
+GOW(wcstod_l, dFppp)
 GO(wcstof, fFpp)
 GO(wcstof128, DFpp)
 //GO(__wcstof128_internal, 
@@ -2268,7 +2276,7 @@ GO(wcstold, DFpp)
 //GOW(wcstold_l, DFpp!)
 //GO(__wcstol_internal, 
 //GO(__wcstol_l, 
-//GOW(wcstol_l, lFppi!)
+GOW(wcstol_l, lFppip)
 GOW(wcstoll, IFppi)
 //GO(__wcstoll_internal, 
 //GOW(__wcstoll_l, 
@@ -2279,7 +2287,7 @@ GOW(wcstoq, IFppi)
 GO(wcstoul, LFppi)
 //GO(__wcstoul_internal, 
 //GO(__wcstoul_l, 
-//GOW(wcstoul_l, LFppi!)
+GOW(wcstoul_l, LFppip)
 GOW(wcstoull, UFppi)
 //GO(__wcstoull_internal, 
 //GOW(__wcstoull_l, 
@@ -2391,11 +2399,21 @@ GOW(writev, lFipi)
 //GO(xdr_u_quad_t, iF!p)
 //GO(xdr_u_short, iF!p)
 //GO(xdr_vector, iF!puu@)
+#ifdef STATICBUILD
+//GO(xdr_void, 
+//GO(xdr_wrapstring, iF!p)
+#else
 GO(xdr_void, iFv)
 //GO(xdr_wrapstring, iF!p)
+#endif
 //GO(xencrypt, 
-GO(__xmknod, iFipup)
-GO(__xmknodat, iFiipup)
+#ifdef STATICBUILD
+//GO(__xmknod, iFipup)
+//GO(__xmknodat, iFiipup)
+#else
+GOM(__xmknod, iFEipup)
+GOM(__xmknodat, iFEiipup)
+#endif
 GO(__xpg_basename, pFp)
 GOW(__xpg_sigpause, iFi)
 GO(__xpg_strerror_r, pFipL)
@@ -2423,7 +2441,11 @@ DATAM(program_invocation_short_name, sizeof(void*))
 
 DATAM(__libc_single_threaded, 1)
 
+#ifdef STATICBUILD
+//GO(iconvctl, 
+#else
 GO(iconvctl, iFlip)
+#endif
 GO(dummy__ZnwmSt11align_val_tRKSt9nothrow_t, pFLLp) // for mallochook.c
 
 #ifdef ANDROID
@@ -2434,3 +2456,11 @@ GO(__errno, pFv)
 //GOM(__libc_init,
 //GO(__errno,
 #endif
+#ifdef STATICBUILD
+GO(dummy_pFLp, pFLp)
+GO(dummy_pFpLLp, pFpLLp)
+#else
+// not needed in no-static build
+//GO(dummy_pFLp, pFLp)
+//GO(dummy_pFpLLp, pFpLLp)
+#endif
