@@ -19,10 +19,16 @@
 #include "elfloader.h"
 #include "elfs/elfloader_private.h"
 
+typedef struct dllib_s {
+    library_t*  lib;
+    int         count;
+    int         dlopened;
+    int         is_self;
+    int         full;
+} dllib_t;
+
 typedef struct dlprivate_s {
-    library_t   **libs;
-    size_t      *count;
-    size_t      *dlopened;
+    dllib_t     *dllibs;
     size_t      lib_sz;
     size_t      lib_cap;
     char*       last_error;
@@ -37,8 +43,6 @@ void FreeDLPrivate(dlprivate_t **lib) {
     box_free(*lib);
 }
 
-static needed_libs_t dl_loaded = {0};
-
 // dead_cells consider the "2" value to be some king of issue?
 #define MIN_NLIB 3
 
@@ -53,11 +57,30 @@ void* my_dlvsym(x64emu_t* emu, void *handle, void *symbol, const char *vername)
 int my_dlinfo(x64emu_t* emu, void* handle, int request, void* info) EXPORT;
 
 #define LIBNAME libdl
-const char* libdlName = "libdl.so.2";
+#ifdef ANDROID
+    const char* libdlName = "libdl.so";
+#else
+    const char* libdlName = "libdl.so.2";
+#endif
 
-#define CLEARERR    if(dl->last_error) box_free(dl->last_error); dl->last_error = NULL;
+#define CLEARERR    if(dl->last_error) {box_free(dl->last_error); dl->last_error = NULL;}
+
+void RemoveDlopen(library_t** lib, size_t idx)
+{
+    if(!my_context)
+        return;
+    dlprivate_t *dl = my_context->dlprivate;
+    if(dl && idx<dl->lib_sz) {
+        if(lib!=&dl->dllibs[idx].lib)
+            dl->dllibs[idx].lib = NULL;
+        dl->dllibs[idx].count = 0;
+        dl->dllibs[idx].dlopened = 0;
+        dl->dllibs[idx].full = 0;
+    }
+}
 
 extern int box64_zoom;
+void setupTraceMapLib(lib_t* maplib);
 // Implementation
 void* my_dlopen(x64emu_t* emu, void *filename, int flag)
 {
@@ -68,6 +91,7 @@ void* my_dlopen(x64emu_t* emu, void *filename, int flag)
     dlprivate_t *dl = my_context->dlprivate;
     size_t dlopened = 0;
     int is_local = (flag&0x100)?0:1;  // if not global, then local, and that means symbols are not put in the global "pot" for other libs
+    int deepbind = (flag&0x0008)?1:0;  // RTLD_DEEPBIND means LOCAL before GLOBAL scope during symbol resolution
     CLEARERR
     if(filename) {
         char* rfilename = (char*)alloca(MAX_PATH);
@@ -112,50 +136,88 @@ void* my_dlopen(x64emu_t* emu, void *filename, int flag)
         }
         // check if alread dlopenned...
         for (size_t i=MIN_NLIB; i<dl->lib_sz; ++i) {
-            if(IsSameLib(dl->libs[i], rfilename)) {
+            if(dl->dllibs[i].full && IsSameLib(dl->dllibs[i].lib, rfilename)) {
                 if(flag&0x4) {   // don't re-open in RTLD_NOLOAD mode
-                    if(dl->count[i]==0 && dl->dlopened[i]) {
+                    if(dl->dllibs[i].count==0 && dl->dllibs[i].dlopened) {
                         printf_dlsym(LOG_DEBUG, " => not present anymore\n");
                         return NULL;    // don't re-open in RTLD_NOLOAD mode
                     }
                 }
-                IncRefCount(dl->libs[i], emu);
-                dl->count[i] = dl->count[i]+1;
-                printf_dlsym(LOG_DEBUG, "dlopen: Recycling %s/%p count=%ld (dlopened=%ld, elf_index=%d)\n", rfilename, (void*)(i+1), dl->count[i], dl->dlopened[i], GetElfIndex(dl->libs[i]));
+                IncRefCount(dl->dllibs[i].lib, emu);
+                ++dl->dllibs[i].count;
+                if(!is_local && isLibLocal(dl->dllibs[i].lib))
+                    promoteLocalLibGlobal(dl->dllibs[i].lib);
+                printf_dlsym(LOG_DEBUG, "dlopen: Recycling %s/%p count=%ld (dlopened=%ld, elf_index=%d)\n", rfilename, (void*)(i+1), dl->dllibs[i].count, dl->dllibs[i].dlopened, GetElfIndex(dl->dllibs[i].lib));
                 return (void*)(i+1);
             }
         }
+        lib = GetLibInternal(rfilename);
         if(flag&0x4) {   //RTLD_NOLOAD is just a "check" if lib is already loaded
+            if(lib) {
+                if(dl->lib_sz == dl->lib_cap) {
+                    dl->lib_cap += 4;
+                    dl->dllibs = (dllib_t*)box_realloc(dl->dllibs, sizeof(dllib_t)*dl->lib_cap);
+                    // memset count...
+                    memset(dl->dllibs+dl->lib_sz, 0, (dl->lib_cap-dl->lib_sz)*sizeof(dllib_t));
+                    if(!dl->lib_sz)
+                        dl->lib_sz = MIN_NLIB;
+                }
+                intptr_t idx = dl->lib_sz++;
+                dl->dllibs[idx].lib = lib;
+                ++dl->dllibs[idx].count;
+                dl->dllibs[idx].dlopened = dlopened;
+                dl->dllibs[idx].is_self = lib?0:1;
+                dl->dllibs[idx].full = 1;
+                IncRefCount(dl->dllibs[idx].lib, emu);
+                SetDlOpenIdx(lib, idx);
+                printf_dlsym(LOG_DEBUG, "dlopen: New handle %p (%s), dlopened=%ld\n", (void*)(idx+1), (char*)filename, dlopened);
+                return (void*)(idx+1);
+
+            }
             printf_dlsym(LOG_DEBUG, " => not present\n");
             return NULL;
         }
-        dlopened = (GetLibInternal(rfilename)==NULL);
+        dlopened = (lib==NULL);
+        // cleanup the old deferredInit state
+        int old_deferredInit = my_context->deferredInit;
+        my_context->deferredInit = 1;
+        elfheader_t** old_deferredInitList = my_context->deferredInitList;
+        my_context->deferredInitList = NULL;
+        int old_deferredInitSz = my_context->deferredInitSz;
+        int old_deferredInitCap = my_context->deferredInitCap;
+        my_context->deferredInitSz = my_context->deferredInitCap = 0;
+
         // Then open the lib
-        my_context->deferedInit = 1;
         int bindnow = (!box64_musl && (flag&0x2))?1:0;
-        needed_libs_t tmp = {0};
-        char* names[] = {rfilename};
-        library_t* libs[] = { NULL };
-        tmp.size = tmp.cap = 1;
-        tmp.names = names;
-        tmp.libs = libs;
-        if(AddNeededLib(NULL, is_local, bindnow, &tmp, my_context, emu)) {
+        needed_libs_t *tmp = new_neededlib(1);
+        tmp->names[0] = rfilename;
+        if(AddNeededLib(NULL, is_local, bindnow, deepbind, tmp, NULL, my_context, emu)) {
             printf_dlsym(strchr(rfilename,'/')?LOG_DEBUG:LOG_INFO, "Warning: Cannot dlopen(\"%s\"/%p, %X)\n", rfilename, filename, flag);
             if(!dl->last_error)
-                dl->last_error = box_malloc(129);
+                dl->last_error = box_calloc(1, 129);
             snprintf(dl->last_error, 129, "Cannot dlopen(\"%s\"/%p, %X)\n", rfilename, filename, flag);
+            RemoveNeededLib(NULL, is_local, tmp, my_context, emu);
+            if(my_context->deferredInitList)
+                box_free(my_context->deferredInitList);
+            my_context->deferredInit = old_deferredInit;
+            my_context->deferredInitList = old_deferredInitList;
+            my_context->deferredInitSz = old_deferredInitSz;
+            my_context->deferredInitCap = old_deferredInitCap;
             return NULL;
         }
-        add1_neededlib(&dl_loaded);
-        dl_loaded.names[dl_loaded.size-1] = tmp.names[0];
-        dl_loaded.libs[dl_loaded.size-1] = tmp.libs[0];
+        setupTraceMapLib(tmp->libs[0]->maplib);
+        free_neededlib(tmp);
         lib = GetLibInternal(rfilename);
-        RunDeferedElfInit(emu);
+        RunDeferredElfInit(emu);
+        my_context->deferredInit = old_deferredInit;
+        my_context->deferredInitList = old_deferredInitList;
+        my_context->deferredInitSz = old_deferredInitSz;
+        my_context->deferredInitCap = old_deferredInitCap;
     } else {
         // check if already dlopenned...
         for (size_t i=MIN_NLIB; i<dl->lib_sz; ++i) {
-            if(!dl->libs[i]) {
-                dl->count[i] = dl->count[i]+1;
+            if(dl->dllibs[i].is_self) {
+                ++dl->dllibs[i].count;
                 return (void*)(i+1);
             }
         }
@@ -165,18 +227,19 @@ void* my_dlopen(x64emu_t* emu, void *filename, int flag)
     
     if(dl->lib_sz == dl->lib_cap) {
         dl->lib_cap += 4;
-        dl->libs = (library_t**)box_realloc(dl->libs, sizeof(library_t*)*dl->lib_cap);
-        dl->count = (size_t*)box_realloc(dl->count, sizeof(size_t)*dl->lib_cap);
-        dl->dlopened = (size_t*)box_realloc(dl->dlopened, sizeof(size_t)*dl->lib_cap);
+        dl->dllibs = (dllib_t*)box_realloc(dl->dllibs, sizeof(dllib_t)*dl->lib_cap);
         // memset count...
-        memset(dl->count+dl->lib_sz, 0, (dl->lib_cap-dl->lib_sz)*sizeof(size_t));
+        memset(dl->dllibs+dl->lib_sz, 0, (dl->lib_cap-dl->lib_sz)*sizeof(dllib_t));
         if(!dl->lib_sz)
             dl->lib_sz = MIN_NLIB;
     }
-    intptr_t idx = dl->lib_sz++; 
-    dl->libs[idx] = lib;
-    dl->count[idx] = dl->count[idx]+1;
-    dl->dlopened[idx] = dlopened;
+    intptr_t idx = dl->lib_sz++;
+    dl->dllibs[idx].lib = lib;
+    ++dl->dllibs[idx].count;
+    dl->dllibs[idx].dlopened = dlopened;
+    dl->dllibs[idx].is_self = lib?0:1;
+    dl->dllibs[idx].full = 1;
+    SetDlOpenIdx(lib, idx);
     printf_dlsym(LOG_DEBUG, "dlopen: New handle %p (%s), dlopened=%ld\n", (void*)(idx+1), (char*)filename, dlopened);
     return (void*)(idx+1);
 }
@@ -196,7 +259,8 @@ char* my_dlerror(x64emu_t* emu)
 
 KHASH_SET_INIT_INT(libs);
 
-int recursive_dlsym_lib(kh_libs_t* collection, library_t* lib, const char* rsymbol, uintptr_t *start, uintptr_t *end, int version, const char* vername)
+// TODO: deepbind is probably not followed correctly here
+int recursive_dlsym_lib(kh_libs_t* collection, library_t* lib, const char* rsymbol, uintptr_t *start, uintptr_t *end, int* version, const char** vername, int* veropt)
 {
     if(!lib)
         return 0;
@@ -208,15 +272,15 @@ int recursive_dlsym_lib(kh_libs_t* collection, library_t* lib, const char* rsymb
     // TODO: should use librarian functions instead!
     int weak;
     // look in the library itself
-    if(lib->getglobal(lib, rsymbol, start, end, 0, &weak, version, vername, 1))
+    if(lib->getglobal(lib, rsymbol, start, end, 0, &weak, version, vername, 1, veropt, NULL))
         return 1;
-    if(lib->getweak(lib, rsymbol, start, end, 0, &weak, version, vername, 1))
+    if(lib->getweak(lib, rsymbol, start, end, 0, &weak, version, vername, 1, veropt, NULL))
         return 1;
     // look in other libs
     int n = GetNeededLibsN(lib);
     for (int i=0; i<n; ++i) {
         library_t *l = GetNeededLib(lib, i);
-        if(recursive_dlsym_lib(collection, l, rsymbol, start, end, version, vername))
+        if(recursive_dlsym_lib(collection, l, rsymbol, start, end, version, vername, veropt))
             return 1;
     }
         
@@ -226,42 +290,50 @@ int recursive_dlsym_lib(kh_libs_t* collection, library_t* lib, const char* rsymb
 int my_dlsym_lib(library_t* lib, const char* rsymbol, uintptr_t *start, uintptr_t *end, int version, const char* vername)
 {
     kh_libs_t *collection = kh_init(libs);
-    int ret = recursive_dlsym_lib(collection, lib, rsymbol, start, end, version, vername);
+    int veropt = 0;
+    int ret = recursive_dlsym_lib(collection, lib, rsymbol, start, end, &version, &vername, &veropt);
     kh_destroy(libs, collection);
 
     return ret;
 }
+int GetTID();
 void* my_dlsym(x64emu_t* emu, void *handle, void *symbol)
 {
     (void)emu;
+    static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+    pthread_mutex_lock(&mutex);
     dlprivate_t *dl = my_context->dlprivate;
     uintptr_t start = 0, end = 0;
     char* rsymbol = (char*)symbol;
     CLEARERR
-    printf_dlsym(LOG_DEBUG, "Call to dlsym(%p, \"%s\")%s", handle, rsymbol, dlsym_error?"":"\n");
+    printf_dlsym(LOG_DEBUG, "%04d|Call to dlsym(%p, \"%s\")%s", GetTID(), handle, rsymbol, dlsym_error?"":"\n");
     if(handle==NULL) {
         // special case, look globably
-        if(GetGlobalSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, NULL, -1, NULL)) {
+        if(GetGlobalSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, NULL, -1, NULL, 0, NULL)) {
             printf_dlsym(LOG_NEVER, "%p\n", (void*)start);
+            pthread_mutex_unlock(&mutex);
             return (void*)start;
         }
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Symbol \"%s\" not found in %p)\n", rsymbol, handle);
         printf_dlsym(LOG_NEVER, "%p\n", NULL);
+        pthread_mutex_unlock(&mutex);
         return NULL;
     }
     if(handle==(void*)~0LL) {
         // special case, look globably but no self (RTLD_NEXT)
         elfheader_t *elf = FindElfAddress(my_context, *(uintptr_t*)R_RSP); // use return address to guess "self"
-        if(GetNoSelfSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, elf, 0, -1, NULL)) {
+        if(GetNoSelfSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, elf, 0, -1, NULL, 0, NULL)) {
             printf_dlsym(LOG_NEVER, "%p\n", (void*)start);
+            pthread_mutex_unlock(&mutex);
             return (void*)start;
         }
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Symbol \"%s\" not found in %p)\n", rsymbol, handle);
         printf_dlsym(LOG_NEVER, "%p\n", NULL);
+        pthread_mutex_unlock(&mutex);
         return NULL;
     }
     size_t nlib = (size_t)handle;
@@ -269,46 +341,51 @@ void* my_dlsym(x64emu_t* emu, void *handle, void *symbol)
     // size_t is unsigned
     if(nlib>=dl->lib_sz) {
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Bad handle %p)\n", handle);
         printf_dlsym(LOG_NEVER, "%p\n", NULL);
+        pthread_mutex_unlock(&mutex);
         return NULL;
     }
-    if(dl->count[nlib]==0) {
+    if(!dl->dllibs[nlib].count || !dl->dllibs[nlib].full) {
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Bad handle %p (already closed))\n", handle);
         printf_dlsym(LOG_NEVER, "%p\n", (void*)NULL);
+        pthread_mutex_unlock(&mutex);
         return NULL;
     }
-    if(dl->libs[nlib]) {
-        if(my_dlsym_lib(dl->libs[nlib], rsymbol, &start, &end, -1, NULL)==0) {
+    if(dl->dllibs[nlib].lib) {
+        if(my_dlsym_lib(dl->dllibs[nlib].lib, rsymbol, &start, &end, -1, NULL)==0) {
             // not found
-            printf_dlsym(LOG_NEVER, "%p\nCall to dlsym(%s, \"%s\") Symbol not found\n", NULL, GetNameLib(dl->libs[nlib]), rsymbol);
+            printf_dlsym(LOG_NEVER, "%p\nCall to dlsym(%s, \"%s\") Symbol not found\n", NULL, GetNameLib(dl->dllibs[nlib].lib), rsymbol);
             printf_log(LOG_DEBUG, " Symbol not found\n");
             if(!dl->last_error)
-                dl->last_error = box_malloc(129);
-            snprintf(dl->last_error, 129, "Symbol \"%s\" not found in %p(%s)", rsymbol, handle, GetNameLib(dl->libs[nlib]));
+                dl->last_error = box_calloc(1, 129);
+            snprintf(dl->last_error, 129, "Symbol \"%s\" not found in %p(%s)", rsymbol, handle, GetNameLib(dl->dllibs[nlib].lib));
+            pthread_mutex_unlock(&mutex);
             return NULL;
         }
     } else {
         // still usefull?
         //  => look globably
-        const char* defver = GetDefaultVersion(my_context->globaldefver, rsymbol);
-        if(!defver) defver = GetDefaultVersion(my_context->weakdefver, rsymbol);
-        if(GetGlobalSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, NULL, -1, defver)) {
+        if(GetGlobalSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, NULL, -1, NULL, 0, NULL)) {
             printf_dlsym(LOG_NEVER, "%p\n", (void*)start);
+            pthread_mutex_unlock(&mutex);
             return (void*)start;
         }
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Symbol \"%s\" not found in %p)\n", rsymbol, handle);
         printf_dlsym(LOG_NEVER, "%p\n", NULL);
+        pthread_mutex_unlock(&mutex);
         return NULL;
     }
     printf_dlsym(LOG_NEVER, "%p\n", (void*)start);
+    pthread_mutex_unlock(&mutex);
     return (void*)start;
 }
+static int actualy_closing = 0;
 int my_dlclose(x64emu_t* emu, void *handle)
 {
     (void)emu;
@@ -320,22 +397,33 @@ int my_dlclose(x64emu_t* emu, void *handle)
     // size_t is unsigned
     if(nlib>=dl->lib_sz) {
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Bad handle %p)\n", handle);
         printf_dlsym(LOG_DEBUG, "dlclose: %s\n", dl->last_error);
         return -1;
     }
-    if(dl->count[nlib]==0) {
+    if(!dl->dllibs[nlib].count || !dl->dllibs[nlib].full) {
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Bad handle %p (already closed))\n", handle);
         printf_dlsym(LOG_DEBUG, "dlclose: %s\n", dl->last_error);
         return -1;
     }
-    dl->count[nlib] = dl->count[nlib]-1;
-    FiniLibrary(dl->libs[nlib], emu);
+    --dl->dllibs[nlib].count;
+    elfheader_t* h = GetElf(dl->dllibs[nlib].lib);
+    if((h && !h->gnuunique) || !h || actualy_closing)
+        DecRefCount(&dl->dllibs[nlib].lib, emu);
     return 0;
 }
+#ifdef ANDROID
+#ifndef RTLD_DL_SYMENT
+#define RTLD_DL_SYMENT 1
+#endif
+#ifndef RTLD_DL_LINKMAP
+#define RTLD_DL_LINKMAP 2
+#endif
+#endif
+
 int my_dladdr1(x64emu_t* emu, void *addr, void *i, void** extra_info, int flags)
 {
     //int dladdr(void *addr, Dl_info *info);
@@ -353,7 +441,7 @@ int my_dladdr1(x64emu_t* emu, void *addr, void *i, void** extra_info, int flags)
     if(flags==RTLD_DL_SYMENT) {
         printf_log(LOG_INFO, "Warning, unimplement call to dladdr1 with RTLD_DL_SYMENT flags\n");
     } else if (flags==RTLD_DL_LINKMAP) {
-        printf_log(LOG_INFO, "Warning, partially unimplemented call to dladdr1 with RTLD_DL_LINKMAP flags\n");
+        printf_log(LOG_INFO, "Warning, partially unimplemented call to dladdr1 with RTLD_DL_LINKMAP flags for %s\n", lib?lib->name:"current process");
         *(linkmap_t**)extra_info = getLinkMapLib(lib);
     }
     return (info->dli_sname)?1:0;   // success is non-null here...
@@ -372,12 +460,12 @@ void* my_dlvsym(x64emu_t* emu, void *handle, void *symbol, const char *vername)
     printf_dlsym(LOG_DEBUG, "Call to dlvsym(%p, \"%s\", %s)%s", handle, rsymbol, vername?vername:"(nil)", dlsym_error?"":"\n");
     if(handle==NULL) {
         // special case, look globably
-        if(GetGlobalSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, NULL, version, vername)) {
+        if(GetGlobalSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, NULL, version, vername, 0, NULL)) {
             printf_dlsym(LOG_NEVER, "%p\n", (void*)start);
             return (void*)start;
         }
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Symbol \"%s\" version %s not found in %p)\n", rsymbol, vername?vername:"(nil)", handle);
             printf_dlsym(LOG_NEVER, "%p\n", NULL);
         return NULL;
@@ -385,12 +473,12 @@ void* my_dlvsym(x64emu_t* emu, void *handle, void *symbol, const char *vername)
     if(handle==(void*)~0LL) {
         // special case, look globably but no self (RTLD_NEXT)
         elfheader_t *elf = FindElfAddress(my_context, *(uintptr_t*)R_RSP); // use return address to guess "self"
-        if(GetNoSelfSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, elf, 0, version, vername)) {
+        if(GetNoSelfSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, elf, 0, version, vername, 0, NULL)) {
                 printf_dlsym(LOG_NEVER, "%p\n", (void*)start);
             return (void*)start;
         }
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Symbol \"%s\" version %s not found in %p)\n", rsymbol, vername?vername:"(nil)", handle);
             printf_dlsym(LOG_NEVER, "%p\n", NULL);
         return NULL;
@@ -400,33 +488,31 @@ void* my_dlvsym(x64emu_t* emu, void *handle, void *symbol, const char *vername)
     // size_t is unsigned
     if(nlib>=dl->lib_sz) {
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Bad handle %p)\n", handle);
             printf_dlsym(LOG_NEVER, "%p\n", NULL);
         return NULL;
     }
-    if(dl->count[nlib]==0) {
+    if(!dl->dllibs[nlib].count || !dl->dllibs[nlib].full) {
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Bad handle %p (already closed))\n", handle);
             printf_dlsym(LOG_NEVER, "%p\n", (void*)NULL);
         return NULL;
     }
-    if(dl->libs[nlib]) {
-        if(my_dlsym_lib(dl->libs[nlib], rsymbol, &start, &end, version, vername)==0) {
+    if(dl->dllibs[nlib].lib) {
+        if(my_dlsym_lib(dl->dllibs[nlib].lib, rsymbol, &start, &end, version, vername)==0) {
             // not found
-                printf_dlsym(LOG_NEVER, "%p\nCall to dlvsym(%s, \"%s\", %s) Symbol not found\n", NULL, GetNameLib(dl->libs[nlib]), rsymbol, vername?vername:"(nil)");
+                printf_dlsym(LOG_NEVER, "%p\nCall to dlvsym(%s, \"%s\", %s) Symbol not found\n", NULL, GetNameLib(dl->dllibs[nlib].lib), rsymbol, vername?vername:"(nil)");
             printf_log(LOG_DEBUG, " Symbol not found\n");
             if(!dl->last_error)
-                dl->last_error = box_malloc(129);
-            snprintf(dl->last_error, 129, "Symbol \"%s\" not found in %p(%s)", rsymbol, handle, GetNameLib(dl->libs[nlib]));
+                dl->last_error = box_calloc(1, 129);
+            snprintf(dl->last_error, 129, "Symbol \"%s\" not found in %p(%s)", rsymbol, handle, GetNameLib(dl->dllibs[nlib].lib));
             return NULL;
         }
     } else {
         // still usefull?
-         const char* defver = GetDefaultVersion(my_context->globaldefver, rsymbol);
-        if(!defver) defver = GetDefaultVersion(my_context->weakdefver, rsymbol);
-        if(GetGlobalSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, NULL, -1, defver)) {
+        if(GetGlobalSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, NULL, -1, NULL, 0, NULL)) {
             printf_dlsym(LOG_NEVER, "%p\n", (void*)start);
             return (void*)start;
         }
@@ -434,7 +520,7 @@ void* my_dlvsym(x64emu_t* emu, void *handle, void *symbol, const char *vername)
             printf_dlsym(LOG_NEVER, "%p\nCall to dlvsym(%s, \"%s\", %s) Symbol not found\n", NULL, "Self", rsymbol, vername?vername:"(nil)");
         printf_log(LOG_DEBUG, " Symbol not found\n");
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Symbol \"%s\" version %s not found in %p)\n", rsymbol, vername?vername:"(nil)", handle);
         return NULL;
     }
@@ -453,19 +539,19 @@ int my_dlinfo(x64emu_t* emu, void* handle, int request, void* info)
     // size_t is unsigned
     if(nlib>=dl->lib_sz) {
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Bad handle %p)\n", handle);
         printf_dlsym(LOG_DEBUG, "dlinfo: %s\n", dl->last_error);
         return -1;
     }
-    if(dl->count[nlib]==0) {
+    if(!dl->dllibs[nlib].count || !dl->dllibs[nlib].full) {
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "Bad handle %p (already closed))\n", handle);
         printf_dlsym(LOG_DEBUG, "dlinfo: %s\n", dl->last_error);
         return -1;
     }
-    library_t *lib = dl->libs[nlib];
+    library_t *lib = dl->dllibs[nlib].lib;
     //elfheader_t *h = (GetElfIndex(lib)>-1)?my_context->elfs[GetElfIndex(lib)]:NULL;
     switch(request) {
         case 2: // RTLD_DI_LINKMAP
@@ -476,7 +562,7 @@ int my_dlinfo(x64emu_t* emu, void* handle, int request, void* info)
         default:
             printf_log(LOG_NONE, "Warning, unsupported call to dlinfo(%p, %d, %p)\n", handle, request, info);
         if(!dl->last_error)
-            dl->last_error = box_malloc(129);
+            dl->last_error = box_calloc(1, 129);
         snprintf(dl->last_error, 129, "unsupported call to dlinfo request:%d\n", request);
     }
     return -1;
@@ -493,13 +579,42 @@ typedef struct my_dl_find_object_s {
 
 EXPORT int my__dl_find_object(x64emu_t* emu, void* addr, my_dl_find_object_t* result)
 {
-    printf_log(LOG_INFO, "Unimplemented _dl_find_object called\n");
+    //printf_log(LOG_INFO, "Unimplemented _dl_find_object called\n");
+    uintptr_t start=0, sz=0;
+    elfheader_t* h = FindElfAddress(my_context, (uintptr_t)addr);
+    if(h) {
+        // find an actual elf
+        /*const char* name =*/ FindNearestSymbolName(h, addr, &start, &sz);
+        result->dlfo_map_start = (void*)start;
+        result->dlfo_map_end = (void*)(start+sz-1);
+        result->dlfo_eh_frame = (void*)(h->ehframehdr+h->delta);
+        result->dlfo_flags = 0;   // unused it seems
+        result->dlf_link_map = (struct link_map *)getLinkMapElf(h);
+        return 0;
+    }
     return -1;
 }
 
-#define CUSTOM_INIT\
-    if(!box64_isglibc234) setNeededLibs(lib, 1, "libc.so.6");
+void closeAllDLOpenned()
+{
+    dlprivate_t *dl = my_context->dlprivate;
+    actualy_closing = 1;
+    if(dl) {
+        x64emu_t* emu = thread_get_emu();
+        for(size_t i=0; i<dl->lib_sz; ++i)
+            while(dl->dllibs[i].count) {
+                printf_log(LOG_DEBUG, "  closing %s\n", dl->dllibs[i].lib->name);
+                my_dlclose(emu, (void*)(i+1));
+            }
+    }
+}
 
+#ifdef STATICBUILD
+//extern void* _dlfcn_hook;
+#endif
+
+#define CUSTOM_FINI \
+    closeAllDLOpenned();
 
 // define all standard library functions
 #include "wrappedlib_init.h"
