@@ -41,7 +41,11 @@ GOW(atan2, dFdd)
 GOW(atan2f, fFff)
 GOM(__atan2f_finite, fFff)
 GOM(__atan2_finite, dFdd)
-// atan2l   // Weak
+#ifdef HAVE_LD80BITS
+GOW(atan2l, lFDD)
+#else
+GO2(atan2l, lFKK, atan2)
+#endif
 GOW(atanf, fFf)
 GOW(atanh, dFd)
 GOW(atanhf, fFf)
@@ -49,13 +53,18 @@ GOW(atanhf, fFf)
 // __atanh_finite
 #ifdef HAVE_LD80BITS
 GOW(atanhl, DFD)
+GOW(atanl, DFD)
 #else
 GO2(atanhl, KFK, atanh)
+GO2(atanl, KFK, atan)
 #endif
-// atanl    // Weak
 GOW(cabs, XFX)
 GOW(cabsf, xFx)
-// cabsl    // Weak
+#ifdef HAVE_LD80BITS
+GOW(cabsl, DFY)
+#else
+GO2(cabsl, KFy, cabs)
+#endif
 GOW(cacos, XFX)
 GOW(cacosf, xFx)
 GOW(cacosh, XFX)
@@ -71,10 +80,10 @@ GOW(casinh, XFX)
 GOW(casinhf, xFx)
 // casinhl  // Weak
 // casinl   // Weak
-GOS(catan, XFX)
-GOM(catanf, xFx)
-GOS(catanh, XFX)
-GOM(catanhf, xFx)
+GOW(catan, XFX)
+GOW(catanf, xFx)
+GOW(catanh, XFX)
+GOW(catanhf, xFx)
 // catanhl  // Weak
 // catanl   // Weak
 GOW(cbrt, dFd)
@@ -88,8 +97,13 @@ GOW(ccos, XFX)
 GOW(ccosf, xFx)
 GOW(ccosh, XFX)
 GOW(ccoshf, xFx)
-// ccoshl   // Weak
-// ccosl    // Weak
+#ifdef HAVE_LD80BITS
+GOW(ccoshl, YFY)
+GOW(ccosl, YFY)
+#else
+GO2(ccoshl, yFy, ccosh)
+GO2(ccosl, yFy, ccos)
+#endif
 GOW(ceil, dFd)
 GOW(ceilf, fFf)
 #ifdef HAVE_LD80BITS
@@ -99,7 +113,11 @@ GO2(ceill, KFK, ceil)
 #endif
 GOW(cexp, XFX)
 GOW(cexpf, xFx)
-// cexpl    // Weak
+#ifdef HAVE_LD80BITS
+GOW(cexpl, YFY)
+#else
+GO2(cexpl, yFy, cexp)
+#endif
 // cimag    // Weak
 // cimagf   // Weak
 // cimagl   // Weak
@@ -110,8 +128,12 @@ GOW(clog, XFX)
 // __clog10f
 // clog10l  // Weak
 // __clog10l
-//GOM(clogf, UFV)    // float complex doesn't trigger the ret 4, but return an u64!
-// clogl    // Weak
+GOW(clogf, xFx)
+#ifdef HAVE_LD80BITS
+GOW(clogl, YFY)
+#else
+GO2(clogl, yFy, clog)
+#endif
 // conj // Weak
 // conjf    // Weak
 // conjl    // Weak
@@ -124,11 +146,20 @@ GOW(cosh, dFd)
 GOW(coshf, fFf)
 GOM(__coshf_finite, fFf)
 GOM(__cosh_finite, dFd)
-// coshl    // Weak
-// cosl // Weak
+#ifdef HAVE_LD80BITS
+GOW(coshl, DFD)
+GOW(cosl, DFD)
+#else
+GO2(coshl, KFK, cosh)
+GO2(cosl, KFK, cos)
+#endif
 GOW(cpow, XFXX)
 GOW(cpowf, xFxx)
-// cpowl    // Weak
+#ifdef HAVE_LD80BITS
+GOW(cpowl, YFYY)
+#else
+GO2(cpowl, yFyy, cpow)
+#endif
 GOW(cproj, XFX)
 GOW(cprojf, xFx)
 // cprojl   // Weak
@@ -139,17 +170,31 @@ GOW(csin, XFX)
 GOW(csinf, xFx)
 GOW(csinh, XFX)
 GOW(csinhf, xFx)
-// csinhl   // Weak
-// csinl    // Weak
+#ifdef HAVE_LD80BITS
+GOW(csinhl, YFY)
+GOW(csinl, YFY)
+#else
+GO2(csinhl, yFy, csinh)
+GO2(csinl, yFy, csin)
+#endif
 GOW(csqrt, XFX)
 GOW(csqrtf, xFx)
-// csqrtl   // Weak
+#ifdef HAVE_LD80BITS
+GOW(csqrtl, YFY)
+#else
+GO2(csqrtl, yFy, csqrt)
+#endif
 GOW(ctan, XFX)
 GOW(ctanf, xFx)
 GOW(ctanh, XFX)
 GOW(ctanhf, xFx)
-// ctanhl   // Weak
-// ctanl    // Weak
+#ifdef HAVE_LD80BITS
+GOW(ctanhl, YFY)
+GOW(ctanl, YFY)
+#else
+GO2(ctanhl, yFy, ctanh)
+GO2(ctanl, yFy, ctan)
+#endif
 // __cxa_finalize  // Weak
 // drem // Weak
 // dremf    // Weak
@@ -178,7 +223,11 @@ GOW(exp2, dFd)
 GOW(exp2f, fFf)
 GOM(__exp2f_finite, fFf)
 GOM(__exp2_finite, dFd)
-// exp2l    // Weak
+#ifdef HAVE_LD80BITS
+GOW(exp2l, DFD)
+#else
+GO2(exp2l, KFK, exp2)
+#endif
 GOW(expf, fFf)
 GOM(__expf_finite, fFf)
 GOM(__exp_finite, dFd)
@@ -189,7 +238,11 @@ GO2(expl, KFK, exp)
 #endif
 GOW(expm1, dFd)
 GOW(expm1f, fFf)
-// expm1l   // Weak
+#ifdef HAVE_LD80BITS
+GOW(expm1l, DFD)
+#else
+GO2(expm1l, KFK, expm1)
+#endif
 GOW(fabs, dFd)
 GOW(fabsf, fFf)
 // fabsl    // Weak
@@ -225,7 +278,11 @@ GO2(floorl, KFK, floor)
 #endif
 GOW(fma, dFddd)
 GOW(fmaf, fFfff)
-// fmal // Weak
+#ifdef HAVE_LD80BITS
+GOW(fmal, DFDDD)
+#else
+GO2(fmal, KFKKK, fma)
+#endif
 GOW(fmax, dFdd)
 GOW(fmaxf, fFff)
 // fmaxl    // Weak
@@ -260,7 +317,11 @@ GOW(hypot, dFdd)
 GOW(hypotf, fFff)
 GOM(__hypotf_finite, fFff)
 GOM(__hypot_finite, dFdd)
-// hypotl   // Weak
+#ifdef HAVE_LD80BITS
+GOW(hypotl, DFDD)
+#else
+GO2(hypotl, KFKK, hypot)
+#endif
 GOW(ilogb, iFd)
 GOW(ilogbf, iFf)
 // ilogbl   // Weak
@@ -282,7 +343,11 @@ GO(jn, dFid)
 GO(jnf, fFif)
 // __jnf_finite
 // __jn_finite
-// jnl
+#ifdef HAVE_LD80BITS
+GO(jnl, DFiD)
+#else
+GO2(jnl, KFiK, jn)
+#endif
 GOW(ldexp, dFdi)
 GOW(ldexpf, fFfi)
 #ifdef HAVE_LD80BITS
@@ -306,7 +371,11 @@ GO2(lgammal_r, KFKp, lgamma_r)
 #endif
 GOW(lgamma_r, dFdp)
 // __lgamma_r_finite
+#ifdef STATICBUILD
+//DATAV(_LIB_VERSION, 8)
+#else
 DATAV(_LIB_VERSION, 8)
+#endif
 GOWM(llrint, IFEd)
 GOWM(llrintf, IFEf)
 #ifdef HAVE_LD80BITS
@@ -329,12 +398,20 @@ GO2(log10l, KFK, log10) //Weak
 #endif
 GOW(log1p, dFd)
 GOW(log1pf, fFf)
-// log1pl   // Weak
+#ifdef HAVE_LD80BITS
+GOW(log1pl, DFD)
+#else
+GO2(log1pl, KFK, log1p)
+#endif
 GOW(log2, dFd)
 GOW(log2f, fFf)
 GOM(__log2f_finite, fFf)
 GOM(__log2_finite, dFd)
-// log2l    // Weak
+#ifdef HAVE_LD80BITS
+GOW(log2l, DFD)
+#else
+GO2(log2l, KFK, log2)
+#endif
 GOW(logb, dFd)
 GOW(logbf, fFf)
 // logbl    // Weak
@@ -351,7 +428,11 @@ GOWM(lrintf, iFEf)
 // lrintl   // Weak
 GOW(lround, iFd)
 GOW(lroundf, iFf)
-// lroundl  // Weak
+#ifdef HAVE_LD80BITS
+GOW(lroundl, lFD)
+#else
+GO2(lroundl, lFK, lround)
+#endif
 // matherr  // Weak
 GOW(modf, dFdp)
 GOW(modff, fFfp)
@@ -364,17 +445,21 @@ GOWM(nearbyintf, fFEf)
 // nearbyintl   // Weak
 GOW(nextafter, dFdd)
 GOW(nextafterf, fFff)
-// nextafterl   // Weak
+#ifdef HAVE_LD80BITS
+GOW(nextafterl, DFDD)
+#else
+GO2(nextafterl, KFKK, nextafter)
+#endif
 GOW(nexttoward, dFdD)
 GOW(nexttowardf, fFfD)
 // nexttowardl  // Weak
 GOW(pow, dFdd)
-GOW(pow10, dFd)
-GOW(pow10f, fFf)
+GOWM(pow10, dFd)
+GOWM(pow10f, fFf)
 #ifdef HAVE_LD80BITS
-GOW(pow10l, DFD)
+GOWM(pow10l, DFD)
 #else
-GOW(pow10l, KFK)
+GOWM(pow10l, KFK)
 #endif
 GOW(powf, fFff)
 GOM(__powf_finite, fFff)
@@ -400,7 +485,11 @@ GOW(roundf, fFf)
 GO(roundeven, dFd)  //since C23
 GO(roundevenf, fFf)
 //GO(roundevenl, DFD)
-// roundl   // Weak
+#ifdef HAVE_LD80BITS
+GOW(roundl, DFD)
+#else
+GO2(roundl, KFK, round)
+#endif
 // scalb    // Weak
 // scalbf   // Weak
 // __scalbf_finite
@@ -421,14 +510,23 @@ DATAB(signgam, 8)
 GOW(sin, dFd)
 GOW(sincos, vFdpp)
 GOW(sincosf, vFfpp)
-// sincosl  // Weak
+#ifdef HAVE_LD80BITS
+GOW(sincosl, vFDpp)
+#else
+GO2(sincosl, vFKpp, sincos)
+#endif
 GOW(sinf, fFf)
 GOW(sinh, dFd)
 GOW(sinhf, fFf)
 GOM(__sinhf_finite, fFf)
 GOM(__sinh_finite, dFd)
-// sinhl    // Weak
-// sinl // Weak
+#ifdef HAVE_LD80BITS
+GOW(sinhl, DFD)
+GOW(sinl, DFD)
+#else
+GO2(sinhl, KFK, sinh)
+GO2(sinl, KFK, sin)
+#endif
 GOW(sqrt, dFd)
 GOW(sqrtf, fFf)
 GOM(__sqrtf_finite, fFf)
@@ -442,8 +540,13 @@ GO(tan, dFd)
 GOW(tanf, fFf)
 GOW(tanh, dFd)
 GOW(tanhf, fFf)
-// tanhl    // Weak
-// tanl // Weak
+#ifdef HAVE_LD80BITS
+GOW(tanhl, DFD)
+GOW(tanl, DFD)
+#else
+GO2(tanhl, KFK, tanh)
+GO2(tanl, KFK, tan)
+#endif
 GOW(tgamma, dFd)
 GOW(tgammaf, fFf)
 #ifdef HAVE_LD80BITS
@@ -468,4 +571,8 @@ GO(yn, dFid)
 GO(ynf, fFif)
 // __ynf_finite
 // __yn_finite
-// ynl
+#ifdef HAVE_LD80BITS
+GO(ynl, DFiD)
+#else
+GO2(ynl, KFiK, yn)
+#endif
