@@ -19,7 +19,7 @@ GO(SmcRelease, pFp)
 GO(SmcRequestSaveYourself, vFpiiiii)
 GOM(SmcRequestSaveYourselfPhase2, iFEppp)
 GO(SmcSaveYourselfDone, vFpi)
-//GOM(SmcSetErrorHandler, BFEB)
+GOM(SmcSetErrorHandler, pFEp)
 GO(SmcSetProperties, vFpip)
 GO(SmcVendor, pFp)
 GO(SmFreeProperty, vFp)
@@ -43,3 +43,5 @@ GO(SmsSaveYourself, vFpiiii)
 GO(SmsSaveYourselfPhase2, vFp)
 //GOM(SmsSetErrorHandler, BFEB)
 GO(SmsShutdownCancelled, vFp)
+
+GO(dummy_vFpiiLiip, vFpiiLiip) // for error handler
\ No newline at end of file
