@@ -391,26 +391,9 @@ EXPORT void* my_XextAddDisplay(x64emu_t* emu, void* extinfo, void* dpy, void* ex
 }
 
 #ifdef ANDROID
-    #define CUSTOM_INIT                 \
-        getMy(lib);                     \
-        setNeededLibs(lib, 5,           \
-            "libX11.so",              \
-            "libxcb.so",              \
-            "libXau.so",              \
-            "libdl.so",               \
-            "libXdmcp.so");
+#define NEEDED_LIBS "libX11.so", "libxcb.so", "libXau.so", "libdl.so", "libXdmcp.so"
 #else
-    #define CUSTOM_INIT                 \
-        getMy(lib);                     \
-        setNeededLibs(lib, 5,           \
-            "libX11.so.6",              \
-            "libxcb.so.1",              \
-            "libXau.so.6",              \
-            "libdl.so.2",               \
-            "libXdmcp.so.6");
+#define NEEDED_LIBS "libX11.so.6", "libxcb.so.1", "libXau.so.6", "libdl.so.2", "libXdmcp.so.6"
 #endif
 
-#define CUSTOM_FINI \
-    freeMy();
-
 #include "wrappedlib_init.h"
