@@ -20,20 +20,9 @@
 #define LIBNAME libxfixes
 
 #ifdef ANDROID
-    #define CUSTOM_INIT \
-        setNeededLibs(lib, 4,           \
-            "libX11.so",              \
-            "libxcb.so",              \
-            "libXau.so",              \
-            "libXdmcp.so");
+#define NEEDED_LIBS "libX11.so", "libxcb.so", "libXau.so", "libXdmcp.so"
 #else
-    #define CUSTOM_INIT \
-        setNeededLibs(lib, 4,           \
-            "libX11.so.6",              \
-            "libxcb.so.1",              \
-            "libXau.so.6",              \
-            "libXdmcp.so.6");
+#define NEEDED_LIBS "libX11.so.6", "libxcb.so.1", "libXau.so.6", "libXdmcp.so.6"
 #endif
 
 #include "wrappedlib_init.h"
-
