@@ -11,10 +11,18 @@
 #include "librarian/library_private.h"
 #include "x64emu.h"
 
-const char* libxssName = "libXss.so.1";
+#ifdef ANDROID
+    const char* libxssName = "libXss.so";
+#else
+    const char* libxssName = "libXss.so.1";
+#endif
+
 #define LIBNAME libxss
 
-#define CUSTOM_INIT \
-    setNeededLibs(lib, 2, "libX11.so.6", "libXext.so.6");
+#ifdef ANDROID
+#define NEEDED_LIBS "libX11.so", "libXext.so"
+#else
+#define NEEDED_LIBS "libX11.so.6", "libXext.so.6"
+#endif
 
 #include "wrappedlib_init.h"
