@@ -566,7 +566,7 @@ static int get_sdl_priv(x64emu_t* emu, const char *sym_str, void **w, void **f)
 
 int EXPORT my2_SDL_DYNAPI_entry(x64emu_t* emu, uint32_t version, uintptr_t *table, uint32_t tablesize)
 {
-    int i = 0;
+    uint32_t i = 0;
     uintptr_t tab[tablesize];
     int r = my->SDL_DYNAPI_entry(version, tab, tablesize);
     (void)r;
@@ -661,19 +661,19 @@ EXPORT void* my2_SDL_GL_GetProcAddress(x64emu_t* emu, void* name)
     return getGLProcAddress(emu, (glprocaddress_t)my->SDL_GL_GetProcAddress, rname);
 }
 
-#define nb_once	16
+#define nb_once 16
 typedef void(*sdl2_tls_dtor)(void*);
 static uintptr_t dtor_emu[nb_once] = {0};
 static void tls_dtor_callback(int n, void* a)
 {
-	if(dtor_emu[n]) {
+    if(dtor_emu[n]) {
         RunFunctionFmt(dtor_emu[n], "p", a);
-	}
+    }
 }
 #define GO(N) \
 void tls_dtor_callback_##N(void* a) \
 { \
-	tls_dtor_callback(N, a); \
+    tls_dtor_callback(N, a); \
 }
 
 GO(0)
@@ -694,26 +694,26 @@ GO(14)
 GO(15)
 #undef GO
 static const sdl2_tls_dtor dtor_cb[nb_once] = {
-	 tls_dtor_callback_0, tls_dtor_callback_1, tls_dtor_callback_2, tls_dtor_callback_3
-	,tls_dtor_callback_4, tls_dtor_callback_5, tls_dtor_callback_6, tls_dtor_callback_7
-	,tls_dtor_callback_8, tls_dtor_callback_9, tls_dtor_callback_10,tls_dtor_callback_11
-	,tls_dtor_callback_12,tls_dtor_callback_13,tls_dtor_callback_14,tls_dtor_callback_15
+     tls_dtor_callback_0, tls_dtor_callback_1, tls_dtor_callback_2, tls_dtor_callback_3
+    ,tls_dtor_callback_4, tls_dtor_callback_5, tls_dtor_callback_6, tls_dtor_callback_7
+    ,tls_dtor_callback_8, tls_dtor_callback_9, tls_dtor_callback_10,tls_dtor_callback_11
+    ,tls_dtor_callback_12,tls_dtor_callback_13,tls_dtor_callback_14,tls_dtor_callback_15
 };
 EXPORT int64_t my2_SDL_TLSSet(x64emu_t* emu, uint64_t id, void* value, void* dtor)
 {
-	if(!dtor)
-		return my->SDL_TLSSet(id, value, NULL);
-	int n = 0;
-	while (n<nb_once) {
-		if(!dtor_emu[n] || (dtor_emu[n])==((uintptr_t)dtor)) {
-			dtor_emu[n] = (uintptr_t)dtor;
-			return my->SDL_TLSSet(id, value, dtor_cb[n]);
-		}
-		++n;
-	}
-	printf_log(LOG_NONE, "Error: SDL2 SDL_TLSSet with destructor: no more slot!\n");
-	//emu->quit = 1;
-	return -1;
+    if(!dtor)
+        return my->SDL_TLSSet(id, value, NULL);
+    int n = 0;
+    while (n<nb_once) {
+        if(!dtor_emu[n] || (dtor_emu[n])==((uintptr_t)dtor)) {
+            dtor_emu[n] = (uintptr_t)dtor;
+            return my->SDL_TLSSet(id, value, dtor_cb[n]);
+        }
+        ++n;
+    }
+    printf_log(LOG_NONE, "Error: SDL2 SDL_TLSSet with destructor: no more slot!\n");
+    //emu->quit = 1;
+    return -1;
 }
 
 EXPORT void my2_SDL_AddEventWatch(x64emu_t* emu, void* p, void* userdata)
@@ -820,17 +820,28 @@ EXPORT void my2_SDL_GetJoystickGUIDInfo(SDL_JoystickGUID guid, uint16_t *vend, u
     }
 }
 
+EXPORT unsigned long my2_SDL_GetThreadID(x64emu_t* emu, void* thread)
+{
+    unsigned long ret = my->SDL_GetThreadID(thread);
+    int max = 10;
+    while (!ret && max--) {
+        sched_yield();
+        ret = my->SDL_GetThreadID(thread);
+    }
+    return ret;
+}
+
+#undef HAS_MY
+
+#define ALTMY my2_
+
 #define CUSTOM_INIT \
     box64->sdl2lib = lib;                   \
     getMy(lib);                             \
     box64->sdl2allocrw = my->SDL_AllocRW;   \
-    box64->sdl2freerw  = my->SDL_FreeRW;    \
-    SETALT(my2_);                           \
-    setNeededLibs(lib, 4,                   \
-        "libdl.so.2",                       \
-        "libm.so.6",                        \
-        "librt.so.1",                       \
-        "libpthread.so.0");
+    box64->sdl2freerw  = my->SDL_FreeRW;
+
+#define NEEDED_LIBS "libdl.so.2", "libm.so.6", "librt.so.1", "libpthread.so.0"
 
 #define CUSTOM_FINI \
     my->SDL_Quit();                                             \
@@ -841,5 +852,4 @@ EXPORT void my2_SDL_GetJoystickGUIDInfo(SDL_JoystickGUID guid, uint16_t *vend, u
     my_context->sdl2allocrw = NULL;                             \
     my_context->sdl2freerw = NULL;
 
-
 #include "wrappedlib_init.h"
