@@ -3,6 +3,7 @@
 #endif
 
 //%S G SDL_JoystickGUID UU
+// SDL_GUID is struct uin8_t data[16] => UU when in args, H when is return
 
 // _fini
 // _init
@@ -94,6 +95,7 @@ GO(SDL_DestroyRenderer, vFp)
 GO(SDL_DestroySemaphore, vFp)
 GO(SDL_DestroyTexture, vFp)
 GO(SDL_DestroyWindow, vFp)
+GO(SDL_DestroyWindowSurface, iFp)
 GO(SDL_DetachThread, vFp)
 GO(SDL_DisableScreenSaver, vFv)
 GO(SDL_DuplicateSurface, pFp)
@@ -136,6 +138,7 @@ GO(SDL_GameControllerGetBindForAxis, HFpi)
 GO(SDL_GameControllerGetBindForButton, HFpi)
 GO(SDL_GameControllerGetButton, CFpi)
 GO(SDL_GameControllerGetButtonFromString, iFp)
+GO(SDL_GameControllerGetFirmwareVersion, WFp)
 GO(SDL_GameControllerGetJoystick, pFp)
 GO(SDL_GameControllerGetNumTouchpads, iFp)
 GO(SDL_GameControllerGetNumTouchpadFingers, iFpi)
@@ -143,6 +146,7 @@ GO(SDL_GameControllerGetPlayerIndex, iFp)
 GO(SDL_GameControllerGetProduct, WFp)
 GO(SDL_GameControllerGetProductVersion, WFp)
 GO(SDL_GameControllerGetSensorData, iFpipi)
+GO(SDL_GameControllerGetSensorDataWithTimestamp, iFpippi)
 GO(SDL_GameControllerGetSerial, pFp)
 GO(SDL_GameControllerGetStringForAxis, pFi)
 GO(SDL_GameControllerGetStringForButton, pFi)
@@ -151,6 +155,7 @@ GO(SDL_GameControllerGetType, uFp)
 GO(SDL_GameControllerGetVendor, WFp)
 GO(SDL_GameControllerHasButton, uFpi)
 GO(SDL_GameControllerHasLED, iFp)
+GO(SDL_GameControllerHasRumble, iFp)
 GO(SDL_GameControllerHasSensor, iFpi)
 GO(SDL_GameControllerIsSensorEnabled, uFpi)
 GO(SDL_GameControllerMapping, pFp)
@@ -161,6 +166,8 @@ GO(SDL_GameControllerName, pFp)
 GO(SDL_GameControllerNameForIndex, pFi)
 GO(SDL_GameControllerNumMappings, iFv)
 GO(SDL_GameControllerOpen, pFi)
+GO(SDL_GameControllerPath, pFp)
+GO(SDL_GameControllerPathForIndex, pFi)
 GO(SDL_GameControllerRumble, iFpWWu)
 GO(SDL_GameControllerRumbleTriggers, iFpWWu)
 GO(SDL_GameControllerSetLED, iFpCCC)
@@ -187,6 +194,7 @@ GO(SDL_GetCurrentDisplayMode, iFip)
 GO(SDL_GetCurrentVideoDriver, pFv)
 GO(SDL_GetCursor, pFv)
 // SDL_GetDefaultAssertionHandler
+GO(SDL_GetDefaultAudioInfo, iFppi)
 GO(SDL_GetDefaultCursor, pFv)
 GO(SDL_GetDesktopDisplayMode, iFip)
 GO(SDL_GetDisplayBounds, iFip)
@@ -208,7 +216,7 @@ GO(SDL_GetKeyboardState, pFp)
 GO(SDL_GetKeyFromName, iFp)
 GO(SDL_GetKeyFromScancode, iFu)
 GO(SDL_GetKeyName, pFi)
-GO(SDL_GetMemoryFunctions, vFpppp)
+//GO(SDL_GetMemoryFunctions, vFpppp)
 GO(SDL_GetModState, uFv)
 GO(SDL_GetMouseFocus, pFv)
 GO(SDL_GetMouseState, uFpp)
@@ -221,14 +229,18 @@ GO(SDL_GetNumTouchDevices, iFv)
 GO(SDL_GetNumTouchFingers, iFu)
 GO(SDL_GetNumVideoDisplays, iFv)
 GO(SDL_GetNumVideoDrivers, iFv)
+//GOM(SDL_GetOriginalMemoryFunctions, 
 GO(SDL_GetPerformanceCounter, UFv)
 GO(SDL_GetPerformanceFrequency, LFv)
 GO(SDL_GetPixelFormatName, pFu)
 GO(SDL_GetPlatform, pFv)
+GO(SDL_GetPointDisplayIndex, iFp)
 GO(SDL_GetPowerInfo, uFpp)
 GO(SDL_GetPreferredLocales, pFv)
 GO(SDL_GetPrefPath, pFpp)
+GO(SDL_GetPrimarySelectionText, pFv)
 GO(SDL_GetQueuedAudioSize, uFu)
+GO(SDL_GetRectDisplayIndex, iFp)
 GO(SDL_GetRelativeMouseMode, iFv)
 GO(SDL_GetRelativeMouseState, uFpp)
 GO(SDL_GetRenderDrawBlendMode, iFpp)
@@ -253,9 +265,10 @@ GO(SDL_GetSystemRAM, iFv)
 GO(SDL_GetTextureAlphaMod, iFpp)
 GO(SDL_GetTextureBlendMode, iFpp)
 GO(SDL_GetTextureColorMod, iFpppp)
-GO(SDL_GetThreadID, LFp)
+GOM(SDL_GetThreadID, LFEp)
 GO(SDL_GetThreadName, pFp)
 GO(SDL_GetTicks, uFv)
+GO(SDL_GetTicks64, UFv)
 GO(SDL_GetTouchDevice, lFi)
 GO(SDL_GetTouchDeviceType, iFl)
 GO(SDL_GetTouchFinger, pFli)
@@ -277,6 +290,7 @@ GO(SDL_GetWindowOpacity, iFpp)
 GO(SDL_GetWindowPixelFormat, uFp)
 GO(SDL_GetWindowPosition, vFppp)
 GO(SDL_GetWindowSize, vFppp)
+GO(SDL_GetWindowSizeInPixels, vFppp)
 GO(SDL_GetWindowSurface, pFp)
 GO(SDL_GetWindowTitle, pFp)
 GO(SDL_GetWindowWMInfo, iFpp)
@@ -300,6 +314,7 @@ GO(SDL_GL_SetSwapInterval, iFi)
 GO(SDL_GL_SwapWindow, vFp)
 GO(SDL_GL_UnbindTexture, iFp)
 GO(SDL_GL_UnloadLibrary, vFv)
+GO(SDL_GUIDToString, vFUUpi)
 GO(SDL_HapticClose, vFp)
 GO(SDL_HapticDestroyEffect, vFpi)
 GO(SDL_HapticEffectSupported, iFpp)
@@ -337,9 +352,12 @@ GO(SDL_HasColorKey, uFp)
 GO(SDL_HasEvent, uFu)
 GO(SDL_HasEvents, uFuu)
 GO(SDL_HasIntersection, uFpp)
+GO(SDL_HasLASX, iFv)
+GO(SDL_HasLSX, iFv)
 GOM(SDL_HasMMX, iFv)
 GOM(SDL_HasNEON, iFv)
 GOM(SDL_HasRDTSC, iFv)
+GO(SDL_HasPrimarySelectionText, iFv)
 GO(SDL_HasScreenKeyboardSupport, iFv)
 GOM(SDL_HasSSE, iFv)
 GOM(SDL_HasSSE2, iFv)
@@ -347,6 +365,7 @@ GOM(SDL_HasSSE3, iFv)
 GOM(SDL_HasSSE41, iFv)
 GOM(SDL_HasSSE42, iFv)
 GO(SDL_HideWindow, vFp)
+GO(SDL_HasWindowSurface, iFp)
 GO(SDL_iconv, LFppppp)
 GO(SDL_iconv_close, iFp)
 GO(SDL_iconv_open, pFpp)
@@ -364,6 +383,7 @@ GO(SDL_isspace, iFi)
 GO(SDL_IsTablet, uFv)
 GO(SDL_IsTextInputActive, iFv)
 GO(SDL_itoa, pFipi)
+GO(SDL_JoystickAttachVirtualEx, iFp)
 GO(SDL_JoystickClose, vFp)
 GO(SDL_JoystickCurrentPowerLevel, iFp)
 GO(SDL_JoystickEventState, iFi)
@@ -380,6 +400,7 @@ GO(SDL_JoystickGetDeviceProduct, WFi)
 GO(SDL_JoystickGetDeviceProductVersion, WFi)
 GO(SDL_JoystickGetDeviceType, uFi)
 GO(SDL_JoystickGetDeviceVendor, WFi)
+GO(SDL_JoystickGetFirmwareVersion, WFp)
 GO(SDL_JoystickGetGUID, HFp)
 GO(SDL_JoystickGetGUIDFromString, HFp)
 GO(SDL_JoystickGetGUIDString, vFUUpi)   //uint8_t data[16] passed as 1st argument...
@@ -387,6 +408,7 @@ GO(SDL_JoystickGetHat, CFpi)
 GO(SDL_JoystickGetPlayerIndex, iFp)
 GO(SDL_JoystickGetProduct, WFp)
 GO(SDL_JoystickGetProductVersion, WFp)
+GO(SDL_JoystickGetSerial, pFp)
 GO(SDL_JoystickGetType, uFp)
 GO(SDL_JoystickGetVendor, WFp)
 GO(SDL_JoystickInstanceID, iFp)
@@ -398,7 +420,10 @@ GO(SDL_JoystickNumBalls, iFp)
 GO(SDL_JoystickNumButtons, iFp)
 GO(SDL_JoystickNumHats, iFp)
 GO(SDL_JoystickOpen, pFi)
+GO(SDL_JoystickPath, pFp)
+GO(SDL_JoystickPathForIndex, pFi)
 GO(SDL_JoystickRumble, iFpWWu)
+GO(SDL_JoystickRumbleTriggers, iFpWWu)
 GO(SDL_JoystickUpdate, vFv)
 GO(SDL_LinuxSetThreadPriority, iFli)
 GO(SDL_lltoa, pFlpi)
@@ -413,6 +438,7 @@ GO(SDL_LockAudio, vFv)
 GO(SDL_LockAudioDevice, vFi)
 GO(SDL_LockJoysticks, vFv)
 GO(SDL_LockMutex, iFp)
+GO(SDL_LockSensors, vFv)
 GO(SDL_LockSurface, iFp)
 GO(SDL_LockTexture, iFpppp)
 GO(SDL_log, dFd)
@@ -522,9 +548,11 @@ GO(SDL_RenderSetLogicalSize, iFpii)
 GO(SDL_RenderSetScale, iFpff)
 GO(SDL_RenderSetViewport, iFpp)
 GO(SDL_RenderTargetSupported, iFp)
+GO(SDL_RenderSetVSync, iFpi)
 GO(SDL_RenderWindowToLogical, vFpiipp)
 GO(SDL_ReportAssertion, uFpppi)
 GO(SDL_ResetAssertionReport, vFv)
+GO(SDL_ResetHints, vFv)
 GO(SDL_ResetKeyboard, vFv)
 GO(SDL_RestoreWindow, vFp)
 GOM(SDL_RWFromConstMem, pFEpi)
@@ -541,19 +569,20 @@ GO(SDL_SemTryWait, iFp)
 GO(SDL_SemValue, uFp)
 GO(SDL_SemWait, iFp)
 GO(SDL_SemWaitTimeout, iFpu)
-// SDL_SensorClose
-// SDL_SensorFromInstanceID
-// SDL_SensorGetData
-// SDL_SensorGetDeviceInstanceID
-// SDL_SensorGetDeviceName
-// SDL_SensorGetDeviceNonPortableType
-// SDL_SensorGetDeviceType
-// SDL_SensorGetInstanceID
-// SDL_SensorGetName
-// SDL_SensorGetNonPortableType
-// SDL_SensorGetType
-// SDL_SensorOpen
-// SDL_SensorUpdate
+GO(SDL_SensorClose, vFp)
+GO(SDL_SensorFromInstanceID, pFi)
+GO(SDL_SensorGetData, iFppi)
+GO(SDL_SensorGetDataWithTimestamp, iFpppi)
+GO(SDL_SensorGetDeviceInstanceID, iFi)
+GO(SDL_SensorGetDeviceName, pFi)
+GO(SDL_SensorGetDeviceNonPortableType, iFi)
+GO(SDL_SensorGetDeviceType, iFi)
+GO(SDL_SensorGetInstanceID, iFp)
+GO(SDL_SensorGetName, pFp)
+GO(SDL_SensorGetNonPortableType, iFp)
+GO(SDL_SensorGetType, iFp)
+GO(SDL_SensorOpen, pFi)
+GO(SDL_SensorUpdate, vFv)
 // SDL_SetAssertionHandler
 GO(SDL_SetClipboardText, iFp)
 GO(SDL_SetClipRect, uFpp)
@@ -565,10 +594,11 @@ GOM(SDL_SetEventFilter, vFEpp)
 GO(SDL_SetHint, uFpp)
 GO(SDL_SetHintWithPriority, uFppu)
 GO(SDL_SetMainReady, vFv)
-// SDL_SetMemoryFunctions
+//GOM(SDL_SetMemoryFunctions, 
 GO(SDL_SetModState, vFu)
 GO(SDL_SetPaletteColors, iFppii)
 // SDL_SetPixelFormatPalette
+GO(SDL_SetPrimarySelectionText, iFp)
 GO(SDL_SetRelativeMouseMode, iFi)
 GO(SDL_SetRenderDrawBlendMode, iFpu)
 GO(SDL_SetRenderDrawColor, iFpCCCC)
@@ -583,6 +613,7 @@ GO(SDL_SetTextInputRect, vFp)
 GO(SDL_SetTextureAlphaMod, iFpC)
 GO(SDL_SetTextureBlendMode, iFpu)
 GO(SDL_SetTextureColorMod, iFpCCC)
+GO(SDL_SetTextureScaleMode, iFpu)
 GO(SDL_SetThreadPriority, iFu)
 GO(SDL_SetWindowBordered, vFpi)
 GO(SDL_SetWindowBrightness, iFpf)
@@ -658,6 +689,7 @@ GO(SDL_UnlockAudio, vFv)
 GO(SDL_UnlockAudioDevice, vFi)
 GO(SDL_UnlockJoysticks, vFv)
 GO(SDL_UnlockMutex, iFp)
+GO(SDL_UnlockSensors, vFv)
 GO(SDL_UnlockSurface, vFp)
 GO(SDL_UnlockTexture, vFp)
 GO(SDL_UpdateTexture, iFpppi)
