@@ -101,13 +101,8 @@ EXPORT void* my2_IMG_LoadTextureTyped_RW(x64emu_t* emu, void* rend, void* a, int
     return r;
 }
 
-#define CUSTOM_INIT \
-    getMy(lib);     \
-    SETALT(my2_);   \
-    setNeededLibs(lib, 1, "libSDL2-2.0.so.0");
+#define ALTMY my2_
 
-#define CUSTOM_FINI \
-    freeMy();
+#define NEEDED_LIBS "libSDL2-2.0.so.0"
 
 #include "wrappedlib_init.h"
-
