@@ -15,6 +15,7 @@
 #include "box64context.h"
 #include "librarian.h"
 #include "callback.h"
+#include "myalign.h"
 
 //extern char* libvulkan;
 
@@ -24,7 +25,6 @@ const char* vulkanName = "libvulkan.so.1";
 typedef void(*vFpUp_t)      (void*, uint64_t, void*);
 
 #define ADDED_FUNCTIONS()                           \
-    GO(vkDestroySamplerYcbcrConversion, vFpUp_t)    \
 
 #include "generated/wrappedvulkantypes.h"
 
@@ -74,7 +74,7 @@ static void* resolveSymbol(x64emu_t* emu, void* symbol, const char* rname)
     if(!s->resolved) {
         khint_t k = kh_get(symbolmap, emu->context->vkwrappers, rname);
         const char* constname = kh_key(emu->context->vkwrappers, k);
-        s->addr = AddBridge(emu->context->system, s->w, symbol, 0, constname);
+        s->addr = AddCheckBridge(emu->context->system, s->w, symbol, 0, constname);
         s->resolved = 1;
     }
     void* ret = (void*)s->addr;
@@ -109,7 +109,7 @@ EXPORT void* my_vkGetDeviceProcAddr(x64emu_t* emu, void* device, void* name)
         #define GO(A, W) if(!strcmp(rname, #A)) my->A = (W)my->vkGetDeviceProcAddr(device, name);
         SUPER()
         #undef GO
-    } 
+    }
     if(!symbol) {
         printf_dlsym(LOG_DEBUG, "%p\n", NULL);
         return NULL;    // easy
@@ -225,6 +225,14 @@ typedef struct my_VkDebugReportCallbackCreateInfoEXT_s {
     void*       pUserData;
 } my_VkDebugReportCallbackCreateInfoEXT_t;
 
+typedef struct my_VkXcbSurfaceCreateInfoKHR_s {
+    int         sType;
+    const void* pNext;
+    uint32_t    flags;
+    void**      connection;
+    int         window;
+} my_VkXcbSurfaceCreateInfoKHR_t;
+
 typedef struct my_VkStruct_s {
     int         sType;
     struct my_VkStruct_s* pNext;
@@ -401,13 +409,9 @@ static void* find_DebugUtilsMessengerCallback_Fct(void* fct)
         return -1;
 
 #define CUSTOM_INIT \
-    getMy(lib);     \
     lib->w.priv = dlsym(lib->w.lib, "vkGetInstanceProcAddr"); \
     box64->vkprocaddress = lib->w.priv;
 
-#define CUSTOM_FINI \
-    freeMy();
-
 #include "wrappedlib_init.h"
 
 void fillVulkanProcWrapper(box64context_t* context)
@@ -582,8 +586,18 @@ EXPORT int my_vkCreateSharedSwapchainsKHR(x64emu_t* emu, void* device, uint32_t
 
 CREATE(vkCreateSwapchainKHR)
 CREATE(vkCreateWaylandSurfaceKHR)
-CREATE(vkCreateXcbSurfaceKHR)
+EXPORT int my_vkCreateXcbSurfaceKHR(x64emu_t* emu, void* instance, void* info, my_VkAllocationCallbacks_t* pAllocator, void* pFence)
+{
+    my_VkAllocationCallbacks_t my_alloc;
+    my_VkXcbSurfaceCreateInfoKHR_t* surfaceinfo = info;
+    void* old_conn = surfaceinfo->connection;
+    surfaceinfo->connection = align_xcb_connection(old_conn);
+    int ret = my->vkCreateXcbSurfaceKHR(instance, info, find_VkAllocationCallbacks(&my_alloc, pAllocator), pFence);
+    surfaceinfo->connection = old_conn;
+    return ret;
+}
 CREATE(vkCreateXlibSurfaceKHR)
+CREATE(vkCreateAndroidSurfaceKHR)
 CREATE(vkCreateRenderPass2)
 CREATE(vkCreateRenderPass2KHR)
 
@@ -600,6 +614,23 @@ EXPORT int my_vkRegisterDisplayEventEXT(x64emu_t* emu, void* device, uint64_t di
 
 CREATE(vkCreateValidationCacheEXT)
 
+EXPORT int my_vkCreateShadersEXT(x64emu_t* emu, void* device, uint32_t count, void** pCreateInfos, my_VkAllocationCallbacks_t* pAllocator, void* pShaders)
+{
+    my_VkAllocationCallbacks_t my_alloc;
+    int ret = my->vkCreateShadersEXT(device, count, pCreateInfos, find_VkAllocationCallbacks(&my_alloc, pAllocator), pShaders);
+    return ret;
+}
+
+EXPORT int my_vkCreateExecutionGraphPipelinesAMDX(x64emu_t* emu, void* device, uint64_t pipelineCache, uint32_t count, void** pCreateInfos, my_VkAllocationCallbacks_t* pAllocator, void* pPipeLines)
+{
+    my_VkAllocationCallbacks_t my_alloc;
+    int ret = my->vkCreateExecutionGraphPipelinesAMDX(device, pipelineCache, count, pCreateInfos, find_VkAllocationCallbacks(&my_alloc, pAllocator), pPipeLines);
+    return ret;
+}
+
+DESTROY64(vkDestroyShaderEXT)
+
+
 DESTROY64(vkDestroyBuffer)
 DESTROY64(vkDestroyBufferView)
 DESTROY64(vkDestroyCommandPool)
@@ -648,7 +679,7 @@ EXPORT int my_vkCreateDebugUtilsMessengerEXT(x64emu_t* emu, void* device, my_VkD
         info->pfnUserCallback = find_DebugUtilsMessengerCallback_Fct(info->pfnUserCallback);
         info = (my_VkDebugUtilsMessengerCreateInfoEXT_t*)info->pNext;
     }
-    return my->vkCreateDebugUtilsMessengerEXT(device, pAllocateInfo, find_VkAllocationCallbacks(&my_alloc, pAllocator), p); 
+    return my->vkCreateDebugUtilsMessengerEXT(device, pAllocateInfo, find_VkAllocationCallbacks(&my_alloc, pAllocator), p);
 }
 DESTROY(vkDestroyDebugUtilsMessengerEXT)
 
@@ -707,13 +738,17 @@ DESTROY64(vkDestroyOpticalFlowSessionNV)
 CREATE(vkCreateMicromapEXT)
 DESTROY64(vkDestroyMicromapEXT)
 
+CREATE(vkCreateCudaFunctionNV)
+CREATE(vkCreateCudaModuleNV)
+DESTROY64(vkDestroyCudaFunctionNV)
+DESTROY64(vkDestroyCudaModuleNV)
 
 EXPORT void my_vkGetPhysicalDeviceProperties(x64emu_t* emu, void* device, void* pProps)
 {
     my->vkGetPhysicalDeviceProperties(device, pProps);
 }
 
-EXPORT void my_vkGetPhysicalDeviceSparseImageFormatProperties(x64emu_t* emu, void* device, int format, int type, int samples, int usage, int tiling, uint32_t* count, void** pProps)
+EXPORT void my_vkGetPhysicalDeviceSparseImageFormatProperties(x64emu_t* emu, void* device, int format, int type, int samples, uint32_t usage, int tiling, uint32_t* count, void** pProps)
 {
     my->vkGetPhysicalDeviceSparseImageFormatProperties(device, format, type, samples, usage, tiling, count, pProps);
 }
@@ -740,7 +775,7 @@ EXPORT void my_vkGetPhysicalDeviceMemoryProperties(x64emu_t* emu, void* device,
     my->vkGetPhysicalDeviceMemoryProperties(device, pProps);
 }
 
-EXPORT void my_vkCmdPipelineBarrier(x64emu_t* emu, void* device, int src, int dst, int dep,
+EXPORT void my_vkCmdPipelineBarrier(x64emu_t* emu, void* device, uint32_t src, uint32_t dst, uint32_t dep,
     uint32_t barrierCount, void* pBarriers, uint32_t bufferCount, void* pBuffers, uint32_t imageCount, void* pImages)
 {
     my->vkCmdPipelineBarrier(device, src, dst, dep, barrierCount, pBarriers, bufferCount, pBuffers, imageCount, pImages);
@@ -756,10 +791,10 @@ EXPORT int my_vkCreateDebugReportCallbackEXT(x64emu_t* emu, void* instance,
     return my->vkCreateDebugReportCallbackEXT(instance, &dbg, find_VkAllocationCallbacks(&my_alloc, alloc), callback);
 }
 
-EXPORT int my_vkDestroyDebugReportCallbackEXT(x64emu_t* emu, void* instance, void* callback, void* alloc)
+EXPORT void my_vkDestroyDebugReportCallbackEXT(x64emu_t* emu, void* instance, void* callback, void* alloc)
 {
     my_VkAllocationCallbacks_t my_alloc;
-    return my->vkDestroyDebugReportCallbackEXT(instance, callback, find_VkAllocationCallbacks(&my_alloc, alloc));
+    my->vkDestroyDebugReportCallbackEXT(instance, callback, find_VkAllocationCallbacks(&my_alloc, alloc));
 }
 
 CREATE(vkCreateHeadlessSurfaceEXT)
