@@ -44,8 +44,8 @@ GOM(vkAllocateMemory, iFEpppp)
 GO(vkBeginCommandBuffer, iFpp)
 GO(vkBindBufferMemory, iFpUUU)
 GO(vkBindImageMemory, iFpUUU)
-GO(vkCmdBeginQuery, vFpUui)
-GO(vkCmdBeginRenderPass, vFppp)
+GO(vkCmdBeginQuery, vFpUuu)
+GO(vkCmdBeginRenderPass, vFppi)
 GO(vkCmdBindDescriptorSets, vFpiUuupup)
 GO(vkCmdBindIndexBuffer, vFpUUi)
 GO(vkCmdBindPipeline, vFppU)
@@ -58,7 +58,7 @@ GO(vkCmdCopyBuffer, vFpUUup)
 GO(vkCmdCopyBufferToImage, vFpUUiup)
 GO(vkCmdCopyImage, vFpUiUiup)
 GO(vkCmdCopyImageToBuffer, vFpUiUup)
-GO(vkCmdCopyQueryPoolResults, vFpUuuUUUi)
+GO(vkCmdCopyQueryPoolResults, vFpUuuUUUu)
 GO(vkCmdDispatch, vFpuuu)
 GO(vkCmdDispatchIndirect, vFpUU)
 GO(vkCmdDraw, vFpuuuu)
@@ -70,23 +70,23 @@ GO(vkCmdEndRenderPass, vFp)
 GO(vkCmdExecuteCommands, vFpup)
 GO(vkCmdFillBuffer, vFpUUUu)
 GO(vkCmdNextSubpass, vFpi)
-GOM(vkCmdPipelineBarrier, vFEpiiiupupup)
-GO(vkCmdPushConstants, vFpUiuup)
-GO(vkCmdResetEvent, vFpUi)
+GOM(vkCmdPipelineBarrier, vFEpuuuupupup)
+GO(vkCmdPushConstants, vFpUuuup)
+GO(vkCmdResetEvent, vFpUu)
 GO(vkCmdResetQueryPool, vFpUuu)
 GO(vkCmdResolveImage, vFpUiUiup)
 GO(vkCmdSetBlendConstants, vFpp)
 GO(vkCmdSetDepthBias, vFpfff)
 GO(vkCmdSetDepthBounds, vFpff)
-GO(vkCmdSetEvent, vFpUi)
+GO(vkCmdSetEvent, vFpUu)
 GO(vkCmdSetLineWidth, vFpf)
 GO(vkCmdSetScissor, vFpuup)
-GO(vkCmdSetStencilCompareMask, vFpiu)
-GO(vkCmdSetStencilReference, vFpiu)
-GO(vkCmdSetStencilWriteMask, vFpiu)
+GO(vkCmdSetStencilCompareMask, vFpuu)
+GO(vkCmdSetStencilReference, vFpuu)
+GO(vkCmdSetStencilWriteMask, vFpuu)
 GO(vkCmdSetViewport, vFpuup)
 GO(vkCmdUpdateBuffer, vFpUUUp)
-GO(vkCmdWaitEvents, vFpupiiupupup)
+GO(vkCmdWaitEvents, vFpupuuupupup)
 GO(vkCmdWriteTimestamp, vFpiUu)
 GOM(vkCreateBuffer, iFEpppp)
 GOM(vkCreateBufferView, iFEpppp)
@@ -140,7 +140,7 @@ GO(vkFlushMappedMemoryRanges, iFpup)    // should wrap the array of VkMappedMemo
 GO(vkFreeCommandBuffers, vFpUup)
 GO(vkFreeDescriptorSets, iFpUup)
 GOM(vkFreeMemory, iFEpUp)
-GO(vkGetBufferMemoryRequirements, iFpUp)
+GO(vkGetBufferMemoryRequirements, vFpUp)
 GO(vkGetDeviceMemoryCommitment, vFpUp)
 GOM(vkGetDeviceProcAddr, pFEpp)
 GO(vkGetDeviceQueue, vFpuup)
@@ -152,29 +152,29 @@ GO(vkGetImageSubresourceLayout, vFpUpp)
 GOM(vkGetInstanceProcAddr, pFEpp)
 GO(vkGetPhysicalDeviceFeatures, vFpp)
 GO(vkGetPhysicalDeviceFormatProperties, vFpip)
-GO(vkGetPhysicalDeviceImageFormatProperties, iFpiiiiip) //VkImageFormatProperties sems OK
+GO(vkGetPhysicalDeviceImageFormatProperties, iFpiiiuup) // VkImageFormatProperties sems OK
 GOM(vkGetPhysicalDeviceMemoryProperties, vFEpp)
 GOM(vkGetPhysicalDeviceProperties, vFEpp)
 GO(vkGetPhysicalDeviceQueueFamilyProperties, vFppp)   //VkQueueFamilyProperties is OK
-GOM(vkGetPhysicalDeviceSparseImageFormatProperties, vFEpiiiiipp)
+GOM(vkGetPhysicalDeviceSparseImageFormatProperties, vFEpiiiuipp)
 GO(vkGetPipelineCacheData, iFpUpp)
-GO(vkGetQueryPoolResults, iFpUuuLpUi)
+GO(vkGetQueryPoolResults, iFpUuuLpUu)
 GO(vkGetRenderAreaGranularity, vFpUp)
 GO(vkInvalidateMappedMemoryRanges, iFpup)   //VkMappedMemoryRange seems OK
-GO(vkMapMemory, iFpUUUip)
+GO(vkMapMemory, iFpUUUup)
 GO(vkMergePipelineCaches, iFpUup)
 GO(vkQueueBindSparse, iFpupU)
 GO(vkQueueSubmit, iFpupU)
 GO(vkQueueWaitIdle, iFp)
-GO(vkResetCommandBuffer, iFpi)
-GO(vkResetCommandPool, iFpUi)
-GO(vkResetDescriptorPool, iFpUi)
+GO(vkResetCommandBuffer, iFpu)
+GO(vkResetCommandPool, iFpUu)
+GO(vkResetDescriptorPool, iFpUu)
 GO(vkResetEvent, iFpU)
 GO(vkResetFences, iFpup)
 GO(vkSetEvent, iFpU)
 GO(vkUnmapMemory, vFpU)
 GOM(vkUpdateDescriptorSets, vFEpupup)
-GO(vkWaitForFences, iFpupiU)
+GO(vkWaitForFences, iFpupuU)
 
 // VK_VERSION_1_1
 GO(vkBindBufferMemory2, iFpup)
@@ -184,6 +184,7 @@ GO(vkCmdSetDeviceMask, vFpu)
 GOM(vkCreateDescriptorUpdateTemplate, iFEpppp)
 GOM(vkCreateSamplerYcbcrConversion, iFEpppp)
 GOM(vkDestroyDescriptorUpdateTemplate, vFEpUp)
+GOM(vkDestroySamplerYcbcrConversion, vFEpUp)
 GO(vkEnumerateInstanceVersion, iFp)
 GO(vkEnumeratePhysicalDeviceGroups, iFppp)  //VkPhysicalDeviceGroupProperties seems OK
 GO(vkGetBufferMemoryRequirements2, iFppp)
@@ -204,7 +205,7 @@ GO(vkGetPhysicalDeviceQueueFamilyProperties2, vFppp)    //VkQueueFamilyPropertie
 GO(vkGetPhysicalDeviceSparseImageFormatProperties2, vFpppp) //VkSparseImageFormatProperties2 seems OK
 GO(vkGetPhysicalDeviceToolProperties, iFppp)
 GO(vkUpdateDescriptorSetWithTemplate, vFpUUp)
-GO(vkTrimCommandPool, vFpUi)
+GO(vkTrimCommandPool, vFpUu)
 
 // VK_VERSION_1_2
 GO(vkResetQueryPool, vFpUuu)
@@ -230,11 +231,11 @@ GO(vkCmdCopyBufferToImage2, vFpp)
 GO(vkCmdCopyImage2, vFpp)
 GO(vkCmdCopyImageToBuffer2, vFpp)
 GO(vkCmdResolveImage2, vFpp)
-GO(vkCmdSetDepthBiasEnable, vFpi)
+GO(vkCmdSetDepthBiasEnable, vFpu)
 GO(vkCmdSetLogicOp, vFpi)
 GO(vkCmdSetPatchControlPoints, vFpu)
-GO(vkCmdSetPrimitiveRestartEnable, vFpi)
-GO(vkCmdSetRasterizerDiscardEnable, vFpi)
+GO(vkCmdSetPrimitiveRestartEnable, vFpu)
+GO(vkCmdSetRasterizerDiscardEnable, vFpu)
 GOM(vkCreatePrivateDataSlot, iFEpppp)
 GOM(vkDestroyPrivateDataSlot, vFEpUp)
 GO(vkGetPrivateData, vFpiUUp)
@@ -254,7 +255,7 @@ GO(vkCmdSetDepthBoundsTestEnable, vFpu)
 GO(vkCmdSetDepthCompareOp, vFpu)
 GO(vkCmdSetDepthTestEnable, vFpu)
 GO(vkCmdSetDepthWriteEnable, vFpu)
-GO(vkCmdSetFrontFace, vFpu)
+GO(vkCmdSetFrontFace, vFpi)
 GO(vkCmdSetPrimitiveTopology, vFpi)
 GO(vkCmdSetScissorWithCount, vFpup)
 GO(vkCmdSetStencilOp, vFpuiiii)
@@ -263,8 +264,8 @@ GO(vkCmdSetViewportWithCount, vFpup)
 
 // VK_EXT_debug_report
 GOM(vkCreateDebugReportCallbackEXT, iFEpppp)
-GO(vkDebugReportMessageEXT, vFpiiULipp)
-GOM(vkDestroyDebugReportCallbackEXT, iFEppp)
+GO(vkDebugReportMessageEXT, vFpuiULipp)
+GOM(vkDestroyDebugReportCallbackEXT, vFEppp)
 
 //VK_EXT_debug_utils
 GO(vkCmdBeginDebugUtilsLabelEXT, vFpp)  //TODO: Check alignement of this extension
@@ -277,7 +278,7 @@ GO(vkQueueEndDebugUtilsLabelEXT, vFp)
 GO(vkQueueInsertDebugUtilsLabelEXT, vFpp)
 GO(vkSetDebugUtilsObjectNameEXT, iFpp)
 GO(vkSetDebugUtilsObjectTagEXT, iFpp)
-GO(vkSubmitDebugUtilsMessageEXT, vFpppp)
+GO(vkSubmitDebugUtilsMessageEXT, vFpiup)
 
 // VK_KHR_external_memory_capabilities
 GO(vkGetPhysicalDeviceExternalBufferPropertiesKHR, vFppp)
@@ -304,11 +305,14 @@ GO(vkGetPhysicalDeviceSurfaceSupportKHR, iFpuUp)
 
 // VK_KHR_xcb_surface
 GOM(vkCreateXcbSurfaceKHR, iFEpppp)
-GO(vkGetPhysicalDeviceXcbPresentationSupportKHR, iFpupp)
+GO(vkGetPhysicalDeviceXcbPresentationSupportKHR, uFpubp)
 
 // VK_KHR_xlib_surface
 GOM(vkCreateXlibSurfaceKHR, iFEpppp)
-GO(vkGetPhysicalDeviceXlibPresentationSupportKHR, iFpupp)
+GO(vkGetPhysicalDeviceXlibPresentationSupportKHR, uFpupp)
+
+// VK_KHR_android_surface
+GOM(vkCreateAndroidSurfaceKHR, iFEpppp)
 
 // VK_KHR_swapchain
 GO(vkAcquireNextImageKHR, iFpUUUUp)
@@ -363,7 +367,7 @@ GOM(vkCreateSharedSwapchainsKHR, iFEpuppp)
 
 // VK_KHR_wayland_surface
 GOM(vkCreateWaylandSurfaceKHR, iFEpppp)
-GO(vkGetPhysicalDeviceWaylandPresentationSupportKHR, iFpup)
+GO(vkGetPhysicalDeviceWaylandPresentationSupportKHR, uFpup)
 
 // VK_KHR_device_group_creation
 GO(vkEnumeratePhysicalDeviceGroupsKHR, iFppp)
@@ -418,7 +422,7 @@ GO(vkResetQueryPoolEXT, vFpUuu)
 // no functions
 
 // VK_EXT_transform_feedback
-GO(vkCmdBeginQueryIndexedEXT, vFpUuiu)
+GO(vkCmdBeginQueryIndexedEXT, vFpUuuu)
 GO(vkCmdBeginTransformFeedbackEXT, vFpuupp)
 GO(vkCmdBindTransformFeedbackBuffersEXT, vFpuuppp)
 GO(vkCmdDrawIndirectByteCountEXT, vFpuuUUuu)
@@ -474,7 +478,7 @@ GOM(vkCreateRenderPass2KHR, iFEpppp)
 GO(vkCmdBindVertexBuffers2EXT, vFpuupppp)
 GO(vkCmdSetCullModeEXT, vFpu)
 GO(vkCmdSetDepthBoundsTestEnableEXT, vFpu)
-GO(vkCmdSetDepthCompareOpEXT, vFpu)
+GO(vkCmdSetDepthCompareOpEXT, vFpi)
 GO(vkCmdSetDepthTestEnableEXT, vFpu)
 GO(vkCmdSetDepthWriteEnableEXT, vFpu)
 GO(vkCmdSetFrontFaceEXT, vFpu)
@@ -493,10 +497,10 @@ GO(vkCmdDrawIndexedIndirectCountAMD, vFpUUUUuu)
 GO(vkCmdDrawIndirectCountAMD, vFpUUUUuu)
 
 // VK_AMD_buffer_marker
-GO(vkCmdWriteBufferMarkerAMD, vFpuUUu)
+GO(vkCmdWriteBufferMarkerAMD, vFpiUUu)
 
 // VK_AMD_shader_info
-GO(vkGetShaderInfoAMD, iFpUuupp)
+GO(vkGetShaderInfoAMD, iFpUiipp)
 
 // VK_EXT_debug_marker
 GO(vkCmdDebugMarkerBeginEXT, vFpp)
@@ -507,15 +511,17 @@ GO(vkDebugMarkerSetObjectTagEXT, iFpp)
 
 // VK_EXT_discard_rectangles
 GO(vkCmdSetDiscardRectangleEXT, vFpuup)
+GO(vkCmdSetDiscardRectangleEnableEXT, vFpu)
+GO(vkCmdSetDiscardRectangleModeEXT, vFpi)
 
 // VK_EXT_display_control
 GO(vkDisplayPowerControlEXT, iFpUp)
-GO(vkGetSwapchainCounterEXT, iFpUup)
+GO(vkGetSwapchainCounterEXT, iFpUip)
 GOM(vkRegisterDeviceEventEXT, iFEpppp)
 GOM(vkRegisterDisplayEventEXT, iFEpUppp)
 
 // VK_EXT_external_memory_host
-GO(vkGetMemoryHostPointerPropertiesEXT, iFpupp)
+GO(vkGetMemoryHostPointerPropertiesEXT, iFpipp)
 
 // VK_EXT_hdr_metadata
 GO(vkSetHdrMetadataEXT, vFpupp)
@@ -557,6 +563,7 @@ GO(vkCmdDrawMeshTasksIndirectNV, vFpUUuu)
 GO(vkCmdDrawMeshTasksNV, vFpuu)
 
 // VK_NV_scissor_exclusive
+GO(vkCmdSetExclusiveScissorEnableNV, vFpuup)
 GO(vkCmdSetExclusiveScissorNV, vFpuup)
 
 // VK_NV_shading_rate_image
@@ -592,7 +599,7 @@ GO(vkSignalSemaphoreKHR, iFpp)
 GO(vkWaitSemaphoresKHR, iFppU)
 
 // VK_AMD_display_native_hdr
-GO(vkSetLocalDimmingAMD, vFpUi)
+GO(vkSetLocalDimmingAMD, vFpUu)
 
 // VK_EXT_color_write_enable
 GO(vkCmdSetColorWriteEnableEXT, vFpup)
@@ -657,7 +664,7 @@ GO(vkCopyAccelerationStructureToMemoryKHR, iFpUp)
 GO(vkCopyMemoryToAccelerationStructureKHR, iFpUp)
 GOM(vkCreateAccelerationStructureKHR, iFEpppp)
 GOM(vkDestroyAccelerationStructureKHR, vFEpUp)
-GO(vkGetAccelerationStructureBuildSizesKHR, vFpUppp)
+GO(vkGetAccelerationStructureBuildSizesKHR, vFpippp)
 GO(vkGetAccelerationStructureDeviceAddressKHR, UFpp)
 GO(vkGetDeviceAccelerationStructureCompatibilityKHR, vFppp)
 GO(vkWriteAccelerationStructuresPropertiesKHR, iFpupiLpL)
@@ -718,6 +725,8 @@ GO(vkCmdDecodeVideoKHR, vFpp)
 
 // VK_KHR_video_encode_queue
 GO(vkCmdEncodeVideoKHR, vFpp)
+GO(vkGetEncodedVideoSessionParametersKHR, iFppppp)
+GO(vkGetPhysicalDeviceVideoEncodeQualityLevelPropertiesKHR, iFppp)
 
 // VK_NVX_binary_import
 GO(vkCmdCuLaunchKernelNVX, vFpp)
@@ -732,7 +741,7 @@ GO(vkGetImageViewHandleNVX, uFpp)
 
 // VK_NV_device_generated_commands
 GO(vkCmdBindPipelineShaderGroupNV, vFpiUu)
-GO(vkCmdExecuteGeneratedCommandsNV, vFpip)
+GO(vkCmdExecuteGeneratedCommandsNV, vFpup)
 GO(vkCmdPreprocessGeneratedCommandsNV, vFpp)
 GOM(vkCreateIndirectCommandsLayoutNV, iFEpppp)
 GOM(vkDestroyIndirectCommandsLayoutNV, vFEpUp)
@@ -746,7 +755,7 @@ GO(vkCmdSetFragmentShadingRateEnumNV, vFpip)
 
 // VK_NV_ray_tracing
 GO(vkBindAccelerationStructureMemoryNV, iFpup)
-GO(vkCmdBuildAccelerationStructureNV, vFppUUiUUUU)
+GO(vkCmdBuildAccelerationStructureNV, vFppUUuUUUU)
 GO(vkCmdCopyAccelerationStructureNV, vFpUUi)
 GO(vkCmdTraceRaysNV, vFpUUUUUUUUUUUuuu)
 GO(vkCmdWriteAccelerationStructuresPropertiesNV, vFpupiUu)
@@ -774,37 +783,37 @@ GOM(vkDestroyOpticalFlowSessionNV, vFEpUp)
 GO(vkGetPhysicalDeviceOpticalFlowImageFormatsNV, iFpppp)
 
 // VK_EXT_extended_dynamic_state3
-GO(vkCmdSetAlphaToCoverageEnableEXT, vFpi)
-GO(vkCmdSetAlphaToOneEnableEXT, vFpi)
+GO(vkCmdSetAlphaToCoverageEnableEXT, vFpu)
+GO(vkCmdSetAlphaToOneEnableEXT, vFpu)
 GO(vkCmdSetColorBlendAdvancedEXT, vFpuup)
 GO(vkCmdSetColorBlendEnableEXT, vFpuup)
 GO(vkCmdSetColorBlendEquationEXT, vFpuup)
 GO(vkCmdSetColorWriteMaskEXT, vFpuup)
 GO(vkCmdSetConservativeRasterizationModeEXT, vFpi)
 GO(vkCmdSetCoverageModulationModeNV, vFpi)
-GO(vkCmdSetCoverageModulationTableEnableNV, vFpi)
+GO(vkCmdSetCoverageModulationTableEnableNV, vFpu)
 GO(vkCmdSetCoverageModulationTableNV, vFpup)
 GO(vkCmdSetCoverageReductionModeNV, vFpi)
-GO(vkCmdSetCoverageToColorEnableNV, vFpi)
+GO(vkCmdSetCoverageToColorEnableNV, vFpu)
 GO(vkCmdSetCoverageToColorLocationNV, vFpu)
-GO(vkCmdSetDepthClampEnableEXT, vFpi)
-GO(vkCmdSetDepthClipEnableEXT, vFpi)
-GO(vkCmdSetDepthClipNegativeOneToOneEXT, vFpi)
+GO(vkCmdSetDepthClampEnableEXT, vFpu)
+GO(vkCmdSetDepthClipEnableEXT, vFpu)
+GO(vkCmdSetDepthClipNegativeOneToOneEXT, vFpu)
 GO(vkCmdSetExtraPrimitiveOverestimationSizeEXT, vFpf)
 GO(vkCmdSetLineRasterizationModeEXT, vFpi)
-GO(vkCmdSetLineStippleEnableEXT, vFpi)
-GO(vkCmdSetLogicOpEnableEXT, vFpi)
+GO(vkCmdSetLineStippleEnableEXT, vFpu)
+GO(vkCmdSetLogicOpEnableEXT, vFpu)
 GO(vkCmdSetPolygonModeEXT, vFpi)
 GO(vkCmdSetProvokingVertexModeEXT, vFpi)
 GO(vkCmdSetRasterizationSamplesEXT, vFpi)
 GO(vkCmdSetRasterizationStreamEXT, vFpu)
-GO(vkCmdSetRepresentativeFragmentTestEnableNV, vFpi)
-GO(vkCmdSetSampleLocationsEnableEXT, vFpi)
+GO(vkCmdSetRepresentativeFragmentTestEnableNV, vFpu)
+GO(vkCmdSetSampleLocationsEnableEXT, vFpu)
 GO(vkCmdSetSampleMaskEXT, vFpip)
-GO(vkCmdSetShadingRateImageEnableNV, vFpi)
+GO(vkCmdSetShadingRateImageEnableNV, vFpu)
 GO(vkCmdSetTessellationDomainOriginEXT, vFpi)
 GO(vkCmdSetViewportSwizzleNV, vFpuup)
-GO(vkCmdSetViewportWScalingEnableNV, vFpi)
+GO(vkCmdSetViewportWScalingEnableNV, vFpu)
 
 // VK_KHR_external_memory_win32
 GO(vkGetMemoryWin32HandleKHR, iFppp)
@@ -818,17 +827,17 @@ GO(vkGetDescriptorSetHostMappingVALVE, vFppp)
 GO(vkGetDescriptorSetLayoutHostMappingInfoVALVE, vFppp)
 
 // VK_EXT_descriptor_buffer
-GO(vkCmdBindDescriptorBufferEmbeddedSamplersEXT, vFppUu)
+GO(vkCmdBindDescriptorBufferEmbeddedSamplersEXT, vFpiUu)
 GO(vkCmdBindDescriptorBuffersEXT, vFpup)
 GO(vkCmdSetDescriptorBufferOffsetsEXT, vFppUuupp)
-GO(vkGetAccelerationStructureOpaqueCaptureDescriptorDataEXT, vFppp)
-GO(vkGetBufferOpaqueCaptureDescriptorDataEXT, vFppp)
+GO(vkGetAccelerationStructureOpaqueCaptureDescriptorDataEXT, iFppp)
+GO(vkGetBufferOpaqueCaptureDescriptorDataEXT, iFppp)
 GO(vkGetDescriptorEXT, vFppLp)
 GO(vkGetDescriptorSetLayoutBindingOffsetEXT, vFpUup)
 GO(vkGetDescriptorSetLayoutSizeEXT, vFpUp)
-GO(vkGetImageOpaqueCaptureDescriptorDataEXT, vFppp)
-GO(vkGetImageViewOpaqueCaptureDescriptorDataEXT, vFppp)
-GO(vkGetSamplerOpaqueCaptureDescriptorDataEXT, vFppp)
+GO(vkGetImageOpaqueCaptureDescriptorDataEXT, iFppp)
+GO(vkGetImageViewOpaqueCaptureDescriptorDataEXT, iFppp)
+GO(vkGetSamplerOpaqueCaptureDescriptorDataEXT, iFppp)
 
 // VK_KHR_cooperative_matrix
 GO(vkGetPhysicalDeviceCooperativeMatrixPropertiesKHR, iFppp)
@@ -849,7 +858,7 @@ GO(vkCmdDrawMeshTasksIndirectCountEXT, vFpUUUUuu)
 GO(vkCmdDrawMeshTasksIndirectEXT, vFpUUuu)
 
 // VK_EXT_opacity_micromap
-GO(vkBuildMicromapsEXT, iFpiup)
+GO(vkBuildMicromapsEXT, iFppup)
 GO(vkCmdBuildMicromapsEXT, vFpup)
 GO(vkCmdCopyMemoryToMicromapEXT, vFpp)
 GO(vkCmdCopyMicromapEXT, vFpp)
@@ -865,4 +874,112 @@ GO(vkGetMicromapBuildSizesEXT, vFpipp)
 GO(vkWriteMicromapsPropertiesEXT, iFpupiLpL)
 
 // VK_KHR_ray_tracing_maintenance1
-GO(vkCmdTraceRaysIndirect2KHR, vFpU)
\ No newline at end of file
+GO(vkCmdTraceRaysIndirect2KHR, vFpU)
+
+// VK_EXT_depth_bias_control
+GO(vkCmdSetDepthBias2EXT, vFpp)
+
+// VK_EXT_host_image_copy
+GO(vkCopyImageToImageEXT, iFpp)
+GO(vkCopyImageToMemoryEXT, iFpp)
+GO(vkCopyMemoryToImageEXT, iFpp)
+GO(vkGetImageSubresourceLayout2EXT, vFpUpp)
+GO(vkTransitionImageLayoutEXT, iFpup)
+
+// VK_KHR_map_memory2
+GO(vkMapMemory2KHR, iFppp)
+GO(vkUnmapMemory2KHR, iFpp)
+
+// VK_EXT_device_fault
+GO(vkGetDeviceFaultInfoEXT, iFppp)
+
+// VK_EXT_acquire_drm_display
+GO(vkAcquireDrmDisplayEXT, iFpiU)
+GO(vkGetDrmDisplayEXT, iFpiup)
+
+// VK_EXT_shader_object
+GO(vkCmdBindShadersEXT, vFpupp)
+GOM(vkCreateShadersEXT, iFEpuppp)
+GOM(vkDestroyShaderEXT, vFpUp)
+GO(vkGetShaderBinaryDataEXT, iFpUpp)
+
+// VK_NV_copy_memory_indirect
+GO(vkCmdCopyMemoryIndirectNV, vFpUuu)
+GO(vkCmdCopyMemoryToImageIndirectNV, vFpUuuUip)
+
+// VK_HUAWEI_cluster_culling_shader
+GO(vkCmdDrawClusterHUAWEI, vFpuuu)
+GO(vkCmdDrawClusterIndirectHUAWEI, vFpUU)
+
+// VK_EXT_attachment_feedback_loop_dynamic_state
+GO(vkCmdSetAttachmentFeedbackLoopEnableEXT, vFpu)
+
+// VK_NV_device_generated_commands_compute
+GO(vkCmdUpdatePipelineIndirectBufferNV, vFpiU)
+GO(vkGetPipelineIndirectDeviceAddressNV, UFpp)
+GO(vkGetPipelineIndirectMemoryRequirementsNV, vFppp)
+
+// VK_QCOM_tile_properties
+GO(vkGetDynamicRenderingTilePropertiesQCOM, iFppp)
+GO(vkGetFramebufferTilePropertiesQCOM, iFpUpp)
+
+// VK_NV_external_memory_capabilities
+GO(vkGetPhysicalDeviceExternalImageFormatPropertiesNV, iFpiiiuuup)
+
+// VK_EXT_pipeline_properties
+GO(vkGetPipelinePropertiesEXT, iFppp)
+
+// VK_EXT_direct_mode_display
+GO(vkReleaseDisplayEXT, iFpU)
+
+// VK_KHR_calibrated_timestamps
+GO(vkGetCalibratedTimestampsKHR, iFpuppp)
+GO(vkGetPhysicalDeviceCalibrateableTimeDomainsKHR, iFppp)
+
+// VK_KHR_video_encode_h264
+// no functions
+
+// VK_KHR_video_encode_h265
+// no functions
+
+// VK_KHR_video_maintenance1
+// no functions
+
+// VK_NV_cuda_kernel_launch
+GO(vkCmdCudaLaunchKernelNV, vFpp)
+GOM(vkCreateCudaFunctionNV, iFEpppp)
+GOM(vkCreateCudaModuleNV, iFEpppp)
+GOM(vkDestroyCudaFunctionNV, vFEpUp)
+GOM(vkDestroyCudaModuleNV, vFEpUp)
+GO(vkGetCudaModuleCacheNV, iFpUpp)
+
+// VK_NV_low_latency2
+GO(vkGetLatencyTimingsNV, vFpUp)
+GO(vkLatencySleepNV, iFpUp)
+GO(vkQueueNotifyOutOfBandNV, vFpp)
+GO(vkSetLatencyMarkerNV, vFpUp)
+GO(vkSetLatencySleepModeNV, iFpUp)
+
+// VK_AMDX_shader_enqueue
+GO(vkCmdDispatchGraphAMDX, vFpUp)
+GO(vkCmdDispatchGraphIndirectAMDX, vFpUp)
+GO(vkCmdDispatchGraphIndirectCountAMDX, vFpUU)
+GO(vkCmdInitializeGraphScratchMemoryAMDX, vFpU)
+GOM(vkCreateExecutionGraphPipelinesAMDX, iFEpUuppp)
+GO(vkGetExecutionGraphPipelineNodeIndexAMDX, iFpUpp)
+GO(vkGetExecutionGraphPipelineScratchSizeAMDX, iFpUp)
+
+// VK_KHR_maintenance6
+GO(vkCmdBindDescriptorSets2KHR, vFpp)
+GO(vkCmdPushConstants2KHR, vFpp)
+GO(vkCmdBindDescriptorBufferEmbeddedSamplers2EXT, vFpp)
+GO(vkCmdSetDescriptorBufferOffsets2EXT, vFpp)
+GO(vkCmdPushDescriptorSet2KHR, vFpp)
+GO(vkCmdPushDescriptorSetWithTemplate2KHR, vFpp)
+
+// VK_KHR_dynamic_rendering_local_read
+GO(vkCmdSetRenderingAttachmentLocationsKHR, vFpp)
+GO(vkCmdSetRenderingInputAttachmentIndicesKHR, vFpp)
+
+// VK_KHR_line_rasterization
+GO(vkCmdSetLineStippleKHR, vFpuW)
