@@ -2,13 +2,13 @@
 #error meh!
 #endif
 
-//GO(wl_array_add, 
+GO(wl_array_add, pFpL)
 //GO(wl_array_copy, 
-//GO(wl_array_init, 
-//GO(wl_array_release, 
-//DATA(wl_buffer_interface, 
+GO(wl_array_init, vFp)
+GO(wl_array_release, vFp)
+DATA(wl_buffer_interface, 40)
 //DATA(wl_callback_interface, 
-DATA(wl_compositor_interface, sizeof(void*)*3+sizeof(int)*4)
+DATA(wl_compositor_interface, 40)
 //DATA(wl_data_device_interface, 
 //DATA(wl_data_device_manager_interface, 
 //DATA(wl_data_offer_interface, 
@@ -26,25 +26,25 @@ GO(wl_display_flush, iFp)
 GO(wl_display_get_error, iFp)
 GO(wl_display_get_fd, iFp)
 //GO(wl_display_get_protocol_error, 
-DATA(wl_display_interface, sizeof(void*)*3+sizeof(int)*4)
+DATA(wl_display_interface, 40)
 GO(wl_display_prepare_read, iFp)
 GO(wl_display_prepare_read_queue, iFpp)
 GO(wl_display_read_events, iFp)
 GO(wl_display_roundtrip, iFp)
 GO(wl_display_roundtrip_queue, iFpp)
 GO(wl_event_queue_destroy, vFp)
-DATA(wl_keyboard_interface, sizeof(void*)*3+sizeof(int)*4)
+DATA(wl_keyboard_interface, 40)
 //GO(wl_list_empty, 
-//GO(wl_list_init, 
-//GO(wl_list_insert, 
+GO(wl_list_init, vFp)
+GO(wl_list_insert, vFpp)
 //GO(wl_list_insert_list, 
 //GO(wl_list_length, 
-//GO(wl_list_remove, 
+GO(wl_list_remove, vFp)
 //GO(wl_log_set_handler_client, 
-DATA(wl_output_interface, sizeof(void*)*3+sizeof(int)*4)
-DATA(wl_pointer_interface, sizeof(void*)*3+sizeof(int)*4)
+DATA(wl_output_interface, 40)
+DATA(wl_pointer_interface, 40)
 //GO(wl_proxy_add_dispatcher, 
-GO(wl_proxy_add_listener, iFppp)
+GOM(wl_proxy_add_listener, iFEppp)
 GO(wl_proxy_create, pFpp)
 GO(wl_proxy_create_wrapper, pFp)
 GO(wl_proxy_destroy, vFp)
@@ -61,7 +61,7 @@ GO(wl_proxy_marshal_array_constructor_versioned, pFpuppu)
 //GO(wl_proxy_marshal_array_flags, 
 GO(wl_proxy_marshal_constructor, pFpupppppppppppppppp)  //VAARGS
 GO(wl_proxy_marshal_constructor_versioned, pFpupupppppppppppppppp)  //VAARGS
-//GO(wl_proxy_marshal_flags, 
+GO(wl_proxy_marshal_flags, pFpupuupppppppppppppppppppp) //VAARGS
 GO(wl_proxy_set_queue, vFpp)
 //GO(wl_proxy_set_tag, 
 GO(wl_proxy_set_user_data, vFpp)
