@@ -15,6 +15,7 @@
 #include "box64context.h"
 #include "librarian.h"
 #include "callback.h"
+#include "globalsymbols.h"
 
 const char *xsltName =
 #ifdef ANDROID
@@ -153,10 +154,9 @@ EXPORT void my_xsltSetCtxtSortFunc(x64emu_t* emu, void* ctx, void* handler)
     return my->xsltSetCtxtSortFunc(ctx, find_xsltSortFunc_Fct(handler));
 }
 
-#define CUSTOM_INIT \
-    getMy(lib);
-
-#define CUSTOM_FINI \
-    freeMy();
+EXPORT void my_xsltInit(x64emu_t* emu) {
+    my->xsltInit();
+    my_checkGlobalXslt();
+}
 
 #include "wrappedlib_init.h"
